/*****************************************************************************
 *  rsfpl_noise_figure.c
 *
 *  Rohde & Schwarz FPL spectrum analyzer Instrument Driver
 *  Original Release: May 2018
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *
 *  Copyright (c) 2018, Rohde & Schwarz GmbH & Co. KG
 *  All rights reserved.
 *****************************************************************************/

#include "rsfpl.h"
#include "rsfpl_internal.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif
#include <string.h>
#if defined (_CVI_)
#include <utility.h>
#define _strnicmp strnicmp
#endif
extern ViString gsc_read_y_trace_arr[];

/// HIFN  This function selects the Noise Figure mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_SetNoiseMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteWithOpc(instrSession, "INST:SEL NOIS", 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the use of a common ENR on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR commonSourceEnabled/Turns the use of a common ENR on or off.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCommonSourceEnabled(ViSession instrSession,
                                                          ViBoolean commonSourceEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_NOISE_COMMON_NOISE_SOURCE_ENABLED, commonSourceEnabled),
            2, "Common Source Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the room temperature of the measurement environment. The
/// HIFN  temperature is taken into account when calculating noise results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseRoomTemperature/Defines the room temperature of the measurement environment. The
/// HIPAR noiseRoomTemperature/temperature is taken into account when calculating noise results.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseRoomTemperature(ViSession instrSession,
                                                      ViReal64  noiseRoomTemperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_ROOM_TEMPERATURE, noiseRoomTemperature),
            2, "Noise Room Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of noise source you are using for the calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationNoiseSource/Selects the type of noise source you are using for the calibration.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationNoiseSource(ViSession instrSession,
                                                             ViInt32   calibrationNoiseSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_NOISE_SOURCE, calibrationNoiseSource),
            2, "Calibration Noise Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the ENR mode for the calibration.
/// HIFN     Is available when you use different noise sources for calibration and
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseCalibrationMode/Selects the ENR mode for the calibration.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationMode(ViSession instrSession,
                                                      ViInt32   noiseCalibrationMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_MODE, noiseCalibrationMode),
            2, "Noise Calibration Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constant ENR for all measurement points during
/// HIFN  calibration.
/// HIFN     Is available when you use different noise sources for calibration and
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constantENR/Defines the constant ENR for all measurement points during
/// HIPAR constantENR/calibration.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationConstantENR(ViSession instrSession,
                                                             ViReal64  constantENR)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_CONSTANT_ENR, constantENR),
            2, "Constant ENR");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches ENR mode for the calibration to Spot and defines a constant
/// HIFN  temperatures of a resistor used during calibration.
/// HIFN     The function is available when you have selected a noise source with
/// HIFN  resistor characteristics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coldTemperature/Defines a constant temperature of a resistor not supplied with power
/// HIPAR coldTemperature/(Tcold) used during calibration.
/// HIPAR hotTemperature/Defines a constant temperature of a resistor supplied with power
/// HIPAR hotTemperature/(Thot) used during calibration.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationSpot(ViSession instrSession,
                                                      ViReal64  coldTemperature,
                                                      ViReal64  hotTemperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_MODE, RSFPL_VAL_NOISE_SOURCE_MODE_SPOT));
    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_SPOT_COLD_TEMPERATURE, coldTemperature),
            3, "Cold Temperature");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_SPOT_HOT_TEMPERATURE, hotTemperature),
            4, "Hot Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and queries the calibration noise source smart noise source
/// HIFN  serial number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialNumber/Sets and queries the calibration noise source smart noise source
/// HIPAR serialNumber/serial number.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationSmartSerialNumber(
                            ViSession instrSession,
                            ViString  serialNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_SMART_SERIAL_NUMBER, serialNumber),
            2, "Serial Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an ENR or temperature table for calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Selects an ENR or temperature table for calibration.
ViStatus _VI_FUNC rsfpl_SelectCalibrationTable(ViSession instrSession,
                                               ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_SELECTED_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of noise source you are using for the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementNoiseSource/Selects the type of noise source you are using for the measurement.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementNoiseSource(ViSession instrSession,
                                                             ViInt32   measurementNoiseSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_NOISE_SOURCE, measurementNoiseSource),
            2, "Measurement Noise Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the ENR mode for the actual measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the ENR mode for the actual measurement.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseSourceMeasurementMode(ViSession instrSession,
                                                            ViInt32   measurementMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constant ENR for all measurement points during the actual
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constantENR/Defines the constant ENR for all measurement points during the actual
/// HIPAR constantENR/measurement.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementConstantENR(ViSession instrSession,
                                                             ViReal64  constantENR)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_CONSTANT_ENR, constantENR),
            2, "Constant ENR");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a constant temperature of a resistor used during measurements.
/// HIFN     The attribute is available when you have selected a noise source with
/// HIFN  resistor characteristics with.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coldTemperature/Defines a constant temperature of a resistor not supplied with power
/// HIPAR coldTemperature/(Tcold) used during measurements.
/// HIPAR hotTemperature/Defines a constant temperature of a resistor supplied with power
/// HIPAR hotTemperature/(Thot) used during measurements.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementSpot(ViSession instrSession,
                                                      ViReal64  coldTemperature,
                                                      ViReal64  hotTemperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_SOURCE_MEASUREMENT_MODE, RSFPL_VAL_NOISE_SOURCE_MODE_SPOT));
    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_SPOT_COLD_TEMPERATURE, coldTemperature),
            3, "Cold Temperature");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_SPOT_HOT_TEMPERATURE, hotTemperature),
            4, "Hot Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and queries the calibration noise source smart noise source
/// HIFN  serial number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serialNumber/Sets and queries the calibration noise source smart noise source
/// HIPAR serialNumber/serial number.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementSmartSerialNumber(
                            ViSession instrSession,
                            ViString  serialNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_SMART_SERIAL_NUMBER, serialNumber),
            2, "Serial Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an ENR or temperature table for the actual measurement.
/// HIFN     When you want to edit a table, regardless if you want to use it later
/// HIFN  for a measurement or for calibration, you have to use this attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Selects an ENR or temperature table for the actual measurement.
ViStatus _VI_FUNC rsfpl_SelectNoiseMeasurementTable(ViSession instrSession,
                                                    ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_SELECTED_TABLE, tableName),
            3, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the contents of the currently selected ENR table. The table
/// HIFN  can contain up to 500 entries.
/// HIFN     The ENR table should contain an ENR for all measurement points.
/// HIFN     If you create a new table with this method, it will overwrite the
/// HIFN  current entries of the frequency list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of arrays in Frequencies and ENRs.
/// HIPAR frequencies/Frequency of the measurement point.
/// HIPAR enrs/ENR of the measurement point.
ViStatus _VI_FUNC rsfpl_DefineNoiseMeasurementDiodeTable(ViSession instrSession,
                                                         ViUInt32  arraySize,
                                                         ViReal64  frequencies[],
                                                         ViReal64  enrs[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CORR:ENR:TABL ", arraySize, frequencies, enrs, NULL, NULL, NULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all ENR tables available in the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array Table List
/// HIPAR tableList/Queries all ENR tables available in the application.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsfpl_QueryNoiseMeasurementDiodeTableList(ViSession instrSession,
                                                            ViUInt32  arraySize,
                                                            ViChar    tableList[],
                                                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "CORR:ENR:TABL:LIST?", arraySize, tableList, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an ENR diode table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Deletes an ENR diode table.
ViStatus _VI_FUNC rsfpl_NoiseMeasurementDiodeTableDelete(ViSession instrSession,
                                                         ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_DIODE_TABLE_DELETE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the contents of the currently selected temperature table. The
/// HIFN  table can contain up to 500 entries.
/// HIFN     The temperature table should contain an two temperature values for all
/// HIFN  measurement points.
/// HIFN     If you create a new table with this method, it will overwrite the
/// HIFN  current entries of the frequency list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of arrays in Frequencies, Tcold and
/// HIPAR arraySize/Thot.
/// HIPAR frequencies/Frequency of the measurement point.
/// HIPAR thot/Temperature of the noise source when it is off for the corresponding
/// HIPAR thot/measurement point.
/// HIPAR tcold/Temperature of the noise source when it is on for the corresponding
/// HIPAR tcold/measurement point.
ViStatus _VI_FUNC rsfpl_DefineNoiseMeasurementTemperatureTable(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  frequencies[],
                            ViReal64  thot[],
                            ViReal64  tcold[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CORR:ENR:TABL:TEMP ", arraySize, frequencies, thot, tcold, NULL, NULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all temperature tables available in the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array Table List
/// HIPAR tableList/Queries all temperature tables available in the application.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsfpl_QueryNoiseMeasurementTemperatureTableList(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    tableList[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_QueryViString(instrSession, "CORR:ENR:TABL:TEMP:LIST?", arraySize, tableList, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a temperature table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Deletes a temperature table.
ViStatus _VI_FUNC rsfpl_NoiseMeasurementTemperatureTableDelete(
                            ViSession instrSession,
                            ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_TEMPERATURE_TABLE_DELETE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the input loss mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the input loss mode.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseInputLossMode(ViSession instrSession,
                                                    ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_INPUT_LOSS_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a constant input loss for all measurement points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constantLoss/Defines a constant input loss for all measurement points.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseInputLossConstant(ViSession instrSession,
                                                        ViReal64  constantLoss)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_INPUT_LOSS_CONSTANT, constantLoss),
            3, "Constant Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The specified temperature in Kelvin at the time of measurement is
/// HIFN  considered in the loss calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputCorrectionTemperature/The specified temperature in Kelvin at the time of measurement is
/// HIPAR inputCorrectionTemperature/considered in the loss calculation.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseInputLossTemperature(ViSession instrSession,
                                                           ViReal64  inputCorrectionTemperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_INPUT_LOSS_TEMPERATURE, inputCorrectionTemperature),
            2, "Input Correction Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an input loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Selects an input loss table.
ViStatus _VI_FUNC rsfpl_SelectNoiseInputLossTable(ViSession instrSession,
                                                  ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_INPUT_LOSS_SELECTED_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the contents of the currently selected input loss table. The
/// HIFN  table can contain up to 500 entries.
/// HIFN     The table should contain an input loss for all measurement points.
/// HIFN     If you create a new table with this method, it will overwrite the
/// HIFN  current entries of the loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of arrays in Frequencies and Loss.
/// HIPAR frequencies/Frequency of the measurement point.
/// HIPAR loss/Loss of the measurement point.
ViStatus _VI_FUNC rsfpl_DefineNoiseInputLossTable(ViSession instrSession,
                                                  ViUInt32  arraySize,
                                                  ViReal64  frequencies[],
                                                  ViReal64  loss[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CORR:LOSS:INP:TABL ", arraySize, frequencies, loss, NULL, NULL, NULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an input loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Deletes an input loss table.
ViStatus _VI_FUNC rsfpl_DeleteNoiseInputLossTable(ViSession instrSession,
                                                  ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_INPUT_LOSS_DELETE_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all input loss tables available in the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer in parameter 'Input Loss Table List'.
/// HIPAR inputLossTableList/Queries all input loss tables available in the application. Returns a
/// HIPAR inputLossTableList/comma separated list of table names.
/// HIPAR listLength/Length in bytes of the list returned by the instrument. If List Length
/// HIPAR listLength/is greater than the allocated Buffer Size, some data from the
/// HIPAR listLength/instrument was lost. Resize the buffer to be >= List Length and query
/// HIPAR listLength/again for the complete list.
ViStatus _VI_FUNC rsfpl_QueryNoiseInputLossTableList(ViSession instrSession,
                                                     ViUInt32  bufferSize,
                                                     ViChar    inputLossTableList[],
                                                     ViInt32*  listLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:LOSS:INP:TABL:LIST?", (ViInt32)bufferSize, inputLossTableList, listLength));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the output loss mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the input loss mode.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseOutputLossMode(ViSession instrSession,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_OUTPUT_LOSS_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a constant output loss for all measurement points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constantLoss/Defines a constant output loss for all measurement points.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseOutputLossConstant(ViSession instrSession,
                                                         ViReal64  constantLoss)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_OUTPUT_LOSS_CONSTANT, constantLoss),
            2, "Constant Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The specified temperature in Kelvin at the time of measurement is
/// HIFN  considered in the loss calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputCorrectionTemperature/The specified temperature in Kelvin at the time of measurement is
/// HIPAR outputCorrectionTemperature/considered in the loss calculation.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseOutputLossTemperature(ViSession instrSession,
                                                            ViReal64  outputCorrectionTemperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_OUTPUT_LOSS_TEMPERATURE, outputCorrectionTemperature),
            2, "Output Correction Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an output loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Selects an output loss table.
ViStatus _VI_FUNC rsfpl_SelectNoiseOutputLossTable(ViSession instrSession,
                                                   ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_OUTPUT_LOSS_SELECTED_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the contents of the currently selected output loss table. The
/// HIFN  table can contain up to 500 entries.
/// HIFN     The table should contain an output loss for all measurement points.
/// HIFN     If you create a new table with this method, it will overwrite the
/// HIFN  current entries of the frequency list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of arrays in Frequencies and Loss.
/// HIPAR frequencies/Frequency of the measurement point.
/// HIPAR loss/Loss of the measurement point.
ViStatus _VI_FUNC rsfpl_DefineNoiseOutputLossTable(ViSession instrSession,
                                                   ViUInt32  arraySize,
                                                   ViReal64  frequencies[],
                                                   ViReal64  loss[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CORR:LOSS:OUTP:TABL ", arraySize, frequencies, loss, NULL, NULL, NULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an output loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Deletes an output loss table.
ViStatus _VI_FUNC rsfpl_DeleteNoiseOutputLossTable(ViSession instrSession,
                                                   ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_OUTPUT_LOSS_DELETE_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all output loss tables available in the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer in parameter 'Output Loss Table List'.
/// HIPAR outputLossTableList/Queries all output loss tables available in the application. Returns a
/// HIPAR outputLossTableList/comma separated list of table names.
/// HIPAR listLength/Length in bytes of the list returned by the instrument. If List Length
/// HIPAR listLength/is greater than the allocated Buffer Size, some data from the
/// HIPAR listLength/instrument was lost. Resize the buffer to be >= List Length and query
/// HIPAR listLength/again for the complete list.
ViStatus _VI_FUNC rsfpl_QueryNoiseOutputLossTableList(ViSession instrSession,
                                                      ViUInt32  bufferSize,
                                                      ViChar    outputLossTableList[],
                                                      ViInt32*  listLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:LOSS:OUTP:TABL:LIST?", (ViInt32)bufferSize, outputLossTableList, listLength));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the input loss mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the input loss mode.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationLossMode(ViSession instrSession,
                                                          ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_LOSS_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a constant calibration loss for all measurement points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constantLoss/Defines a constant calibration loss for all measurement points.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationLossConstant(
                            ViSession instrSession,
                            ViReal64  constantLoss)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_LOSS_CONSTANT, constantLoss),
            2, "Constant Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The specified temperature in Kelvin at the time of measurement is
/// HIFN  considered in the loss calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctionTemperature/The specified temperature in Kelvin at the time of measurement is
/// HIPAR correctionTemperature/considered in the loss calculation.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationLossTemperature(
                            ViSession instrSession,
                            ViReal64  correctionTemperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_LOSS_TEMPERATURE, correctionTemperature),
            2, "Correction Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a calibration loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tableName/Selects a calibration loss table.
ViStatus _VI_FUNC rsfpl_SelectNoiseCalibrationLossTable(ViSession instrSession,
                                                        ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_LOSS_SELECTED_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the contents of the currently selected calibration loss table.
/// HIFN  Each entry of the loss table consists of one measurement point and the
/// HIFN  corresponding loss. The table can contain up to 1201 entries.
/// HIFN     If you create a new table with this method, it overwrites the current
/// HIFN  entries of the loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of arrays in Frequencies and Loss.
/// HIPAR frequencies/Frequency of the measurement point.
/// HIPAR loss/Loss of the measurement point.
ViStatus _VI_FUNC rsfpl_DefineNoiseCalibrationLossTable(ViSession instrSession,
                                                        ViUInt32  arraySize,
                                                        ViReal64  frequencies[],
                                                        ViReal64  loss[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CORR:LOSS:CAL:TABL ", arraySize, frequencies, loss, NULL, NULL, NULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a calibration loss table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tablename/Table name.
ViStatus _VI_FUNC rsfpl_DeleteNoiseCalibrationLossTable(ViSession instrSession,
                                                        ViString  tableName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_CALIBRATION_LOSS_DELETE_TABLE, tableName),
            2, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all calibration loss tables available in the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer in parameter 'Calibration Loss Table List'.
/// HIPAR calibrationLossTableList/Queries all calibration loss tables available in the application.
/// HIPAR listLength/Length in bytes of the list returned by the instrument. If List Length
/// HIPAR listLength/is greater than the allocated Buffer Size, some data from the
/// HIPAR listLength/instrument was lost. Resize the buffer to be >= List Length and query
/// HIPAR listLength/again for the complete list.
ViStatus _VI_FUNC rsfpl_QueryNoiseCalibrationLossTableList(ViSession instrSession,
                                                           ViUInt32  bufferSize,
                                                           ViChar    calibrationLossTableList[],
                                                           ViInt32*  listLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:LOSS:CAL:TABL:LIST?", (ViInt32)bufferSize, calibrationLossTableList, listLength));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of DUT you are testing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseDUTType/Selects the type of DUT you are testing.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseDUTType(ViSession instrSession,
                                              ViInt32   noiseDUTType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_DUT_TYPE, noiseDUTType),
            3, "Noise DUT Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of local oscillator you are using. The attribute is
/// HIFN  available for measurements on frequency converting DUTs
/// HIFN  RSFPL_ATTR_NOISE_DUT_TYPE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseDUTLOType/Selects the type of local oscillator you are using. The control is
/// HIPAR noiseDUTLOType/available for measurements on frequency converting DUTs
/// HIPAR noiseDUTLOType/RSFPL_ATTR_NOISE_DUT_TYPE.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseDUTLOType(ViSession instrSession,
                                                ViInt32   noiseDUTLOType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_DUT_LO_TYPE, noiseDUTLOType),
            2, "Noise DUT LO Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the mixer parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR loFrequency/Defines the frequency for DUTs with a fixed LO.
/// HIPAR imageRejection/Defines the image frequency rejection for the DUT.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseDUTMixer(ViSession instrSession,
                                               ViReal64  loFrequency,
                                               ViReal64  imageRejection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_DUT_LO_FREQUENCY, loFrequency),
            2, "LO Frequency");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_DUT_IMAGE_REJECTION, imageRejection),
            3, "Image Rejection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of measurement points analyzed during a sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementPoints/Defines the number of measurement points analyzed during a sweep.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementPoints(ViSession instrSession,
                                                        ViInt32   measurementPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_POINTS, measurementPoints),
            2, "Measurement Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency for single frequency measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR singleFrequency/Defines the frequency for single frequency measurements.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequency(ViSession instrSession,
                                                      ViReal64  singleFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_SINGLE_FREQUENCY, singleFrequency),
            2, "Single Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Couples or decouples the frequency to the contents of the sweep list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyCouplingEnabled/Couples or decouples the frequency to the contents of the sweep list.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequencyCouplingEnabled(
                            ViSession instrSession,
                            ViBoolean frequencyCouplingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_NOISE_SINGLE_FREQUENCY_COUPLING_ENABLED, frequencyCouplingEnabled),
            2, "Frequency Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the software to perform a single frequency measurement in
/// HIFN  continuous sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequencyContinuousSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_SINGLE_FREQUENCY_CONTINUOUS_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the software to perform a single frequency measurement in
/// HIFN  single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequencySingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_SINGLE_FREQUENCY_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency stepsize in the frequency table. The stepsize
/// HIFN  corresponds to the distance from one measurement point to another. If
/// HIFN  you change the stepsize, the application creates a new frequency list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stepsize/Defines the frequency stepsize in the frequency table. The stepsize
/// HIPAR stepsize/corresponds to the distance from one measurement point to another. If
/// HIPAR stepsize/you change the stepsize, the application creates a new frequency list.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFrequencyStepsize(ViSession instrSession,
                                                        ViReal64  stepsize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FREQUENCY_STEPSIZE, stepsize),
            2, "Stepsize");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the contents of the frequency table. It overwrites the current
/// HIFN  contents of the frequency table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of arrays in Data.
/// HIPAR data/Defines a frequency for each entry in the frequency table. A frequency
/// HIPAR data/table can contain up 1201 entries.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFrequencyTableData(ViSession instrSession,
                                                         ViUInt32  arraySize,
                                                         ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "FREQ:TABL:DATA ", arraySize, data, NULL, NULL, NULL, NULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the software to perform a frequency list measurement in
/// HIFN  continuous sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseListMeasurementContinuousSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_LIST_MEASUREMENT_CONTINUOUS_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the software to perform a frequency list measurement in
/// HIFN  single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseListMeasurementSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_LIST_MEASUREMENT_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RF, LO and IF frequency for every measurement point in the
/// HIFN  frequency table. In Fixed IF mode, the LO frequency is different for
/// HIFN  every measurement point. In Fixed LO mode, the IF frequency is
/// HIFN  different for every measurement point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the buffers allocated for the results arrays.
/// HIPAR rfFrequency/Values from the RF Frequency column of the table.
/// HIPAR loFrequency/Values from the LO Frequency column of the table.
/// HIPAR ifFrequency/Values from the IF Frequency column of the table.
/// HIPAR returnedValues/The number of returned values for each result array that was read from
/// HIPAR returnedValues/the instrument. If this value is larger than the allocated buffer size,
/// HIPAR returnedValues/not all values returned by the instrument could be stored in the
/// HIPAR returnedValues/arrays.
ViStatus _VI_FUNC rsfpl_QueryNoiseFrequencyTableData(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViReal64  rfFrequency[],
                                                     ViReal64  loFrequency[],
                                                     ViReal64  ifFrequency[],
                                                     ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   response = NULL;
    ViPReal64 outDblArray = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1), 2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, rfFrequency), 3, "RF Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, loFrequency), 4, "LO Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ifFrequency), 5, "IF Frequency");

    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
        RSFPL_ATTR_NOISE_FREQUENCY_QUERY_TABLE_DATA_RAW,
        RS_VAL_DIRECT_USER_CALL, &response));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &outDblArray, &outArraySize));
    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(outDblArray, outArraySize,
        bufferSize, returnedValues,
        rfFrequency, loFrequency, ifFrequency,
        NULL, NULL, NULL));

Error:
    if (response) free(response);
    if (outDblArray) free(outDblArray);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the software to perform calibration measurements.
/// HIFN     Using RSFPL_ATTR_INITIATE then initiates a calibration instead of the
/// HIFN  actual measurement, until you deliberately select one of the normal
/// HIFN  measurements again.
/// HIFN     Note that calibration data is used only when the second stage
/// HIFN  correction mode has been turned on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR secondStageCorrectionEnabled/Enables or disables 2nd stage correction. If enabled, the application
/// HIPAR secondStageCorrectionEnabled/uses the calibration data to compensate for the inherent noise of the
/// HIPAR secondStageCorrectionEnabled/analyzer when calculating the measurement results.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerSecondStageCorrection(
                            ViSession instrSession,
                            ViBoolean secondStageCorrectionEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_NOISE_ANALYZER_CALIBRATION_DATA_INCLUDED, secondStageCorrectionEnabled),
            2, "Second Stage Correction Enabled");
    if (secondStageCorrectionEnabled)
    {
        checkErr(rsfpl_SetAttributeViString(instrSession, NULL, RSFPL_ATTR_NOISE_ANALYZER_CALIBRATION_MEASUREMENT, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the settling time of the noise source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settlingTime/Defines the settling time of the noise source.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerSettlingTime(ViSession instrSession,
                                                           ViReal64  settlingTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_ANALYZER_SETTLING_TIME, settlingTime),
            2, "Settling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic determination of the reference level on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoReferenceLevelEnabled/Turns automatic determination of the reference level on and off.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerAutoReferenceLevelEnabled(
                            ViSession instrSession,
                            ViBoolean autoReferenceLevelEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_NOISE_ANALYZER_AUTO_REFERENCE_LEVEL_ENABLED, autoReferenceLevelEnabled),
            2, "Auto Reference Level Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the expected gain of the DUT.
/// HIFN     The application uses the gain for automatic reference level detection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR expectedDUTGain/Defines the expected gain of the DUT.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerExpectedDUTGain(
                            ViSession instrSession,
                            ViReal64  expectedDUTGain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_ANALYZER_EXPECTED_DUT_GAIN, expectedDUTGain),
            2, "Expected DUT Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a limit line for a particular result type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit line.
/// HIPAR resultType/Configures a limit line for a particular result type.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseLimitLineResultType(ViSession instrSession,
                                                          ViInt32   limit,
                                                          ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_LIMIT_LINE_RESULT_TYPE,
            "LimitLine", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_NOISE_LIMIT_LINE_RESULT_TYPE, resultType),
            3, "Result Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the items displayed in the Result Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR items/Selects the item.
/// HIPAR enabled/Enabled/Disabled
ViStatus _VI_FUNC rsfpl_ConfigureNoiseDisplayItems(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   items,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2noise;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngNoiseResultType, items, &p2noise),
                3, "items");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:TABL:ITEM %s,%s", p2noise, enabled ? "ON" : "OFF");
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TABL:ITEM %s,%s", window, p2noise, enabled ? "ON" : "OFF");
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic scaling of the y-axis on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR automaticScalingEnabled/Turns automatic scaling of the y-axis on and off.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseDisplayAutomaticScalingEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean automaticScalingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_DISPLAY_AUTOMATIC_SCALING_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_NOISE_DISPLAY_AUTOMATIC_SCALING_ENABLED, automaticScalingEnabled),
            3, "Automatic Scaling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR frequencyType/Selects the type of frequency displayed on the x-axis.
/// HIPAR symbolsEnabled/Turns symbols that represent the measurement points on a trace on and
/// HIPAR symbolsEnabled/off.
/// HIPAR bottom/Defines the bottom value of the y-axis.
/// HIPAR top/Defines the top value of the y-axis.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseDisplayXAxis(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   frequencyType,
                                                   ViBoolean symbolsEnabled,
                                                   ViReal64  bottom,
                                                   ViReal64  top)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSFPL_ATTR_NOISE_DISPLAY_FREQUENCY_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, repCap, RSFPL_ATTR_NOISE_DISPLAY_FREQUENCY_TYPE, frequencyType),
            3, "Frequency Type");

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, repCap, RSFPL_ATTR_NOISE_DISPLAY_X_AXIS_SYMBOLS_ENABLED, symbolsEnabled),
            4, "Symbols Enabled");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_NOISE_DISPLAY_X_AXIS_BOTTOM, bottom),
            5, "Botom");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, repCap, RSFPL_ATTR_NOISE_DISPLAY_X_AXIS_TOP, top),
            6, "Top");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a window to the display in the active channel setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR direction/Direction the new window is added relative to the existing window.
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
/// HIPAR arraySize/Defines the memory size allocated for Returned Window Name.
/// HIPAR returnedWindowName/String containing the name of the existing window the new window is
/// HIPAR returnedWindowName/inserted next to.
ViStatus _VI_FUNC rsfpl_NoiseLayoutAddWindow(ViSession instrSession,
                                             ViString  windowName,
                                             ViInt32   direction,
                                             ViInt32   windowType,
                                             ViUInt32  arraySize,
                                             ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngLayoutDirection, direction, &p2direction),
                3, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngNoiseLayoutWindowType, windowType, &p2type),
                4, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", windowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    checkErr(rsfpl_CheckStatus(instrSession));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        strncpy(returnedWindowName, response, arraySize);
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Replaces the window type (for example from 'Noise' to 'Gain') of an
/// HIFN  already existing window in the active channel setup while keeping its
/// HIFN  position, index and window name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to.
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsfpl_NoiseLayoutReplaceWindow(ViSession instrSession,
                                                 ViString  windowName,
                                                 ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngNoiseLayoutWindowType, windowType, &p2type),
                3, "Window Type");

    (void)snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(rsfpl_WriteCommandWithOPCSync(instrSession, cmd, 0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the measurement mode for the hot and cold power measurements.
/// HIFN     Note that selecting a noise source with resistor characteristics
/// HIFN  automatically selects manual measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementMode/Selects the measurement mode for the hot and cold power measurements.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementMode(ViSession instrSession,
                                                      ViInt32   measurementMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of power measurement to perform next.
/// HIFN     The attribute is available for manual measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR typeOfPowerMeasurement/Selects the type of power measurement to perform next.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementTypeOfPowerMeasurement(
                            ViSession instrSession,
                            ViInt32   typeOfPowerMeasurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViInt32(instrSession, NULL, RSFPL_ATTR_NOISE_MEASUREMENT_TYPE_OF_POWER_MEASUREMENT, typeOfPowerMeasurement),
            2, "Type Of Power Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the noise figure measurement results.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe<n>[:DATA]?
/// HIFN     FORMat[:DATA] REAL,32
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR result/Selects the result display to be read out.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR result/For any graphical result display, the function returns one result for
/// HIPAR result/each measurement point.
ViStatus _VI_FUNC rsfpl_FetchNoiseTraceData(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViInt32   resultType,
                                            ViUInt32  arrayLength,
                                            ViUInt32* actualPoints,
                                            ViReal64  result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, RSFPL_VAL_TRACE_RESULT_TRACE1, RSFPL_VAL_TRACE_RESULT_TRACE4),
            3, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsFpl_rngNoiseResultType, resultType, &p2type),
                4, "Result Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC%ld? %s,%s", window, gsc_read_y_trace_arr[trace - 1], p2type);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, result, (ViInt32*)actualPoints));

    checkErr(rsfpl_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measurement result at the marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects the marker.
/// HIPAR resultType/Selects the result display to be read out.
/// HIPAR result/Returns the specified result at the marker position.
ViStatus _VI_FUNC rsfpl_QueryNoiseMarkerResults(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   resultType,
                                                ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  markerMax = 16;
    ViChar*  p2rngString;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &markerMax));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, markerMax),
            2, "Marker");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &RsFpl_rngNoiseResultType,
                    resultType, &p2rngString),
                3, "Result Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:Y? %s", marker, p2rngString);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measurement result at the delta marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deltaMarker/Selects the delta marker.
/// HIPAR resultType/Selects the result display to be read out.
/// HIPAR result/Returns the specified result at the delta marker position.
ViStatus _VI_FUNC rsfpl_QueryNoiseDeltaMarkerResults(ViSession instrSession,
                                                     ViInt32   deltaMarker,
                                                     ViInt32   resultType,
                                                     ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  markerMax = 16;
    ViChar*  p2rngString;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &markerMax));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, markerMax),
            2, "Delta Marker");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession,
                    &RsFpl_rngNoiseResultType,
                    resultType, &p2rngString),
                3, "Result Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DELT%ld:Y? %s", deltaMarker, p2rngString);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  Queries the RF, RBW and sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the buffers allocated for the results arrays.
/// HIPAR frequency/Returns the frequency.
/// HIPAR bandwidth/Returns the bandwidth.
/// HIPAR sweepTime/Returns the sweep time.
/// HIPAR returnedValues/The number of returned values for each result array that was read from
/// HIPAR returnedValues/the instrument. If this value is larger than the allocated buffer size,
/// HIPAR returnedValues/not all values returned by the instrument could be stored in the
/// HIPAR returnedValues/arrays.
ViStatus _VI_FUNC rsfpl_QueryNoiseData(ViSession instrSession,
                                       ViInt32   bufferSize,
                                       ViReal64  frequency[],
                                       ViReal64  bandwidth[],
                                       ViReal64  sweepTime[],
                                       ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   response = NULL;
    ViPReal64 outDblArray = NULL;
    ViInt32   outArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1), 2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 3, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bandwidth), 4, "Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, sweepTime), 5, "Sweep Time");

    checkErr(RsCore_GetAttributeViStringUnknownLen(instrSession, NULL,
        RSFPL_ATTR_HIDDEN_QUERY_NOISE_DATA,
        RS_VAL_DIRECT_USER_CALL, &response));
    checkErr(RsCore_Convert_String_To_ViReal64Array(instrSession, response, &outDblArray, &outArraySize));
    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(outDblArray, outArraySize,
        bufferSize, returnedValues,
        frequency, bandwidth, sweepTime,
        NULL, NULL, NULL));

Error:
    if (response) free(response);
    if (outDblArray) free(outDblArray);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command configures settings for the noise source output.
/// HIFN     If the noise sources during calibration and measurement are different,
/// HIFN  the command configures settings for the measurement noise source. If a
/// HIFN  smart noise source is used, the  values defined in the SNS table are
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputMatch/You can define the output match either as the VSWR or as the return
/// HIPAR outputMatch/loss (RL). Refer to the datasheet of the noise source for these values.
/// HIPAR outputMatchValue/Defines the VSWR or return loss at the noise source output.
/// HIPAR enrUncertainty/Defines the uncertainty of a noise source.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintySource(
                            ViSession instrSession,
                            ViInt32   outputMatch,
                            ViReal64  outputMatchValue,
                            ViReal64  enrUncertainty)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (outputMatch)
    {
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_VSWR:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_OUTPUT_MATCH_VSWR;
        break;
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_RL:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_OUTPUT_MATCH_RETURN_LOSS;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, outputMatch),
                    2, "Output Match");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, attr, outputMatchValue),
            3, "Output Match Value");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_ENR_UNCERTAINTY, enrUncertainty),
            4, "ENR Uncertainty");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command defines the hot and cold temperature uncertainty of a
/// HIFN  resistor noise source. Refer to the data sheet of the noise source to
/// HIFN  determine its uncertainty.
/// HIFN     If the noise sources during calibration and measurement are different,
/// HIFN  the command defines the uncertainty of the measurement noise source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hotUncertainty/This command defines the hot temperature uncertainty of a resistor
/// HIPAR hotUncertainty/noise source. Refer to the data sheet of the noise source to determine
/// HIPAR hotUncertainty/its uncertainty.
/// HIPAR coldUncertainty/This command defines the cold temperature uncertainty of a resistor
/// HIPAR coldUncertainty/noise source. Refer to the data sheet of the noise source to determine
/// HIPAR coldUncertainty/its uncertainty.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyResistorTempUncertainty(
                            ViSession instrSession,
                            ViReal64  hotUncertainty,
                            ViReal64  coldUncertainty)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_RESISTOR_HOT_UNCERTAINTY, hotUncertainty),
            2, "Hot Uncertainty");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_RESISTOR_COLD_UNCERTAINTY, coldUncertainty),
            3, "Cold Uncertainty");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command configures settings for the calibration noise source
/// HIFN  output.
/// HIFN     This command is available when
/// HIFN  RSFPL_ATTR_NOISE_COMMON_NOISE_SOURCE_ENABLED is off (see
/// HIFN  rsfpl_ConfigureNoiseCommonSourceEnabled). If the Common Source
/// HIFN  parameter is set to on (VI_TRUE), further parameters are ignored. If a
/// HIFN  smart noise source is used, the values defined in the SNS table are
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR commonSource/This command turns matching of the noise source characteristics used
/// HIPAR commonSource/during calibration and measurement on and off.
/// HIPAR outputMatch/You can define the output match either as the VSWR or as the return
/// HIPAR outputMatch/loss (RL). Refer to the datasheet of the noise source for these values.
/// HIPAR outputMatchValue/Defines the VSWR or return loss at the calibration noise source
/// HIPAR outputMatchValue/output.
/// HIPAR enrUncertainty/This command defines the uncertainty of a calibration noise source.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyCalibrationSource(
                            ViSession instrSession,
                            ViBoolean commonSource,
                            ViInt32   outputMatch,
                            ViReal64  outputMatchValue,
                            ViReal64  enrUncertainty)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_COMMON_SOURCE_FOR_MEAS_AND_CAL_ENABLED, commonSource),
            2, "Common Source");

    if (!commonSource)
    {
        switch (outputMatch)
        {
        case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_VSWR:
            attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_OUTPUT_MATCH_VSWR;
            break;
        case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_RL:
            attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_OUTPUT_MATCH_RETURN_LOSS;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, outputMatch),
                        3, "Output Match");
            break;
        }

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, attr, outputMatchValue),
                4, "Output Match Value");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_ENR_UNCERTAINTY, enrUncertainty),
                5, "ENR Uncertainty");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command defines the hot and cold temperature uncertainty of a
/// HIFN  resistor noise source. Refer to the data sheet of the noise source to
/// HIFN  determine its uncertainty.
/// HIFN     This command is available when
/// HIFN  RSFPL_ATTR_NOISE_COMMON_NOISE_SOURCE_ENABLED is off (see
/// HIFN  rsfpl_ConfigureNoiseCommonSourceEnabled) and
/// HIFN  RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_COMMON_SOURCE_FOR_MEAS_AND_CAL_ENABLED
/// HIFN  is off (see rsfpl_ConfigureNoiseFigureUncertaintyCalibrationSource).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hotUncertainty/This command defines the hot temperature uncertainty of a resistor
/// HIPAR hotUncertainty/noise source. Refer to the data sheet of the noise source to determine
/// HIPAR hotUncertainty/its uncertainty.
/// HIPAR coldUncertainty/This command defines the cold temperature uncertainty of a resistor
/// HIPAR coldUncertainty/noise source. Refer to the data sheet of the noise source to determine
/// HIPAR coldUncertainty/its uncertainty.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyCalibrationResistorTempUncertainty(
                            ViSession instrSession,
                            ViReal64  hotUncertainty,
                            ViReal64  coldUncertainty)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_RESISTOR_HOT_UNCERTAINTY, hotUncertainty),
            2, "Hot Uncertainty");

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_RESISTOR_COLD_UNCERTAINTY, coldUncertainty),
            3, "Cold Uncertainty");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command configures settings for the DUT input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputMatch/You can define the input match either as the VSWR or as the return
/// HIPAR inputMatch/loss (RL). Refer to the datasheet of the noise source for these values.
/// HIPAR inputMatchValue/Defines the VSWR or return loss at the DUT input.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyDUTInputMatch(
                            ViSession instrSession,
                            ViInt32   inputMatch,
                            ViReal64  inputMatchValue)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (inputMatch)
    {
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_VSWR:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_INPUT_MATCH_VSWR;
        break;
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_RL:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_INPUT_MATCH_RETURN_LOSS;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, inputMatch),
                    2, "Input Match");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, attr, inputMatchValue),
            3, "Input Match Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command configures settings for the DUT output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputMatch/You can define the output match either as the VSWR or as the return
/// HIPAR outputMatch/loss (RL). Refer to the datasheet of the noise source for these values.
/// HIPAR outputMatchValue/Defines the VSWR or return loss at the DUT output.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyDUTOutputMatch(
                            ViSession instrSession,
                            ViInt32   outputMatch,
                            ViReal64  outputMatchValue)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (outputMatch)
    {
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_VSWR:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_OUTPUT_MATCH_VSWR;
        break;
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_RL:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_OUTPUT_MATCH_RETURN_LOSS;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, outputMatch),
                    2, "Input Match");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, attr, outputMatchValue),
            3, "Output Match Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command configures the automatic or manual definition of the DUT
/// HIFN  characteristics.
/// HIFN     If Use Meas Values is on (VI_TRUE), the manual parameters are ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR useMeasValues/Turns automatic determination of the DUT characteristics for the
/// HIPAR useMeasValues/calculation of the uncertainty on and off.
/// HIPAR noiseFigure/Manually defines the noise level of the DUT. This parameter is ignored
/// HIPAR noiseFigure/if Use Meas Values is on.
/// HIPAR gain/Manually defines the gain of the DUT. This parameter is ignored if Use
/// HIPAR gain/Meas Values is on.
/// HIPAR frequency/Manually defines the frequency for which the uncertainty should be
/// HIPAR frequency/calculated. This parameter is ignored if Use Meas Values is on.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyDUTCharacteristics(
                            ViSession instrSession,
                            ViBoolean useMeasValues,
                            ViReal64  noiseFigure,
                            ViReal64  gain,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_USE_MEASUREMENT_VALUES_ENABLED, useMeasValues),
            2, "Use Meas Values");

    if (!useMeasValues)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_NOISE_FIGURE, noiseFigure),
                3, "Noise Figure");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_GAIN, gain),
                4, "Gain");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_FREQUENCY, frequency),
                5, "Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command includes or excludes an external preamplifier from the
/// HIFN  uncertainty calculation. If the test setup uses an external
/// HIFN  preamplifier, you also have to define its noise figure and gain values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preampEnabled/This command includes or excludes an external preamplifier from the
/// HIPAR preampEnabled/uncertainty calculation. If the test setup uses an external
/// HIPAR preampEnabled/preamplifier, you also have to define its noise figure and gain values.
/// HIPAR noiseFigure/This command defines the noise level of an external preamplifier that
/// HIPAR noiseFigure/may be part of the test setup.
/// HIPAR gain/This command define the gain of an external preamplifier that may be
/// HIPAR gain/part of the test setup.
/// HIPAR inputMatch/You can define the input match of the external preamp either as the
/// HIPAR inputMatch/VSWR or as the return loss (RL). Refer to the datasheet of the preamp
/// HIPAR inputMatch/for these values.
/// HIPAR inputMatchValue/Defines the VSWR or return loss at the external preamp.
ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyPreampCharacteristics(
                            ViSession instrSession,
                            ViBoolean preampEnabled,
                            ViReal64  noiseFigure,
                            ViReal64  gain,
                            ViInt32   inputMatch,
                            ViReal64  inputMatchValue)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_SetAttributeViBoolean(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_ENABLED, preampEnabled),
            2, "Preamp Enabled");

    if (preampEnabled)
    {
        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_NOISE_FIGURE, noiseFigure),
                3, "Noise Figure");

        viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_GAIN, gain),
                4, "Gain");
    }

    switch (inputMatch)
    {
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_VSWR:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_VSWR;
        break;
    case RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_RL:
        attr = RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_RETURN_LOSS;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, inputMatch),
                    5, "Input Match");
        break;
    }

    viCheckParm(rsfpl_SetAttributeViReal64(instrSession, NULL, attr, inputMatchValue),
            6, "Input Match Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command queries the internal uncertainty values of the FPL
/// HIFN  spectrum analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseFigureUncertainty/This command queries the uncertainty value of the spectrum analyzer's
/// HIPAR noiseFigureUncertainty/internal noise.
/// HIPAR gainUncertainty/This command queries the uncertainty value of the spectrum analyzer's
/// HIPAR gainUncertainty/internal gain.
ViStatus _VI_FUNC rsfpl_QueryNoiseFigureUncertaintyAnalyzerCharacteristics(
                            ViSession instrSession,
                            ViReal64* noiseFigureUncertainty,
                            ViReal64* gainUncertainty)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_ANALYZER_NOISE_FIGURE_UNCERTAINTY, noiseFigureUncertainty),
            2, "Noise Figure Uncertainty");

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_ANALYZER_GAIN_UNCERTAINTY, gainUncertainty),
            3, "Gain Uncertainty");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command queries the uncertainty of noise figure results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsfpl_init or
/// HIPAR instrSession/rsfpl_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uncertaintyResult/This command queries the uncertainty of noise figure results.
ViStatus _VI_FUNC rsfpl_QueryNoiseFigureUncertaintyResult(ViSession instrSession,
                                                          ViReal64* uncertaintyResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsfpl_GetAttributeViReal64(instrSession, NULL, RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_RESULT, uncertaintyResult),
            2, "Uncertainty Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
