/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include "rsfpl.h"

 /*****************************************************************************
  * Function: rsfpl_InitAttributes
  * Purpose:  This function inits attributes to the desired values if needed.
  *****************************************************************************/
ViStatus rsfpl_InitAttributes(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    // - Driver Identification -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSFPL_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSFPL_SPECIFIC_DRIVER_PREFIX));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSFPL_SPECIFIC_DRIVER_LOCATOR));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSFPL_SPECIFIC_DRIVER_VENDOR));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSFPL_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSFPL_CLASS_SPEC_MINOR_VERSION));

    // - Driver Capabilities -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSFPL_SUPPORTED_INSTRUMENT_MODELS));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSFPL_GROUP_CAPABILITIES));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSFPL_FUNCTION_CAPABILITIES));

    // - Version Info -
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, RSFPL_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, RSFPL_MINOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, RSFPL_MINOR_MINOR_VERSION));

    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, RSFPL_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, RSFPL_CLASS_SPEC_MINOR_VERSION));

    checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsfpl_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rsfpl_reset function calls this function. The
 *           rsfpl_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsfpl_DefaultInstrSetup(ViSession instrSession)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // Set all the attributes to the default state. Do not update inherent attributes!
    checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

    // Init attributes
    checkErr(rsfpl_InitAttributes(instrSession));

    rsSession->fastSweepInstrument = 0;
    checkErr(RsCore_ResetRegistersEseSre(instrSession));
    checkErr(RsCore_Write(instrSession, "INST SAN;:INIT:CONT 0"));
    checkErr(RsCore_QueryViInt32(instrSession, "*OPC?", NULL));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsfpl_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rsfpl_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsfpl_CheckStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
    {
        checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
    }

Error:
    return error;
}

/****************************************************************************************************
 *  rsfpl_dataReadTrace
    Queries Float array in BIN or ASCII format (sets BIN format, but can handle both)
    Sends command TRAC%ld? <trace> or TRAC? <trace>
    You have to provide an allocated traceData[] buffer
******************************************************************************************************/
ViStatus rsfpl_dataReadTrace(ViSession instrSession,
                             ViInt32   window,
                             ViString  trace,
                             ViInt32   arrayLength,
                             ViReal64  traceData[],
                             ViPInt32  noofPoints)
{  
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    if (window == 0)
    {
        if (trace != NULL && strlen(trace) > 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? %s", trace);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC?");
    }
    else
    {
        if (trace != NULL && strlen(trace) > 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? %s", window, trace);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld?", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, traceData, noofPoints));

Error:
    return error;
}

/****************************************************************************************************
 *  rsfpl_dataQueryInterleavedData
    Queries two Float arrays that are interleaved (ItemA_0, ItemB_0, ItemA_1, ItemB_1, ... ItemA_n, Item B_n)
    If timeout is >=0, the query is sent with OPC sync
    Use this function e.g. for IQ data read
    arraySize = arrayA size = arrayB size
******************************************************************************************************/
ViStatus _VI_FUNC rsfpl_dataReadInterleaved(ViSession instrSession,
                                            ViString  cmd,
                                            ViInt32   timeout,
                                            ViInt32   arraySize,
                                            ViReal64  arrayA[],
                                            ViReal64  arrayB[],
                                            ViPInt32  actualPoints)
{  
    ViStatus  error = VI_SUCCESS;
    ViInt32   i = 0;
    ViInt32   dataSize = 0;
    ViInt32   noofPoints = 0;
    ViReal64* data = NULL;

    if (timeout < 0)
    {
        checkErr(RsCore_QueryFloatArray(instrSession, cmd, &data, &dataSize));
    }
    else
    {
        checkErr(RsCore_QueryFloatArrayWithOpc(instrSession, cmd, timeout, &data, &dataSize));
    }

    noofPoints = dataSize / 2;
    if (actualPoints)
        *actualPoints = noofPoints;

    if (noofPoints > arraySize)
        noofPoints = arraySize;

    for (i = 0; i < noofPoints; i++)
    {
        arrayA[i] = data[i * 2];
        arrayB[i] = data[i * 2 + 1];
    }

Error:
    if (data)
        free(data);

    return error;
}

ViStatus rsfpl_utility_error_message(ViSession instrSession,
                                     ViStatus  errorCode,
                                     ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    (void)(RsCore_LockSession(instrSession));

    static RsCoreStringValueTable errorTable =
    {
        RSFPL_ERROR_CODES_AND_MSGS,
        {0, NULL}
    };

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:  
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
