/*****************************************************************************
 *  rsfpl.h
 *
 *  Rohde & Schwarz FPL spectrum analyzer Instrument Driver
 *  Original Release: June 2018
 *  By: Jiri Kominek
 *
 *  Purpose: VXI Plug&Play instrument driver for R&S FPL
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Copyright (c) 2019, Rohde & Schwarz GmbH & Co. KG
 *  All rights reserved.
 *****************************************************************************/

#ifndef __RSFPL_HEADER
#define __RSFPL_HEADER

#include "rscore.h"
#include "rsfpl_attributes.h"
#include "rsfpl_utility.h"

#include <float.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

    /****************************************************************************
     *----------------- Instrument Driver Revision Information -----------------*
     ****************************************************************************/

     /* Class Driver Identification */

#define RSFPL_CLASS_DRIVER_DESCRIPTION           ""
#define RSFPL_CLASS_DRIVER_PREFIX                ""
#define RSFPL_CLASS_DRIVER_VENDOR                ""
#define RSFPL_CLASS_DRIVER_REVISION              ""
#define RSFPL_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSFPL_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSFPL_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSFPL_SPECIFIC_DRIVER_PREFIX             "RSFPL"
#define RSFPL_SPECIFIC_DRIVER_LOCATOR            ""
#define RSFPL_SPECIFIC_DRIVER_VENDOR             ""
#define RSFPL_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSFPL_MINOR_VERSION                      90L   /* Instrument driver minor version          */
#define RSFPL_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSFPL_SUPPORTED_INSTRUMENT_MODELS        "FPL,ZNL"    /* Insftrument driver supported model(s)     */
#define RSFPL_GROUP_CAPABILITIES                 ""
#define RSFPL_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------------ Useful Macros -----------------------------*
 ****************************************************************************/

 /*****************************************************************************
  *- Useful Macros & Constants -----------------------------------------------*
  *****************************************************************************/

#define RSFPL_SIMULATION_ID_QUERY      "Rohde&Schwarz,FPL,123456/789,1.23"  /* Default definition of ID Query Response for simulation */
#define RSFPL_VALID_ID_RESPONSE_STRING "Rohde"      /* Valid response for identification query */
#define RSFPL_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSFPL_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */

  /****************************************************************************
   *---------------------------- Attribute Defines ---------------------------*
   ****************************************************************************/

   /*- Inherent Instrument Specific Attributes ----------------------------*/

       /* User Options */

#define RSFPL_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSFPL_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSFPL_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSFPL_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSFPL_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSFPL_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSFPL_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSFPL_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSFPL_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSFPL_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSFPL_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSFPL_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSFPL_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSFPL_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSFPL_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSFPL_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSFPL_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSFPL_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSFPL_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSFPL_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSFPL_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSFPL_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSFPL_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSFPL_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSFPL_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSFPL_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSFPL_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSFPL_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSFPL_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSFPL_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSFPL_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

/* BEGIN GENERATE */
#define RSFPL_ATTR_OPERATING_MODE_MULTIVIEW_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6809
#define RSFPL_ATTR_CHANNEL_SETUP_DUPLICATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6810
#define RSFPL_ATTR_CHANNEL_SETUP_DELETE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6813
#define RSFPL_ATTR_OPERATING_MODE_LIST_OF_CHANNEL_SETUPS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6814
#define RSFPL_ATTR_SEQUENCER_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6817
#define RSFPL_ATTR_SEQUENCER_INIT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6818
#define RSFPL_ATTR_SEQUENCER_ABORT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6819
#define RSFPL_ATTR_SEQUENCER_MODE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6820
#define RSFPL_ATTR_RF_INPUT_SELECT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7694
#define RSFPL_ATTR_RF_INPUT_SAW_FILTER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7695
#define RSFPL_ATTR_RF_INPUT_IMPEDANCE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6821
#define RSFPL_ATTR_RF_INPUT_PROTECTION_RESET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6822
#define RSFPL_ATTR_POWER_SENSOR_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6823
#define RSFPL_ATTR_POWER_SENSOR_CONTINUOUS_UPDATE_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6824
#define RSFPL_ATTR_POWER_SENSOR_COUNT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6825
#define RSFPL_ATTR_POWER_SENSOR_AUTO_ASSIGN_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6826
#define RSFPL_ATTR_POWER_SENSOR_TO_REFERENCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6828
#define RSFPL_ATTR_POWER_SENSOR_RELATIVE_POWER_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6829
#define RSFPL_ATTR_POWER_SENSOR_REFERENCE_VALUE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6830
#define RSFPL_ATTR_POWER_SENSOR_ZERO                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6831
#define RSFPL_ATTR_POWER_SENSOR_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6832
#define RSFPL_ATTR_POWER_SENSOR_FREQUENCY_COUPLING                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6833
#define RSFPL_ATTR_POWER_SENSOR_USE_REF_LEVEL_OFFSET_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6834
#define RSFPL_ATTR_POWER_SENSOR_SENSOR_LEVEL_OFFSET                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8116
#define RSFPL_ATTR_POWER_SENSOR_ABSOLUTE_POWER_UNIT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6835
#define RSFPL_ATTR_POWER_SENSOR_RELATIVE_POWER_UNIT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6836
#define RSFPL_ATTR_POWER_SENSOR_NUMBER_OF_READINGS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6837
#define RSFPL_ATTR_POWER_SENSOR_MEAS_TIME                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6838
#define RSFPL_ATTR_POWER_SENSOR_AVERAGING_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6839
#define RSFPL_ATTR_POWER_SENSOR_DUTY_CYCLE_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6840
#define RSFPL_ATTR_POWER_SENSOR_DUTY_CYCLE_VALUE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6841
#define RSFPL_ATTR_POWER_SENSOR_FETCH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6842
#define RSFPL_ATTR_POWER_SENSOR_READ                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6843
#define RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7502
#define RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_LEVEL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7503
#define RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_HYSTERESIS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7504
#define RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_DROPOUT_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7505
#define RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_HOLDOFF_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7506
#define RSFPL_ATTR_POWER_SENSOR_EXTERNAL_TRIGGER_SLOPE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7507
#define RSFPL_ATTR_INTERNAL_GENERATOR_STATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8117
#define RSFPL_ATTR_INTERNAL_GENERATOR_TRACKING_GENERATOR_STATE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8118
#define RSFPL_ATTR_INTERNAL_GENERATOR_POWER_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8119
#define RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8120
#define RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_OFFSET                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8121
#define RSFPL_ATTR_INTERNAL_GENERATOR_CW_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8122
#define RSFPL_ATTR_INTERNAL_GENERATOR_FREQUENCY_OFFSET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8123
#define RSFPL_ATTR_INTERNAL_GENERATOR_CALIBRATION_METHOD                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8124
#define RSFPL_ATTR_INTERNAL_GENERATOR_SAVE_AS_TRANSDUCER_FACTOR                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8130
#define RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_START                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8131
#define RSFPL_ATTR_INTERNAL_GENERATOR_LEVEL_STOP                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8132
#define RSFPL_ATTR_OUTPUT_NOISE_SOURCE_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6844
#define RSFPL_ATTR_OUTPUT_SOURCE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6845
#define RSFPL_ATTR_OUTPUT_FREQUENCY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6846
#define RSFPL_ATTR_OUTPUT_USER_PORT_OUTPUT_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6847
#define RSFPL_ATTR_OUTPUT_USER_PORT_OUTPUT_VALUE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6848
#define RSFPL_ATTR_OUTPUT_SIGNAL_POLARITY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6849
#define RSFPL_ATTR_OUTPUT_LOUDSPEAKER_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6850
#define RSFPL_ATTR_OUTPUT_LOUDSPEAKER_MUTE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8133
#define RSFPL_ATTR_OUTPUT_LOUDSPEAKER_VOLUME                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6851
#define RSFPL_ATTR_USER_PORT_INPUT_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6852
#define RSFPL_ATTR_USER_PORT_VALUE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6853
#define RSFPL_ATTR_PREAMPLIFIER_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6854
#define RSFPL_ATTR_PREAMPLIFIER_GAIN                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7550
#define RSFPL_ATTR_MARKER_TO_REFERENCE_LEVEL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6856
#define RSFPL_ATTR_AMPLITUDE_UNIT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6857
#define RSFPL_ATTR_REFERENCE_LEVEL                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6858
#define RSFPL_ATTR_REFERENCE_LEVEL_OFFSET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6859
#define RSFPL_ATTR_ATTENUATION                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6860
#define RSFPL_ATTR_ATTENUATION_COUPLING_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6861
#define RSFPL_ATTR_Y_AXIS_AUTOSCALE_ONCE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6863
#define RSFPL_ATTR_Y_AXIS_SCALING                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6864
#define RSFPL_ATTR_Y_AXIS_REFERENCE_VALUE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7617
#define RSFPL_ATTR_Y_AXIS_SCALING_MODE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6866
#define RSFPL_ATTR_SUBWINDOW_Y_AXIS_SCALING                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7696
#define RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_VALUE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8134
#define RSFPL_ATTR_SUBWINDOW_Y_AXIS_SCALING_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7697
#define RSFPL_ATTR_SUBWINDOW_Y_AXIS_REFERENCE_LEVEL_POSITION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6865
#define RSFPL_ATTR_SUBWINDOW_Y_AXIS_RANGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6862
#define RSFPL_ATTR_SUBWINDOW_Y_AXIS_GRID_SPACING                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7616
#define RSFPL_ATTR_FREQUENCY_CENTER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6867
#define RSFPL_ATTR_FREQUENCY_SPAN                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6868
#define RSFPL_ATTR_FREQUENCY_SPAN_FULL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6869
#define RSFPL_ATTR_FREQUENCY_START                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6870
#define RSFPL_ATTR_FREQUENCY_STOP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6871
#define RSFPL_ATTR_FREQUENCY_OFFSET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6872
#define RSFPL_ATTR_FREQUENCY_CENTER_UP                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6873
#define RSFPL_ATTR_FREQUENCY_CENTER_DOWN                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6874
#define RSFPL_ATTR_FREQUENCY_STEP_SIZE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6875
#define RSFPL_ATTR_FREQUENCY_STEP_SIZE_COUPLING_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6876
#define RSFPL_ATTR_FREQUENCY_STEP_SIZE_COUPLING_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6877
#define RSFPL_ATTR_FREQUENCY_STEP_SIZE_FACTOR                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6878
#define RSFPL_ATTR_FREQUENCY_MARKER_TO_CENTER                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6879
#define RSFPL_ATTR_FREQUENCY_MARKER_TO_STEP_SIZE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6882
#define RSFPL_ATTR_FREQUENCY_ANNOTATION_MODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8346
#define RSFPL_ATTR_SIGNAL_TRACKING_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6891
#define RSFPL_ATTR_SIGNAL_TRACKING_BANDWIDTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6892
#define RSFPL_ATTR_SIGNAL_TRACKING_THRESHOLD                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6893
#define RSFPL_ATTR_SIGNAL_TRACKING_TRACE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6894
#define RSFPL_ATTR_HORIZONTAL_SCALE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7501
#define RSFPL_ATTR_SUBWINDOW_HORIZONTAL_SCALE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7698
#define RSFPL_ATTR_RBW                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6895
#define RSFPL_ATTR_RBW_AUTO_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6896
#define RSFPL_ATTR_VBW                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6897
#define RSFPL_ATTR_VBW_AUTO_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6898
#define RSFPL_ATTR_RBW_TO_VBW_RATIO                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6899
#define RSFPL_ATTR_VBW_FILTER_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6900
#define RSFPL_ATTR_RBW_TO_SPAN_RATIO                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6901
#define RSFPL_ATTR_RBW_FILTER_TYPE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6902
#define RSFPL_ATTR_ADJUST_ALL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6883
#define RSFPL_ATTR_ADJUST_FREQUENCY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6884
#define RSFPL_ATTR_ADJUST_LEVEL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6885
#define RSFPL_ATTR_ADJUST_MANUAL_LENGTH_OF_MEASUREMENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6886
#define RSFPL_ATTR_ADJUST_LENGTH_OF_MEASUREMENT_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6887
#define RSFPL_ATTR_ADJUST_LOWER_LEVEL_HYSTERESIS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6888
#define RSFPL_ATTR_ADJUST_UPPER_LEVEL_HYSTERESIS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6889
#define RSFPL_ATTR_ADJUST_WAIT_FOR_TRIGGER_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6890
#define RSFPL_ATTR_SWEEP_COUNT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6903
#define RSFPL_ATTR_CURRENT_NUMBER_OF_SWEEPS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6904
#define RSFPL_ATTR_SWEEP_MODE_CONTINUOUS_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6905
#define RSFPL_ATTR_CAPTURE_TIME                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6906
#define RSFPL_ATTR_PARTIAL_SPAN_OPTIMIZATION_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6907
#define RSFPL_ATTR_SWEEP_POINTS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6908
#define RSFPL_ATTR_SWEEP_TIME                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6909
#define RSFPL_ATTR_SWEEP_TIME_AUTO_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6910
#define RSFPL_ATTR_SWEEP_TYPE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6911
#define RSFPL_ATTR_SWEEP_TYPE_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8347
#define RSFPL_ATTR_SWEEP_FFT_SUBSPAN                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8348
#define RSFPL_ATTR_WINDOW_SWEEP_POINTS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7699
#define RSFPL_ATTR_TRIGGER_SOURCE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6912
#define RSFPL_ATTR_TRIGGER_LEVEL_IFP                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6913
#define RSFPL_ATTR_TRIGGER_LEVEL_IQ_POWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6914
#define RSFPL_ATTR_TRIGGER_DROP_OUT_TIME                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6915
#define RSFPL_ATTR_TRIGGER_OFFSET                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6916
#define RSFPL_ATTR_TRIGGER_SLOPE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6917
#define RSFPL_ATTR_TRIGGER_HYSTERESIS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6918
#define RSFPL_ATTR_TRIGGER_HOLDOFF                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6919
#define RSFPL_ATTR_EXTERNAL_TRIGGER_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7700
#define RSFPL_ATTR_GATE_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6921
#define RSFPL_ATTR_GATE_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6922
#define RSFPL_ATTR_GATE_DELAY                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6923
#define RSFPL_ATTR_GATE_LENGTH                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6924
#define RSFPL_ATTR_GATE_POLARITY                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6925
#define RSFPL_ATTR_GATE_SOURCE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6926
#define RSFPL_ATTR_MARKER_TABLE_DISPLAY_MODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6927
#define RSFPL_ATTR_MARKER_INFO_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6928
#define RSFPL_ATTR_MARKER_STEPSIZE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6929
#define RSFPL_ATTR_MARKER_LINKING                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7701
#define RSFPL_ATTR_MARKER_EXCLUDE_LO_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6930
#define RSFPL_ATTR_MARKER_PEAK_EXCURSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6931
#define RSFPL_ATTR_MARKER_SEARCH_LIMITS_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6932
#define RSFPL_ATTR_MARKER_SEARCH_LIMIT_LEFT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6933
#define RSFPL_ATTR_MARKER_SEARCH_LIMIT_RIGHT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6934
#define RSFPL_ATTR_MARKER_SEARCH_ZOOM_LIMITS_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6935
#define RSFPL_ATTR_MARKER_SEARCH_THRESHOLD_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6936
#define RSFPL_ATTR_MARKER_SEARCH_THRESHOLD                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6937
#define RSFPL_ATTR_DELTA_MARKER_MODE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6938
#define RSFPL_ATTR_ALL_DELTA_MARKERS_OFF                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6939
#define RSFPL_ATTR_MARKER_RESULT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6940
#define RSFPL_ATTR_ALL_MARKERS_OFF                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6941
#define RSFPL_ATTR_MARKER_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6944
#define RSFPL_ATTR_MARKER_ASSIGNED_TRACE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6945
#define RSFPL_ATTR_MARKER_POSITION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6946
#define RSFPL_ATTR_MARKER_AMPLITUDE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7702
#define RSFPL_ATTR_MARKER_PROBABILITY                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7703
#define RSFPL_ATTR_MARKER_ZOOM_FACTOR                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8135
#define RSFPL_ATTR_DELTA_MARKER_REFERENCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6947
#define RSFPL_ATTR_DELTA_MARKER_REFERENCE_FIXED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6948
#define RSFPL_ATTR_MARKER_AUTO_MAX_PEAK_SEARCH_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6949
#define RSFPL_ATTR_MARKER_MAX_PEAK_LEFT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6950
#define RSFPL_ATTR_MARKER_MAX_PEAK_NEXT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6951
#define RSFPL_ATTR_MARKER_MAX_PEAK_RIGHT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6952
#define RSFPL_ATTR_MARKER_MAX_PEAK                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6953
#define RSFPL_ATTR_MARKER_AUTO_MIN_PEAK_SEARCH_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6954
#define RSFPL_ATTR_MARKER_MIN_PEAK_LEFT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6955
#define RSFPL_ATTR_MARKER_MIN_PEAK_NEXT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6956
#define RSFPL_ATTR_MARKER_MIN_PEAK_RIGHT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6957
#define RSFPL_ATTR_MARKER_MIN_PEAK                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6958
#define RSFPL_ATTR_FIXED_REFERENCE_MARKER_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6959
#define RSFPL_ATTR_FIXED_REFERENCE_MARKER_MAX_PEAK                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6960
#define RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_HORIZONTAL               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6961
#define RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_VERTICAL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6962
#define RSFPL_ATTR_FIXED_REFERENCE_MARKER_REFERENCE_POINT_LEVEL_OFFSET             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6963
#define RSFPL_ATTR_MARKER_PEAK_SEARCH_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6964
#define RSFPL_ATTR_MARKER_PEAK_LIST_LABELS_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6965
#define RSFPL_ATTR_MARKER_PEAK_LIST_MAXIMUM_NUMBER_OF_PEAKS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6966
#define RSFPL_ATTR_MARKER_PEAK_LIST_PEAKS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6967
#define RSFPL_ATTR_MARKER_PEAK_LIST_NUMBER_OF_FOUND_PEAKS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6968
#define RSFPL_ATTR_MARKER_PEAK_LIST_SORT_MODE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6969
#define RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6972
#define RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_ALL_OFF                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7704
#define RSFPL_ATTR_NOISE_MEASUREMENT_MARKER_RESULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6974
#define RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6975
#define RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_ALL_OFF                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6976
#define RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_MARKER_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6977
#define RSFPL_ATTR_BAND_POWER_MARKER_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6978
#define RSFPL_ATTR_BAND_POWER_MARKER_ALL_OFF                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7705
#define RSFPL_ATTR_BAND_POWER_MARKER_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6980
#define RSFPL_ATTR_BAND_POWER_MARKER_RESULT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6981
#define RSFPL_ATTR_BAND_POWER_MARKER_SPAN                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6982
#define RSFPL_ATTR_N_DB_DOWN_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6983
#define RSFPL_ATTR_N_DB_DOWN_DISTANCE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6984
#define RSFPL_ATTR_N_DB_DOWN_RESULT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6986
#define RSFPL_ATTR_N_DB_DOWN_Q_FACTOR                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6987
#define RSFPL_ATTR_SIGNAL_COUNT_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6989
#define RSFPL_ATTR_SIGNAL_COUNT_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6990
#define RSFPL_ATTR_SIGNAL_COUNT_RESOLUTION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6991
#define RSFPL_ATTR_MARKER_DEMODULATION_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6992
#define RSFPL_ATTR_MARKER_DEMODULATION_CONTINUOUS_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6993
#define RSFPL_ATTR_MARKER_DEMODULATION_STOP_TIME                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6994
#define RSFPL_ATTR_MARKER_DEMODULATION_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6995
#define RSFPL_ATTR_MARKER_DEMODULATION_SELECTIVE_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6996
#define RSFPL_ATTR_MARKER_DEMODULATION_SELECTIVE_THRESHOLD                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6997
#define RSFPL_ATTR_DELTA_MARKER_RESULT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6998
#define RSFPL_ATTR_DELTA_MARKER_RELATIVE_POSITION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6999
#define RSFPL_ATTR_DELTA_MARKER_LINK_TO_MARKER_1                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7000
#define RSFPL_ATTR_DELTA_MARKER_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7003
#define RSFPL_ATTR_DELTA_MARKER_ASSIGNED_TRACE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7004
#define RSFPL_ATTR_DELTA_MARKER_POSITION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7005
#define RSFPL_ATTR_DELTA_MARKER_MAX_PEAK_LEFT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7006
#define RSFPL_ATTR_DELTA_MARKER_MAX_PEAK_NEXT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7007
#define RSFPL_ATTR_DELTA_MARKER_MAX_PEAK_RIGHT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7008
#define RSFPL_ATTR_DELTA_MARKER_MAX_PEAK                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7009
#define RSFPL_ATTR_DELTA_MARKER_MIN_PEAK_LEFT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7010
#define RSFPL_ATTR_DELTA_MARKER_MIN_PEAK_NEXT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7011
#define RSFPL_ATTR_DELTA_MARKER_MIN_PEAK_RIGHT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7012
#define RSFPL_ATTR_DELTA_MARKER_MIN_PEAK                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7013
#define RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7014
#define RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_AUTO_PEAK_SEARCH_ENABLED   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7015
#define RSFPL_ATTR_PHASE_NOISE_MEASUREMENT_DELTA_MARKER_RESULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7016
#define RSFPL_ATTR_BAND_POWER_DELTA_MARKER_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7017
#define RSFPL_ATTR_BAND_POWER_DELTA_MARKER_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7018
#define RSFPL_ATTR_BAND_POWER_DELTA_MARKER_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7019
#define RSFPL_ATTR_BAND_POWER_DELTA_MARKER_SPAN                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7020
#define RSFPL_ATTR_SPECTROGRAM_MARKER_FRAME                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7021
#define RSFPL_ATTR_SPECTROGRAM_MARKER_SEARCH_AREA                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7022
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_LEVEL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7023
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_LEVEL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7024
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK_ABOVE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7025
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK_BELOW                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7026
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK_NEXT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7027
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MAX_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7028
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK_ABOVE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7029
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK_BELOW                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7030
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK_NEXT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7031
#define RSFPL_ATTR_SPECTROGRAM_MARKER_MIN_PEAK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7032
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_FRAME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7033
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_SEARCH_AREA                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7034
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_LEVEL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7035
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_LEVEL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7036
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK_ABOVE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7037
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK_BELOW                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7038
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK_NEXT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7039
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MAX_PEAK                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7040
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK_ABOVE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7041
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK_BELOW                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7042
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK_NEXT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7043
#define RSFPL_ATTR_SPECTROGRAM_DELTA_MARKER_MIN_PEAK                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7044
#define RSFPL_ATTR_TRACE_AVERAGE_COUNT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7045
#define RSFPL_ATTR_TRACE_AVERAGE_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7046
#define RSFPL_ATTR_TRACE_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7047
#define RSFPL_ATTR_TRACE_DETECTOR_AUTO_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7049
#define RSFPL_ATTR_TRACE_DETECTOR                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7050
#define RSFPL_ATTR_TRACE_HOLD_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7051
#define RSFPL_ATTR_TRACE_SMOOTHING_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7052
#define RSFPL_ATTR_TRACE_SMOOTHING_VALUE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7053
#define RSFPL_ATTR_TRACE_AVERAGE_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7054
#define RSFPL_ATTR_DISPLAY_WINDOW_PRESET_TRACES                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8136
#define RSFPL_ATTR_SUBWINDOW_TRACE_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7706
#define RSFPL_ATTR_SUBWINDOW_TRACE_HOLD_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7707
#define RSFPL_ATTR_SUBWINDOW_TRACE_SMOOTHING_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7708
#define RSFPL_ATTR_SUBWINDOW_TRACE_SMOOTHING_VALUE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7709
#define RSFPL_ATTR_DISPLAY_SUBWINDOW_PRESET_TRACES                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8137
#define RSFPL_ATTR_SUBWINDOW_TRACE_MODE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7048
#define RSFPL_ATTR_TRACE_MATHEMATICS_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7060
#define RSFPL_ATTR_TRACE_MATHEMATICS_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7061
#define RSFPL_ATTR_TRACE_MATHEMATICS_POSITION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7062
#define RSFPL_ATTR_SPECTROGRAM_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7064
#define RSFPL_ATTR_SPECTROGRAM_3D_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7710
#define RSFPL_ATTR_SPECTROGRAM_SELECTED_FRAME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7065
#define RSFPL_ATTR_SPECTROGRAM_HISTORY_DEPTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7066
#define RSFPL_ATTR_SPECTROGRAM_TIME_STAMP_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7067
#define RSFPL_ATTR_SPECTROGRAM_NUMBER_OF_FRAMES                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7069
#define RSFPL_ATTR_SPECTROGRAM_CONTINOUS_SWEEP_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7070
#define RSFPL_ATTR_SPECTROGRAM_CLEAR                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7071
#define RSFPL_ATTR_SPECTROGRAM_LAYOUT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7072
#define RSFPL_ATTR_SPECTROGRAM_SELECT_TRACE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7711
#define RSFPL_ATTR_COLOR_MAP_RESTORE_DEFAULT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7073
#define RSFPL_ATTR_COLOR_MAP_SCHEME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7074
#define RSFPL_ATTR_COLOR_MAP_STARTING_POINT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7075
#define RSFPL_ATTR_COLOR_MAP_END_POINT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7076
#define RSFPL_ATTR_COLOR_MAP_CURVE_SHAPE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7077
#define RSFPL_ATTR_ABORT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7078
#define RSFPL_ATTR_INITIATE_SYNCHRONIZED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7079
#define RSFPL_ATTR_INITIATE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7080
#define RSFPL_ATTR_CONTINUE_MEASUREMENT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7081
#define RSFPL_ATTR_SEND_SW_TRIGGER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7082
#define RSFPL_ATTR_MEASUREMENT_MODE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7083
#define RSFPL_ATTR_DISABLE_POWER_MEASUREMENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7084
#define RSFPL_ATTR_POWER_MEASUREMENT_SELECT_AND_ENABLE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7085
#define RSFPL_ATTR_POWER_MEASUREMENT_ADJUST_SETTINGS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7086
#define RSFPL_ATTR_POWER_MEASUREMENT_TRACE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7087
#define RSFPL_ATTR_POWER_MEASUREMENT_TRACE_DISPLAY_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7088
#define RSFPL_ATTR_POWER_MEASUREMENTS_FIND_REFERENCE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7089
#define RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8138
#define RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_REFERENCE_MODE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8139
#define RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_REFERENCE_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8140
#define RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_1_DB_POINT_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8141
#define RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_3_DB_POINT_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8142
#define RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_N_DB_POINT_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8143
#define RSFPL_ATTR_COMPRESSION_POINT_MEASUREMENT_N_DB_POINT_VALUE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8144
#define RSFPL_ATTR_QUERY_COMPRESSION_POINT_MEASUREMENT_1_DB_RESULT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8148
#define RSFPL_ATTR_QUERY_COMPRESSION_POINT_MEASUREMENT_3_DB_RESULT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8149
#define RSFPL_ATTR_QUERY_COMPRESSION_POINT_MEASUREMENT_N_DB_RESULT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8150
#define RSFPL_ATTR_CHANNEL_POWER_DISPLAY_MODE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7092
#define RSFPL_ATTR_CHANNEL_POWER_AS_DENSITY_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7093
#define RSFPL_ATTR_POWER_CHANNEL_NUMBER_OF_TRANSMISSION_CHANNELS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7094
#define RSFPL_ATTR_CHANNEL_POWER_CHANNEL_PAIRS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7095
#define RSFPL_ATTR_CHANNEL_POWER_WEIGHTING_FILTER_ENABLED_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7096
#define RSFPL_ATTR_CHANNEL_POWER_WEIGHTING_FILTER_VALUE_ALL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7097
#define RSFPL_ATTR_CHANNEL_POWER_ACLR_LIMIT_CHECK_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7098
#define RSFPL_ATTR_ACLR_MEASUREMENT_CONFIGURATIONS_CATALOG                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7099
#define RSFPL_ATTR_ACLR_SELECTED_MEASUREMENT_CONFIGURATION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7100
#define RSFPL_ATTR_ACLR_MEASUREMENT_CONFIGURATIONS_LOAD                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7512
#define RSFPL_ATTR_ACLR_MEASUREMENT_CONFIGURATIONS_SAVE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7102
#define RSFPL_ATTR_ACLR_MEASUREMENT_CONFIGURATIONS_DELETE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7104
#define RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_BANDWIDTH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7106
#define RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_NAME                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7107
#define RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_SPACING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7108
#define RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_WEIGHTING_FILTER_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7109
#define RSFPL_ATTR_CHANNEL_POWER_TX_CHANNEL_WEIGHTING_FILTER_VALUE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7110
#define RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_BANDWIDTH                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7111
#define RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_NAME                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7112
#define RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_SPACING                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7113
#define RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_WEIGHTING_FILTER_ENABLED         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7114
#define RSFPL_ATTR_CHANNEL_POWER_ADJACENT_CHANNEL_WEIGHTING_FILTER_VALUE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7115
#define RSFPL_ATTR_CHANNEL_POWER_ADJACENT_LIMIT_ABSOLUTE_CHECK_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7116
#define RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_BANDWIDTH                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7121
#define RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_NAME                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7122
#define RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_SPACING                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7123
#define RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_ENABLED        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7124
#define RSFPL_ATTR_CHANNEL_POWER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_VALUE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7125
#define RSFPL_ATTR_REFERENCE_CHANNEL_SET_CP_REFERENCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7131
#define RSFPL_ATTR_REFERENCE_CHANNEL_NUMBER_AUTO_TYPE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7132
#define RSFPL_ATTR_REFERENCE_CHANNEL_NUMBER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7133
#define RSFPL_ATTR_PERCENT_OF_POWER_BANDWIDTH                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7134
#define RSFPL_ATTR_SEM_INITIATE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7135
#define RSFPL_ATTR_SEM_NUMBER_OF_SUB_BLOCKS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7136
#define RSFPL_ATTR_SEM_CONFIGURATIONS_RESTORE_DEFAULT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7137
#define RSFPL_ATTR_SEM_CONFIGURATIONS_LOAD                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7139
#define RSFPL_ATTR_SEM_CONFIGURATIONS_SAVE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7141
#define RSFPL_ATTR_SEM_CONFIGURATIONS_RESTORE_LIMITS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7142
#define RSFPL_ATTR_SEM_SUB_BLOCK_FREQUENCY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7143
#define RSFPL_ATTR_SWEEP_LIST_FAST_SEM_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7146
#define RSFPL_ATTR_SWEEP_LIST_SYMMETRICAL_SETUP_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8151
#define RSFPL_ATTR_SWEEP_LIST_NUMBER_OF_RANGES                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7147
#define RSFPL_ATTR_SEM_POWER_CLASSES_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7148
#define RSFPL_ATTR_SEM_POWER_CLASSES_MANUAL_LIMIT_LINE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7149
#define RSFPL_ATTR_SEM_POWER_CLASSES_NUMBER_OF_CLASSES                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7150
#define RSFPL_ATTR_SWEEP_LIST_RANGE_INSERT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7152
#define RSFPL_ATTR_SWEEP_LIST_RANGE_DELETE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7154
#define RSFPL_ATTR_SWEEP_LIST_RANGE_START                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7155
#define RSFPL_ATTR_SWEEP_LIST_RANGE_STOP                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7156
#define RSFPL_ATTR_SWEEP_LIST_RANGE_FILTER_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7157
#define RSFPL_ATTR_SWEEP_LIST_RANGE_RBW                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7158
#define RSFPL_ATTR_SWEEP_LIST_RANGE_VBW                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7159
#define RSFPL_ATTR_SWEEP_LIST_RANGE_SWEEP_TIME_AUTO_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7160
#define RSFPL_ATTR_SWEEP_LIST_RANGE_SWEEP_TIME_MANUAL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7161
#define RSFPL_ATTR_SWEEP_LIST_RANGE_REFERENCE_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7162
#define RSFPL_ATTR_SWEEP_LIST_RANGE_ATTENUATION_AUTO_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7163
#define RSFPL_ATTR_SWEEP_LIST_RANGE_ATTENUATION_MANUAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7164
#define RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7165
#define RSFPL_ATTR_SWEEP_LIST_RANGE_PREAMPLIFIER_LEVEL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7712
#define RSFPL_ATTR_SWEEP_LIST_RANGE_TRANSDUCER_FILE_NAME                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7166
#define RSFPL_ATTR_SWEEP_LIST_RANGE_MULTI_LIMIT_CALCULATION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7167
#define RSFPL_ATTR_SWEEP_LIST_RANGE_MIN_SWEEP_POINTS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7168
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7169
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7170
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_CHECK_MODE_ALL_RANGES              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7171
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_MAX                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7172
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_MIN                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7173
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_ABSOLUTE_START                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7174
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_ABSOLUTE_STOP                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7175
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7176
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_STOP                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7177
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_START_FUNCTION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7178
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_MAX_START_FUNCTION_LIMIT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7179
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_STOP_FUNCTION             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7180
#define RSFPL_ATTR_SWEEP_LIST_POWER_CLASS_LIMIT_RELATIVE_MAX_STOP_FUNCTION_LIMIT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7181
#define RSFPL_ATTR_REFERENCE_RANGE_POWER_REFERENCE_TYPE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7182
#define RSFPL_ATTR_REFERENCE_RANGE_TX_BANDWIDTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7183
#define RSFPL_ATTR_REFERENCE_RANGE_RRC_FILTER_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7184
#define RSFPL_ATTR_REFERENCE_RANGE_ALPHA                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7185
#define RSFPL_ATTR_REFERENCE_RANGE_NUMBER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7186
#define RSFPL_ATTR_SEM_LIST_EVALUATION_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7187
#define RSFPL_ATTR_SEM_LIST_EVALUATION_ALL_DETECTED_PEAKS_INCLUDED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7188
#define RSFPL_ATTR_SEM_LIST_EVALUATION_THRESHOLD                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7189
#define RSFPL_ATTR_SEM_LIST_EVALUATION_PEAK_LABELS_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7190
#define RSFPL_ATTR_SEM_LIST_EVALUATION_INITIATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7191
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_INITIATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7193
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGES_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7194
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_NUMBER_OF_RANGES                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7195
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_BREAK_MEASUREMENT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7713
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_ADJUST_X_AXIS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7714
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_DELETE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7196
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_START                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7197
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_STOP                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7198
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_FILTER_TYPE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7199
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_RBW                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7200
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_VBW                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7201
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_TIME_AUTO_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7202
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_TIME_MANUAL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7203
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_DETECTOR                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7204
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_REFERENCE_LEVEL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7205
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ATTENUATION_AUTO_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7206
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ATTENUATION_MANUAL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7207
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_PREAMPLIFIER_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7208
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_PREAMPLIFICATION_LEVEL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7209
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_SWEEP_POINTS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7210
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_TRANSDUCER_FILE_NAME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7211
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ABS_LIMIT_START                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7212
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_RANGE_ABS_LIMIT_STOP                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7213
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7214
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_NUMBER_OF_PEAKS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7215
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_ALL_DETECTED_PEAKS_INCLUDED  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7216
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_THRESHOLD                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7217
#define RSFPL_ATTR_SPURIOUS_EMISSIONS_LIST_EVALUATION_PEAK_LABELS_ENABLED          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7218
#define RSFPL_ATTR_SIGNAL_STATISTIC_APD_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7715
#define RSFPL_ATTR_SIGNAL_STATISTIC_CCDF_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7716
#define RSFPL_ATTR_SIGNAL_STATISTIC_NUMBER_OF_SAMPLES                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7717
#define RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_X_REF_LEVEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7718
#define RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_X_RANGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7719
#define RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_Y_UNIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7720
#define RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_Y_MINIMUM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7721
#define RSFPL_ATTR_SIGNAL_STATISTIC_SCALING_Y_MAXIMUM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7722
#define RSFPL_ATTR_SIGNAL_STATISTIC_ADJUST_SETTINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7723
#define RSFPL_ATTR_SIGNAL_STATISTIC_DEFAULT_SETTING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7724
#define RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7727
#define RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_START                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7728
#define RSFPL_ATTR_SIGNAL_STATISTIC_GATE_RANGE_STOP                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7729
#define RSFPL_ATTR_SIGNAL_STATISTIC_GATE_PERIOD                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7730
#define RSFPL_ATTR_SIGNAL_STATISTIC_GATE_COMMENT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7731
#define RSFPL_ATTR_TIME_DOMAIN_POWER_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7219
#define RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGING_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7220
#define RSFPL_ATTR_TIME_DOMAIN_POWER_ALL_OFF                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7221
#define RSFPL_ATTR_TIME_DOMAIN_POWER_PEAK_HOLD_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7222
#define RSFPL_ATTR_TIME_DOMAIN_POWER_RMS_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7223
#define RSFPL_ATTR_TIME_DOMAIN_POWER_RMS_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7224
#define RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_RMS_RESULT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7225
#define RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_RMS_RESULT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7226
#define RSFPL_ATTR_TIME_DOMAIN_POWER_MEAN_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7227
#define RSFPL_ATTR_TIME_DOMAIN_POWER_MEAN_RESULT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7228
#define RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_MEAN_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7229
#define RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_MEAN_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7230
#define RSFPL_ATTR_TIME_DOMAIN_POWER_POSITIVE_PEAK_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7231
#define RSFPL_ATTR_TIME_DOMAIN_POWER_POSITIVE_PEAK_RESULT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7232
#define RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_POSITIVE_PEAK_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7233
#define RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_POSITIVE_PEAK_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7234
#define RSFPL_ATTR_TIME_DOMAIN_POWER_STANDARD_DEVIATION_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7235
#define RSFPL_ATTR_TIME_DOMAIN_POWER_STANDARD_DEVIATION_RESULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7236
#define RSFPL_ATTR_TIME_DOMAIN_POWER_AVERAGE_STANDARD_DEVIATION_RESULT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7237
#define RSFPL_ATTR_TIME_DOMAIN_POWER_MAXIMUM_STANDARD_DEVIATION_RESULT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7238
#define RSFPL_ATTR_HARMONIC_DISTORTION_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7239
#define RSFPL_ATTR_HARMONIC_DISTORTION_NUMBER_OF_HARMONICS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7240
#define RSFPL_ATTR_HARMONIC_DISTORTION_RBW_AUTO_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7241
#define RSFPL_ATTR_HARMONIC_DISTORTION_INITIATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7242
#define RSFPL_ATTR_THIRD_ORDER_INTERCEPT_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7245
#define RSFPL_ATTR_THIRD_ORDER_INTERCEPT_SEARCH_SIGNAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7246
#define RSFPL_ATTR_THIRD_ORDER_INTERCEPT_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7247
#define RSFPL_ATTR_THIRD_ORDER_INTERCEPT_RESULT_MAXIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8152
#define RSFPL_ATTR_THIRD_ORDER_INTERCEPT_RESULT_MINIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8153
#define RSFPL_ATTR_AM_MODULATION_DEPTH_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7248
#define RSFPL_ATTR_AM_MODULATION_DEPTH_SEARCH_SIGNAL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7249
#define RSFPL_ATTR_AM_MODULATION_DEPTH_RESULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7250
#define RSFPL_ATTR_DISABLE_LIST_EVALUATION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7256
#define RSFPL_ATTR_PULSE_POWER_MEASUREMENT_FILTER_TYPE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7257
#define RSFPL_ATTR_PULSE_POWER_MEASUREMENT_MINIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7258
#define RSFPL_ATTR_DISPLAY_CHANNEL_BAR_INFORMATION_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7265
#define RSFPL_ATTR_DISPLAY_X_AXIS_LABEL_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7266
#define RSFPL_ATTR_DISPLAY_STATUS_BAR_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7267
#define RSFPL_ATTR_DISPLAY_SOFTKEY_BAR_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7268
#define RSFPL_ATTR_DISPLAY_TOOLBAR_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7269
#define RSFPL_ATTR_DISPLAY_TOUCH_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7270
#define RSFPL_ATTR_DISPLAY_DATE_AND_TIME_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7271
#define RSFPL_ATTR_DISPLAY_FRONT_PANEL_KEYS_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7272
#define RSFPL_ATTR_DISPLAY_BRIGHTNESS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7273
#define RSFPL_ATTR_DISPLAY_DATE_AND_TIME_FORMAT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7274
#define RSFPL_ATTR_DISPLAY_TEMPORARY_SIZE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7516
#define RSFPL_ATTR_DISPLAY_FORMAT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7732
#define RSFPL_ATTR_COLORS_AND_THEMES_CATALOG                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7275
#define RSFPL_ATTR_COLORS_AND_THEMES_SELECTED_THEME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7276
#define RSFPL_ATTR_LAYOUT_REMOVE_WINDOW                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7282
#define RSFPL_ATTR_LAYOUT_WINDOW_LIST                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7283
#define RSFPL_ATTR_DISPLAY_WINDOW_SET_FOCUS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8156
#define RSFPL_ATTR_DISPLAY_SUBWINDOW_SET_FOCUS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8157
#define RSFPL_ATTR_DISPLAY_SINGLE_ZOOM_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7286
#define RSFPL_ATTR_SUBWINDOW_DISPLAY_SINGLE_ZOOM_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7733
#define RSFPL_ATTR_MULTIPLE_ZOOM_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7288
#define RSFPL_ATTR_SUBWINDOW_MULTIPLE_ZOOM_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7735
#define RSFPL_ATTR_LAYOUT_REPLACE_WINDOW                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7291
#define RSFPL_ATTR_LAYOUT_WINDOW_REMOVE_WINDOW                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7293
#define RSFPL_ATTR_LAYOUT_WINDOW_IDENTIFICATION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7294
#define RSFPL_ATTR_LAYOUT_WINDOW_TYPE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8351
#define RSFPL_ATTR_FREQUENCY_DISPLAY_LINE_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7295
#define RSFPL_ATTR_FREQUENCY_DISPLAY_LINE_POSITION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7296
#define RSFPL_ATTR_TIME_DISPLAY_LINE_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7297
#define RSFPL_ATTR_TIME_DISPLAY_LINE_POSITION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7298
#define RSFPL_ATTR_POWER_DISPLAY_LINE_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8158
#define RSFPL_ATTR_POWER_DISPLAY_LINE_POSITION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8159
#define RSFPL_ATTR_HORIZONTAL_DISPLAY_LINE_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7299
#define RSFPL_ATTR_HORIZONTAL_DISPLAY_LINE_POSITION                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7300
#define RSFPL_ATTR_LIMIT_NAMES_OF_ACTIVE_LIMIT_LINES                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7301
#define RSFPL_ATTR_LIMIT_DELETE_RESULTS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7302
#define RSFPL_ATTR_LIMIT_LINE_CHECK_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7305
#define RSFPL_ATTR_LIMIT_LINE_UNITS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7306
#define RSFPL_ATTR_LIMIT_LINE_COMMENT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7307
#define RSFPL_ATTR_LIMIT_LINE_COPY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7308
#define RSFPL_ATTR_LIMIT_LINE_NAME                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7310
#define RSFPL_ATTR_LIMIT_LINE_DELETE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7311
#define RSFPL_ATTR_LIMIT_LINE_CONTROL_DOMAIN                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7312
#define RSFPL_ATTR_LIMIT_LINE_CONTROL_SPACING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7313
#define RSFPL_ATTR_LIMIT_LINE_CONTROL_SCALING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7314
#define RSFPL_ATTR_LIMIT_LINE_CONTROL_OFFSET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7315
#define RSFPL_ATTR_LIMIT_LINE_SHIFT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7316
#define RSFPL_ATTR_LIMIT_LINE_LOWER_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7318
#define RSFPL_ATTR_LIMIT_LINE_LOWER_SCALING                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7319
#define RSFPL_ATTR_LIMIT_LINE_LOWER_SPACING                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7320
#define RSFPL_ATTR_LIMIT_LINE_LOWER_MARGIN                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7321
#define RSFPL_ATTR_LIMIT_LINE_LOWER_THRESHOLD                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7322
#define RSFPL_ATTR_LIMIT_LINE_LOWER_OFFSET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7323
#define RSFPL_ATTR_LIMIT_LINE_LOWER_SHIFT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7324
#define RSFPL_ATTR_LIMIT_LINE_UPPER_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7326
#define RSFPL_ATTR_LIMIT_LINE_UPPER_SCALING                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7327
#define RSFPL_ATTR_LIMIT_LINE_UPPER_SPACING                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7328
#define RSFPL_ATTR_LIMIT_LINE_UPPER_MARGIN                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7329
#define RSFPL_ATTR_LIMIT_LINE_UPPER_THRESHOLD                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7330
#define RSFPL_ATTR_LIMIT_LINE_UPPER_OFFSET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7331
#define RSFPL_ATTR_LIMIT_LINE_UPPER_SHIFT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7332
#define RSFPL_ATTR_LIMIT_LINES_ASSIGN_TRACE_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7334
#define RSFPL_ATTR_LIMIT_LINE_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7335
#define RSFPL_ATTR_LOAD_LIMIT_LINE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7336
#define RSFPL_ATTR_IQ_ANALYZER_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7517
#define RSFPL_ATTR_IQ_ANALYZER_EVALUATION_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7518
#define RSFPL_ATTR_IQ_ANALYZER_SAMPLE_RATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7520
#define RSFPL_ATTR_IQ_ANALYZER_ANALYSIS_BANDWIDTH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7521
#define RSFPL_ATTR_IQ_ANALYZER_SWAP_IQ_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7522
#define RSFPL_ATTR_IQ_ANALYZER_RECORD_LENGTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7523
#define RSFPL_ATTR_IQ_ANALYZER_RBW_MODE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7524
#define RSFPL_ATTR_IQ_ANALYZER_MANUAL_RBW                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7525
#define RSFPL_ATTR_IQ_ANALYZER_TRANSFORMATION_ALGORITHM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7526
#define RSFPL_ATTR_IQ_ANALYZER_FFT_LENGTH                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7527
#define RSFPL_ATTR_IQ_ANALYZER_WINDOW_FUNCTION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7528
#define RSFPL_ATTR_IQ_ANALYZER_WINDOW_OVERLAP                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7529
#define RSFPL_ATTR_IQ_ANALYZER_WINDOW_LENGTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7530
#define RSFPL_ATTR_IQ_ANALYZER_TRIGGER_OFFSET                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7531
#define RSFPL_ATTR_IQ_ANALYZER_BRANCH_FOR_PEAKSEARCH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7532
#define RSFPL_ATTR_IQ_ANALYZER_GATING_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7533
#define RSFPL_ATTR_IQ_ANALYZER_GATING_MODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7534
#define RSFPL_ATTR_IQ_ANALYZER_GATING_GAP                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7535
#define RSFPL_ATTR_IQ_ANALYZER_GATING_LENGTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7536
#define RSFPL_ATTR_IQ_ANALYZER_GATING_NUMBER_OF_GATE_PERIODS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7537
#define RSFPL_ATTR_IQ_ANALYZER_LAYOUT_REPLACE_WINDOW                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7541
#define RSFPL_ATTR_IQ_ANALYZER_IQ_DATA_COMMENT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7546
#define RSFPL_ATTR_IQ_ANALYZER_SELECT_IQ_DATA_FILE_RAWSTRING                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8160
#define RSFPL_ATTR_NOISE_COMMON_NOISE_SOURCE_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7551
#define RSFPL_ATTR_NOISE_ROOM_TEMPERATURE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7552
#define RSFPL_ATTR_NOISE_CALIBRATION_NOISE_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7553
#define RSFPL_ATTR_NOISE_CALIBRATION_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7554
#define RSFPL_ATTR_NOISE_CALIBRATION_CONSTANT_ENR                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7555
#define RSFPL_ATTR_NOISE_CALIBRATION_SELECTED_TABLE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7556
#define RSFPL_ATTR_NOISE_CALIBRATION_SMART_SERIAL_NUMBER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8352
#define RSFPL_ATTR_NOISE_CALIBRATION_SPOT_COLD_TEMPERATURE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7557
#define RSFPL_ATTR_NOISE_CALIBRATION_SPOT_HOT_TEMPERATURE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7558
#define RSFPL_ATTR_NOISE_MEASUREMENT_NOISE_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7559
#define RSFPL_ATTR_NOISE_SOURCE_MEASUREMENT_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7560
#define RSFPL_ATTR_NOISE_MEASUREMENT_CONSTANT_ENR                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7561
#define RSFPL_ATTR_NOISE_MEASUREMENT_SMART_SERIAL_NUMBER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8353
#define RSFPL_ATTR_NOISE_MEASUREMENT_SPOT_COLD_TEMPERATURE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7562
#define RSFPL_ATTR_NOISE_MEASUREMENT_SPOT_HOT_TEMPERATURE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7563
#define RSFPL_ATTR_NOISE_MEASUREMENT_SELECTED_TABLE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7564
#define RSFPL_ATTR_NOISE_MEASUREMENT_DIODE_TABLE_LIST                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7566
#define RSFPL_ATTR_NOISE_MEASUREMENT_DIODE_TABLE_DELETE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7567
#define RSFPL_ATTR_NOISE_MEASUREMENT_TEMPERATURE_TABLE_LIST                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7570
#define RSFPL_ATTR_NOISE_MEASUREMENT_TEMPERATURE_TABLE_DELETE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7571
#define RSFPL_ATTR_NOISE_INPUT_LOSS_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7573
#define RSFPL_ATTR_NOISE_INPUT_LOSS_CONSTANT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7574
#define RSFPL_ATTR_NOISE_INPUT_LOSS_TEMPERATURE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7737
#define RSFPL_ATTR_NOISE_INPUT_LOSS_SELECTED_TABLE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7575
#define RSFPL_ATTR_NOISE_INPUT_LOSS_DELETE_TABLE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7577
#define RSFPL_ATTR_NOISE_INPUT_LOSS_TABLE_LIST                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7738
#define RSFPL_ATTR_NOISE_OUTPUT_LOSS_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7579
#define RSFPL_ATTR_NOISE_OUTPUT_LOSS_CONSTANT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7580
#define RSFPL_ATTR_NOISE_OUTPUT_LOSS_TEMPERATURE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7739
#define RSFPL_ATTR_NOISE_OUTPUT_LOSS_SELECTED_TABLE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7581
#define RSFPL_ATTR_NOISE_OUTPUT_LOSS_DELETE_TABLE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7583
#define RSFPL_ATTR_NOISE_OUTPUT_LOSS_TABLE_LIST                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7740
#define RSFPL_ATTR_NOISE_CALIBRATION_LOSS_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7741
#define RSFPL_ATTR_NOISE_CALIBRATION_LOSS_CONSTANT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7742
#define RSFPL_ATTR_NOISE_CALIBRATION_LOSS_TEMPERATURE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7743
#define RSFPL_ATTR_NOISE_CALIBRATION_LOSS_SELECTED_TABLE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7744
#define RSFPL_ATTR_NOISE_CALIBRATION_LOSS_DELETE_TABLE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7746
#define RSFPL_ATTR_NOISE_CALIBRATION_LOSS_TABLE_LIST                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7748
#define RSFPL_ATTR_NOISE_DUT_TYPE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7585
#define RSFPL_ATTR_NOISE_DUT_LO_TYPE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7749
#define RSFPL_ATTR_NOISE_DUT_LO_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7586
#define RSFPL_ATTR_NOISE_DUT_IMAGE_REJECTION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7587
#define RSFPL_ATTR_NOISE_MEASUREMENT_POINTS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7751
#define RSFPL_ATTR_NOISE_SINGLE_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7588
#define RSFPL_ATTR_NOISE_SINGLE_FREQUENCY_COUPLING_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7589
#define RSFPL_ATTR_NOISE_SINGLE_FREQUENCY_CONTINUOUS_SWEEP                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7590
#define RSFPL_ATTR_NOISE_SINGLE_FREQUENCY_SINGLE_SWEEP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7591
#define RSFPL_ATTR_NOISE_FREQUENCY_STEPSIZE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7592
#define RSFPL_ATTR_NOISE_LIST_MEASUREMENT_CONTINUOUS_SWEEP                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7594
#define RSFPL_ATTR_NOISE_LIST_MEASUREMENT_SINGLE_SWEEP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7595
#define RSFPL_ATTR_NOISE_FREQUENCY_QUERY_TABLE_DATA_RAW                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8162
#define RSFPL_ATTR_NOISE_ANALYZER_CALIBRATION_MEASUREMENT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7596
#define RSFPL_ATTR_NOISE_ANALYZER_CALIBRATION_DATA_INCLUDED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7597
#define RSFPL_ATTR_NOISE_ANALYZER_AUTO_REFERENCE_LEVEL_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7598
#define RSFPL_ATTR_NOISE_ANALYZER_EXPECTED_DUT_GAIN                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7599
#define RSFPL_ATTR_NOISE_ANALYZER_SETTLING_TIME                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7600
#define RSFPL_ATTR_NOISE_LIMIT_LINE_RESULT_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7601
#define RSFPL_ATTR_NOISE_DISPLAY_AUTOMATIC_SCALING_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7603
#define RSFPL_ATTR_NOISE_DISPLAY_FREQUENCY_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7604
#define RSFPL_ATTR_NOISE_DISPLAY_X_AXIS_SYMBOLS_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7605
#define RSFPL_ATTR_NOISE_DISPLAY_X_AXIS_BOTTOM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7606
#define RSFPL_ATTR_NOISE_DISPLAY_X_AXIS_TOP                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7607
#define RSFPL_ATTR_NOISE_LAYOUT_REPLACE_WINDOW                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7611
#define RSFPL_ATTR_NOISE_MEASUREMENT_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7613
#define RSFPL_ATTR_NOISE_MEASUREMENT_TYPE_OF_POWER_MEASUREMENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7614
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_RESULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8164
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_OUTPUT_MATCH_VSWR          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8165
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_OUTPUT_MATCH_RETURN_LOSS   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8166
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_ENR_UNCERTAINTY            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8167
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_RESISTOR_COLD_UNCERTAINTY  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8168
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_MEASUREMENT_RESISTOR_HOT_UNCERTAINTY   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8169
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_COMMON_SOURCE_FOR_MEAS_AND_CAL_ENABLED RS_SPECIFIC_PUBLIC_ATTR_BASE + 8170
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_OUTPUT_MATCH_VSWR          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8171
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_OUTPUT_MATCH_RETURN_LOSS   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8172
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_ENR_UNCERTAINTY            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8173
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_RESISTOR_COLD_UNCERTAINTY  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8174
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_CALIBRATION_RESISTOR_HOT_UNCERTAINTY   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8175
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_INPUT_MATCH_VSWR                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8176
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_INPUT_MATCH_RETURN_LOSS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8177
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_OUTPUT_MATCH_VSWR                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8178
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_OUTPUT_MATCH_RETURN_LOSS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8179
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_USE_MEASUREMENT_VALUES_ENABLED     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8180
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_NOISE_FIGURE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8181
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_GAIN                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8182
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_DUT_FREQUENCY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8183
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_ANALYZER_GAIN_UNCERTAINTY              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8184
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_ANALYZER_NOISE_FIGURE_UNCERTAINTY      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8185
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8186
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_NOISE_FIGURE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8187
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_GAIN                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8188
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_VSWR                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8189
#define RSFPL_ATTR_NOISE_FIGURE_UNCERTAINTY_PREAMP_RETURN_LOSS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8190
#define RSFPL_ATTR_ANALOG_DEMODULATION_SCALLING_ANGLE_UNIT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8355
#define RSFPL_ATTR_ANALOG_DEMODULATION_UNIT_THD                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8356
#define RSFPL_ATTR_ANALOG_DEMODULATION_RF_INPUT_YIG_FILTER                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8357
#define RSFPL_ATTR_ADEM_LOAD_CONFIGURATION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7618
#define RSFPL_ATTR_ADEM_SAVE_CONFIGURATION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7620
#define RSFPL_ATTR_ADEM_RESTORE_CONFIGURATIONS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7622
#define RSFPL_ATTR_ADEM_TRIGGER_AM_LEVEL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7623
#define RSFPL_ATTR_ADEM_TRIGGER_RF_LEVEL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7624
#define RSFPL_ATTR_ADEM_TRIGGER_FM_LEVEL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7625
#define RSFPL_ATTR_ADEM_TRIGGER_PM_LEVEL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7626
#define RSFPL_ATTR_ADEM_BANDWIDTH                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7627
#define RSFPL_ATTR_ADEM_FILTER_TYPE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7628
#define RSFPL_ATTR_ADEM_MEASUREMENT_TIME                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7629
#define RSFPL_ATTR_ADEM_SAMPLE_RATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7630
#define RSFPL_ATTR_ADEM_RECORD_LENGTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7631
#define RSFPL_ATTR_ADEM_UNIT_ANGLE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7633
#define RSFPL_ATTR_ADEM_UNIT_THD                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7634
#define RSFPL_ATTR_ADEM_SQUELCH_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7635
#define RSFPL_ATTR_ADEM_SQUELCH_LEVEL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7636
#define RSFPL_ATTR_ADEM_ZERO_PHASE_REFERENCE_POSITION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7637
#define RSFPL_ATTR_ADEM_ZERO_PHASE_REFERENCE_POSITION_MODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8358
#define RSFPL_ATTR_ADEM_AF_COUPLING                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7638
#define RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7639
#define RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_START                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7640
#define RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_LENGTH_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7641
#define RSFPL_ATTR_ADEM_TIME_DOMAIN_ZOOM_LENGTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7642
#define RSFPL_ATTR_ADEM_AF_CENTER                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7643
#define RSFPL_ATTR_ADEM_AF_SPAN                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7644
#define RSFPL_ATTR_ADEM_AF_SPAN_FULL                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7645
#define RSFPL_ATTR_ADEM_AF_START                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7646
#define RSFPL_ATTR_ADEM_AF_STOP                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7647
#define RSFPL_ATTR_ADEM_AF_AUTO_SCALE_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7648
#define RSFPL_ATTR_ADEM_RF_SPAN                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7649
#define RSFPL_ATTR_ADEM_RF_SPAN_ZOOM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7650
#define RSFPL_ATTR_ADEM_AF_FILTER_ALL_OFF                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7651
#define RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7652
#define RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_TYPE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7653
#define RSFPL_ATTR_ADEM_AF_HIGH_PASS_FILTER_CUTOFF_FREQUENCY                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7654
#define RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7655
#define RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_CUTOFF_FREQUENCY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7656
#define RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_TYPE_RELATIVE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7657
#define RSFPL_ATTR_ADEM_AF_LOW_PASS_FILTER_TYPE_ABSOLUTE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7658
#define RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_CCITT_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7659
#define RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_UNWEIGHTED_CCIR_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7660
#define RSFPL_ATTR_ADEM_AF_WEIGHTED_FILTER_WEIGHTED_CCIR_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7661
#define RSFPL_ATTR_ADEM_AF_WEIGHTED_A_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7662
#define RSFPL_ATTR_ADEM_AF_DEEMPHASIS_FILTER_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7663
#define RSFPL_ATTR_ADEM_AF_DEEMPHASIS_FILTER_TYPE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7664
#define RSFPL_ATTR_ADEM_RELATIVE_DEMODULATION_RESULTS_UNIT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7665
#define RSFPL_ATTR_ADEM_AM_DEMODULATION_SET_REFERENCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7666
#define RSFPL_ATTR_ADEM_AM_RELATIVE_DEMODULATION_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7667
#define RSFPL_ATTR_ADEM_AM_RELATIVE_DEMODULATION_RESULT_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7668
#define RSFPL_ATTR_ADEM_AM_RELATIVE_DEMODULATION_REFERENCE_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7669
#define RSFPL_ATTR_ADEM_FM_DEMODULATION_SET_REFERENCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7670
#define RSFPL_ATTR_ADEM_FM_RELATIVE_DEMODULATION_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7671
#define RSFPL_ATTR_ADEM_FM_RELATIVE_DEMODULATION_RESULT_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7672
#define RSFPL_ATTR_ADEM_FM_RELATIVE_DEMODULATION_REFERENCE_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7673
#define RSFPL_ATTR_ADEM_PM_DEMODULATION_SET_REFERENCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7674
#define RSFPL_ATTR_ADEM_PM_RELATIVE_DEMODULATION_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7675
#define RSFPL_ATTR_ADEM_PM_RELATIVE_DEMODULATION_RESULT_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7676
#define RSFPL_ATTR_ADEM_PM_RELATIVE_DEMODULATION_REFERENCE_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7677
#define RSFPL_ATTR_ADEM_LAYOUT_REPLACE_WINDOW                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7681
#define RSFPL_ATTR_ADEM_RESULT_MODULATION_FREQUENCY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7688
#define RSFPL_ATTR_ADEM_RESULT_CARRIER_POWER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7689
#define RSFPL_ATTR_ADEM_SELECT_TRACE_FOR_RESULT_SUMMARY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7690
#define RSFPL_ATTR_ADEM_RESULT_THD                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7691
#define RSFPL_ATTR_ADEM_RESULT_SINAD                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7692
#define RSFPL_ATTR_ADEM_RESULT_CARRIER_OFFSET                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7693
#define RSFPL_ATTR_ADEM_TRACE_RESULT_MODULATION_FREQUENCY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7754
#define RSFPL_ATTR_ADEM_TRACE_RESULT_CARRIER_POWER                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7755
#define RSFPL_ATTR_ADEM_TRACE_RESULT_DISTORTION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8359
#define RSFPL_ATTR_ANALOG_DEMODULATION_RESULT_RANGE_RESULT_TYPE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8360
#define RSFPL_ATTR_EMI_MEASUREMENT_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7756
#define RSFPL_ATTR_EMI_MEASUREMENT_DWELL_TIME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7757
#define RSFPL_ATTR_EMI_MEASUREMENT_MARKER_PEAK_SEARCH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7758
#define RSFPL_ATTR_EMI_MEASUREMENT_MARKER_DETECTOR                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7759
#define RSFPL_ATTR_EMI_MEASUREMENT_MARKER_RESULT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7760
#define RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_CONDITION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7761
#define RSFPL_ATTR_EMI_MEASUREMENT_MARKER_LIMIT_VERTICAL_DISTANCE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7762
#define RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_PEAK_SEARCH                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7763
#define RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_DETECTOR                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7764
#define RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7765
#define RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_CONDITION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7766
#define RSFPL_ATTR_EMI_MEASUREMENT_DELTA_MARKER_LIMIT_VERTICAL_DISTANCE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7767
#define RSFPL_ATTR_EMI_LISN_V_NETWORK_TYPE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7768
#define RSFPL_ATTR_EMI_LISN_V_NETWORK_HIGH_PASS_FILTER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7769
#define RSFPL_ATTR_EMI_LISN_V_NETWORK_PHASE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7770
#define RSFPL_ATTR_VSA_MODE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7771
#define RSFPL_ATTR_VSA_SELECT_RESULT_RANGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7772
#define RSFPL_ATTR_VSA_RF_INPUT_COUPLING                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7773
#define RSFPL_ATTR_VSA_RF_INPUT_YIG_FILTER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7774
#define RSFPL_ATTR_VSA_RF_INPUT_PAD_TYPE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7775
#define RSFPL_ATTR_VSA_OUTPUT_PRESET_REFERENCE_LEVEL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7776
#define RSFPL_ATTR_VSA_OUTPUT_OPIMIZATION_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7777
#define RSFPL_ATTR_VSA_PREAMPLIFIER_COMPENSATION_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7778
#define RSFPL_ATTR_VSA_ATTENUATION_ELECTRONIC_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7779
#define RSFPL_ATTR_VSA_ATTENUATION_ELECTRONIC                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7780
#define RSFPL_ATTR_VSA_SCALLING_QUANTIZE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7781
#define RSFPL_ATTR_VSA_SCALLING_ANGLE_UNIT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7782
#define RSFPL_ATTR_VSA_SCALLING_TIME_UNIT_X                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7783
#define RSFPL_ATTR_VSA_SCALLING_TIME_UNIT_Y                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7784
#define RSFPL_ATTR_VSA_SCALLING_RANGE_PER_DIVISION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7785
#define RSFPL_ATTR_VSA_SCALLING_X_AXIS_REFERENCE_VALUE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7786
#define RSFPL_ATTR_VSA_RESTORES_FACTORY_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7787
#define RSFPL_ATTR_VSA_LOAD_STANDARD                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7788
#define RSFPL_ATTR_VSA_SAVE_STANDARD                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7790
#define RSFPL_ATTR_VSA_DELETE_STANDARD                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7792
#define RSFPL_ATTR_VSA_CONFIGURATION_COMMENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7793
#define RSFPL_ATTR_VSA_STANDARD_PRESET                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7794
#define RSFPL_ATTR_VSA_MODULATION_TYPE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7795
#define RSFPL_ATTR_VSA_MODULATION_FSK_DEVIATION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7796
#define RSFPL_ATTR_VSA_MODULATION_FSK_DEVIATION_ABSOLUTE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7797
#define RSFPL_ATTR_VSA_MODULATION_FSK_N_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7798
#define RSFPL_ATTR_VSA_MODULATION_APSK_N_STATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7799
#define RSFPL_ATTR_VSA_MODULATION_ASK_N_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7800
#define RSFPL_ATTR_VSA_MODULATION_MSK_FORMAT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7801
#define RSFPL_ATTR_VSA_MODULATION_PSK_FORMAT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7802
#define RSFPL_ATTR_VSA_MODULATION_PSK_N_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7803
#define RSFPL_ATTR_VSA_MODULATION_QAM_FORMAT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7804
#define RSFPL_ATTR_VSA_MODULATION_QAM_N_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7805
#define RSFPL_ATTR_VSA_MODULATION_QPSK_FORMAT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7806
#define RSFPL_ATTR_VSA_MODULATION_LOAD_USER_DEFINED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7807
#define RSFPL_ATTR_VSA_MODULATION_FILTER_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7809
#define RSFPL_ATTR_VSA_MODULATION_FILTER_ALPHA_BT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7810
#define RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_SELECT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7811
#define RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_SELECT_USER_DEFINED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7812
#define RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_OFF                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7813
#define RSFPL_ATTR_VSA_MODULATION_TRANSMIT_FILTER_ALPHA_BT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7814
#define RSFPL_ATTR_VSA_MODULATION_MAPPING_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7815
#define RSFPL_ATTR_VSA_MODULATION_MAPPING                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7816
#define RSFPL_ATTR_VSA_MODULATION_SYMBOL_RATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7817
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_BURST_MAX_LENGTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7818
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_BURST_MIN_LENGTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7819
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_RUN_OUT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7820
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_RUN_IN                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7821
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_PATTERN_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7823
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_PATTERN_OFFSET_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7824
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_PATTERN_OFFSET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7825
#define RSFPL_ATTR_VSA_SIGNAL_STRUCTURE_TYPE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7826
#define RSFPL_ATTR_VSA_FRAME_FORMAT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7827
#define RSFPL_ATTR_VSA_FRAME_EDIT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7828
#define RSFPL_ATTR_VSA_FRAME_SAVE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7830
#define RSFPL_ATTR_VSA_FRAME_LOAD                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7832
#define RSFPL_ATTR_VSA_FRAME_NEXT_BOOSTING                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7835
#define RSFPL_ATTR_VSA_FRAME_NEXT_MODULATION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7836
#define RSFPL_ATTR_VSA_FRAME_PREVIOUS_BOOSTING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7837
#define RSFPL_ATTR_VSA_FRAME_PREVIOUS_MODULATION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7838
#define RSFPL_ATTR_VSA_FRAME_DESCRIPTION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7839
#define RSFPL_ATTR_VSA_FRAME_MODE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7840
#define RSFPL_ATTR_VSA_FRAME_MAPPING_CATALOG                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7841
#define RSFPL_ATTR_VSA_FRAME_MAPPING                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7842
#define RSFPL_ATTR_VSA_FRAME_APSK_N_STATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8191
#define RSFPL_ATTR_VSA_FRAME_ASK_N_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8192
#define RSFPL_ATTR_VSA_FRAME_PSK_FORMAT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7843
#define RSFPL_ATTR_VSA_FRAME_PSK_N_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7844
#define RSFPL_ATTR_VSA_FRAME_QAM_FORMAT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7845
#define RSFPL_ATTR_VSA_FRAME_QAM_N_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7846
#define RSFPL_ATTR_VSA_FRAME_QPSK_FORMAT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7847
#define RSFPL_ATTR_VSA_FRAME_USER_DEFINED_MODULATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7848
#define RSFPL_ATTR_VSA_FRAME_PATTERN_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7850
#define RSFPL_ATTR_VSA_KNOWN_DATA_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7851
#define RSFPL_ATTR_VSA_KNOWN_DATA_SOURCE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7852
#define RSFPL_ATTR_VSA_KNOWN_DATA_LOAD_DATA_FILE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7853
#define RSFPL_ATTR_VSA_KNOWN_DATA_FEEDBACK_AUTO                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7855
#define RSFPL_ATTR_VSA_KNOWN_DATA_FEEDBACK_PRBS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7856
#define RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7857
#define RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_AUTO                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7858
#define RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_POLYNOMIAL_AUTO                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7859
#define RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_POLYNOMIAL_ORDER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7860
#define RSFPL_ATTR_VSA_KNOWN_DATA_PRBS_TYPE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7861
#define RSFPL_ATTR_VSA_SIGNAL_CAPTURE_SAMPLE_RATE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7862
#define RSFPL_ATTR_VSA_SIGNAL_CAPTURE_RECORD_LENGTH_AUTO                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7863
#define RSFPL_ATTR_VSA_SIGNAL_CAPTURE_RECORD_LENGTH                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7864
#define RSFPL_ATTR_VSA_SIGNAL_CAPTURE_RECORD_LENGTH_SYMBOLS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7865
#define RSFPL_ATTR_QUERY_SIGNAL_CAPTURE_IQ_BANDWIDTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7866
#define RSFPL_ATTR_VSA_DEMODULATION_NORMALIZE_EVM_TO                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7867
#define RSFPL_ATTR_VSA_DEMODULATION_OFFSET_EVM_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7868
#define RSFPL_ATTR_VSA_DEMODULATION_ESTIMATION_POINTS_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7869
#define RSFPL_ATTR_VSA_DEMODULATION_ESTIMATION_POINTS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7870
#define RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7871
#define RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_FILTER_LENGTH                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7872
#define RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7873
#define RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_FILE_FORMAT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7874
#define RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_LOAD                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7875
#define RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_SAVE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7877
#define RSFPL_ATTR_VSA_DEMODULATION_EQUALIZER_RESET                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7879
#define RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7880
#define RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_AUTO                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7881
#define RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_SER_LEVEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7882
#define RSFPL_ATTR_VSA_DEMODULATION_FINE_SYNC_RESULT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7883
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_AMPLITUDE_DROOP_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7884
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_FSK                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7885
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_TRANSMIT_CHANNEL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7886
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_DEVIATION_ERROR                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7887
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_IQ_IMBALANCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7888
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_IQ_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7889
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_IQ_SKEW                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7890
#define RSFPL_ATTR_VSA_DEMODULATION_COMPENSATE_SYMBOL_CLOCK_ERROR                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7891
#define RSFPL_ATTR_VSA_DEMODULATION_OPTIMIZATION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7892
#define RSFPL_ATTR_VSA_DEMODULATION_COARSE_SYNCHRONIZATION_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7893
#define RSFPL_ATTR_VSA_DEMODULATION_COARSE_SYNCHRONIZATION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7894
#define RSFPL_ATTR_VSA_MEASUREMENT_FILTER_AUTO                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7895
#define RSFPL_ATTR_VSA_MEASUREMENT_FILTER_ALPHA_BT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7896
#define RSFPL_ATTR_VSA_MEASUREMENT_FILTER_OFF                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7897
#define RSFPL_ATTR_VSA_MEASUREMENT_FILTER_NAME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7898
#define RSFPL_ATTR_VSA_MEASUREMENT_FILTER_USER                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7899
#define RSFPL_ATTR_VSA_EVALUATION_RANGE_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7901
#define RSFPL_ATTR_VSA_EVALUATION_RANGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7902
#define RSFPL_ATTR_VSA_BURST_SEARCH_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7903
#define RSFPL_ATTR_VSA_BURST_SEARCH_AUTO                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7904
#define RSFPL_ATTR_VSA_BURST_SEARCH_CONFIGURE_AUTO                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7905
#define RSFPL_ATTR_VSA_BURST_SEARCH_MIN_GAP_LENGTH                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7906
#define RSFPL_ATTR_VSA_BURST_SEARCH_MEASURING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7907
#define RSFPL_ATTR_VSA_BURST_SEARCH_TOLERANCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7908
#define RSFPL_ATTR_VSA_PATTERN_SEARCH_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7909
#define RSFPL_ATTR_VSA_PATTERN_SEARCH_IQ_CORRELATION_THRESHOLD                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7910
#define RSFPL_ATTR_VSA_PATTERN_SEARCH_AUTO                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7911
#define RSFPL_ATTR_VSA_PATTERN_SEARCH_IQ_CORRELATION_LEVEL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7913
#define RSFPL_ATTR_VSA_PATTERN_SEARCH_SYNC_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7914
#define RSFPL_ATTR_VSA_PATTERN_SEARCH_SELECT_FILE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7915
#define RSFPL_ATTR_VSA_PATTERN_NAME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7917
#define RSFPL_ATTR_VSA_PATTERN_COMMENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7918
#define RSFPL_ATTR_VSA_PATTERN_COPY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7919
#define RSFPL_ATTR_VSA_PATTERN_DELETE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7920
#define RSFPL_ATTR_VSA_PATTERN_ADD                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7921
#define RSFPL_ATTR_VSA_PATTERN_REMOVE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7922
#define RSFPL_ATTR_VSA_PATTERN_DATA                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7923
#define RSFPL_ATTR_VSA_PATTERN_N_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7924
#define RSFPL_ATTR_VSA_PATTERN_DESCRIPTION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7925
#define RSFPL_ATTR_VSA_RESULT_RANGE_RESULT_LENGTH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7926
#define RSFPL_ATTR_VSA_RESULT_RANGE_ACCORDING_TO_FRAME_STRUCTURE_FILE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7927
#define RSFPL_ATTR_VSA_RESULT_RANGE_RESULT_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7928
#define RSFPL_ATTR_VSA_RESULT_RANGE_TYPE_TRANSFORMATION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7929
#define RSFPL_ATTR_VSA_RESULT_RANGE_OVERSAMPLING                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7930
#define RSFPL_ATTR_VSA_RESULT_RANGE_INDIVIDUAL_RESULTS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7931
#define RSFPL_ATTR_VSA_RESULT_RANGE_PPS_MODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7932
#define RSFPL_ATTR_VSA_RESULT_RANGE_PPS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7933
#define RSFPL_ATTR_VSA_RESULT_RANGE_ALIGNMENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7934
#define RSFPL_ATTR_VSA_RESULT_RANGE_ALIGNMENT_OFFSET                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7935
#define RSFPL_ATTR_VSA_RESULT_RANGE_REFERENCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7936
#define RSFPL_ATTR_VSA_RESULT_RANGE_SYMBOL_NUMBER_AT_REFERENCE_START               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7937
#define RSFPL_ATTR_VSA_RESULT_RANGE_HIGHLIGHT_SYMBOLS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7938
#define RSFPL_ATTR_VSA_WINDOW_PRESET                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7939
#define RSFPL_ATTR_VSA_WINDOW_ADD_EVALUATION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7940
#define RSFPL_ATTR_VSA_WINDOW_FEED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7941
#define RSFPL_ATTR_VSA_WINDOW_MEASUREMENT_DIRTY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8193
#define RSFPL_ATTR_VSA_MEASUREMENT_REFRESH                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7944
#define RSFPL_ATTR_VSA_MEASUREMENT_START_SYMBOL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7945
#define RSFPL_ATTR_VSA_MEASUREMENT_START_SAMPLES                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7946
#define RSFPL_ATTR_VSA_RESULTS_CURRENT_BER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8194
#define RSFPL_ATTR_VSA_RESULTS_BURST_LENGTH                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7948
#define RSFPL_ATTR_VSA_TRACE_START                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7969
#define RSFPL_ATTR_VSA_TRACE_STOP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7970
#define RSFPL_ATTR_VSA_DSP_MARKER_TO_START_OF_SELECTED_RESULT_RANGE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8196
#define RSFPL_ATTR_VSA_DSP_DELTA_MARKER_TO_START_OF_SELECTED_RESULT_RANGE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8197
#define RSFPL_ATTR_VSA_DSP_CAPTURE_NUMBER_OF_BURSTS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8198
#define RSFPL_ATTR_VSA_DSP_CAPTURE_NUMBER_OF_PATTERNS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8199
#define RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_BURST_LENGTH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8200
#define RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_BURST_PRESENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8201
#define RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_BURST_START                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8202
#define RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_CONFIDENCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8203
#define RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_CORRECT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8204
#define RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_PRESENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8205
#define RSFPL_ATTR_VSA_DSP_CURRENT_RANGE_PATTERN_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8206
#define RSFPL_ATTR_VSA_TRACE_EVALUATION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7972
#define RSFPL_ATTR_VSA_TRACE_SYMBOL_SELECTION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7973
#define RSFPL_ATTR_VSA_DELTAMARKER_ABSOLUTE_PEAK                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7974
#define RSFPL_ATTR_VSA_MARKER_ABSOLUTE_PEAK                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7975
#define RSFPL_ATTR_VSA_DISPLAY_LINES_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7976
#define RSFPL_ATTR_VSA_DISPLAY_LINES_HORIZONTAL_LINE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7977
#define RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_HORIZONTAL_LINE_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7978
#define RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_HORIZONTAL_LINE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7979
#define RSFPL_ATTR_VSA_DISPLAY_LINE_ABSOLUTE_VERTICAL_LINE_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7980
#define RSFPL_ATTR_VSA_DISPLAY_LINE_ABSOLUTE_VERTICAL_LINE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7981
#define RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_VERTICAL_LINE_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7982
#define RSFPL_ATTR_VSA_DISPLAY_LINE_RELATIVE_VERTICAL_LINE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7983
#define RSFPL_ATTR_VSA_MACC_LIMIT_DEFAULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7984
#define RSFPL_ATTR_VSA_MACC_LIMIT_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7985
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_ENABLED          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7986
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_VALUE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7987
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_CURRENT_RESULT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7988
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_MEAN_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7989
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_MEAN_VALUE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7990
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_MEAN_RESULT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7991
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7992
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_VALUE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7993
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_RESULT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7994
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_CURRENT_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7995
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_CURRENT_VALUE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7996
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_CURRENT_RESULT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7997
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_MEAN_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7998
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_MEAN_VALUE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7999
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_MEAN_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8000
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_PEAK_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8001
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_PEAK_VALUE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8002
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_PEAK_PEAK_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8003
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_CURRENT_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8004
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_CURRENT_VALUE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8005
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_CURRENT_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8006
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_MEAN_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8007
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_MEAN_VALUE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8008
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_MEAN_RESULT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8009
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_PEAK_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8010
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_PEAK_VALUE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8011
#define RSFPL_ATTR_VSA_MACC_LIMIT_EVM_RMS_PEAK_RESULT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8012
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_CURRENT_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8013
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_CURRENT_VALUE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8014
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_CURRENT_RESULT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8015
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_MEAN_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8016
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_MEAN_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8017
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_MEAN_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8018
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_PEAK_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8019
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_PEAK_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8020
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_DEVIATION_PEAK_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8021
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_CURRENT_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8022
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_CURRENT_VALUE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8023
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_CURRENT_RESULT      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8024
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_MEAN_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8025
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_MEAN_VALUE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8026
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_MEAN_RESULT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8027
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_PEAK_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8028
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_PEAK_PEAK_VALUE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8029
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_PEAK_PEAK_RESULT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8030
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_CURRENT_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8031
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_CURRENT_VALUE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8032
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_RMS_CURRENT_RESULT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8033
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_MEAN_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8034
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_MEAN_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8035
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_RMS_MEAN_RESULT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8036
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_PEAK_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8037
#define RSFPL_ATTR_VSA_MACC_LIMIT_FREQUENCY_ERROR_RMS_PEAK_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8038
#define RSFPL_ATTR_VSA_MACC_LIMIT_CARRIER_FREQUENCY_ERROR_RMS_PEAK_RESULT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8039
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_CURRENT_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8040
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_CURRENT_VALUE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8041
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_CURRENT_RESULT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8042
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_MEAN_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8043
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_MEAN_VALUE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8044
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_MEAN_RESULT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8045
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_PEAK_ENABLED                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8046
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_PEAK_VALUE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8047
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_PEAK_PEAK_RESULT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8048
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_CURRENT_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8049
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_CURRENT_VALUE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8050
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_CURRENT_RESULT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8051
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_MEAN_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8052
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_MEAN_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8053
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_MEAN_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8054
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_PEAK_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8055
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_PEAK_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8056
#define RSFPL_ATTR_VSA_MACC_LIMIT_MAGNITUDE_ERROR_RMS_PEAK_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8057
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_CURRENT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8058
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_CURRENT_VALUE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8059
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_CURRENT_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8060
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_MEAN_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8061
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_MEAN_VALUE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8062
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_MEAN_RESULT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8063
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_PEAK_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8064
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_PEAK_VALUE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8065
#define RSFPL_ATTR_VSA_MACC_LIMIT_IQ_OFFSET_PEAK_RESULT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8066
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_CURRENT_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8067
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_CURRENT_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8068
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_CURRENT_RESULT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8069
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_MEAN_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8070
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_MEAN_VALUE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8071
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_MEAN_RESULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8072
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_PEAK_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8073
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_PEAK_VALUE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8074
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_PEAK_PEAK_RESULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8075
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_CURRENT_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8076
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_CURRENT_VALUE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8077
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_CURRENT_RESULT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8078
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_MEAN_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8079
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_MEAN_VALUE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8080
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_MEAN_RESULT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8081
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_PEAK_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8082
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_PEAK_VALUE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8083
#define RSFPL_ATTR_VSA_MACC_LIMIT_PHASE_ERROR_RMS_PEAK_RESULT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8084
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_CURRENT_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8085
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_CURRENT_VALUE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8086
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_CURRENT_RESULT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8087
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_MEAN_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8088
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_MEAN_VALUE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8089
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_MEAN_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8090
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_PEAK_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8091
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_PEAK_VALUE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8092
#define RSFPL_ATTR_VSA_MACC_LIMIT_RHO_PEAK_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8093
#define RSFPL_ATTR_PHASE_NOISE_MODE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8207
#define RSFPL_ATTR_PHASE_NOISE_INITIATE_REFERENCE_MEASUREMENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8208
#define RSFPL_ATTR_PHASE_NOISE_FINISH_HALF_DECADE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8209
#define RSFPL_ATTR_PHASE_NOISE_NOMINAL_LEVEL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8210
#define RSFPL_ATTR_PHASE_NOISE_VERIFY_FREQUENCY_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8211
#define RSFPL_ATTR_PHASE_NOISE_FREQUENCY_RELATIVE_TOLERANCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8212
#define RSFPL_ATTR_PHASE_NOISE_FREQUENCY_ABSOLUTE_TOLERANCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8213
#define RSFPL_ATTR_PHASE_NOISE_VERIFY_LEVEL_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8214
#define RSFPL_ATTR_PHASE_NOISE_LEVEL_TOLERANCE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8215
#define RSFPL_ATTR_PHASE_NOISE_ON_VERIFY_FAILED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8216
#define RSFPL_ATTR_PHASE_NOISE_FREQUENCY_TRACKING_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8217
#define RSFPL_ATTR_PHASE_NOISE_LEVEL_TRACKING_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8218
#define RSFPL_ATTR_PHASE_NOISE_REJECT_AM_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8219
#define RSFPL_ATTR_PHASE_NOISE_MAX_FREQ_DRIFT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8220
#define RSFPL_ATTR_PHASE_NOISE_DIGITAL_PLL_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8221
#define RSFPL_ATTR_PHASE_NOISE_DECIMATION_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8222
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_FORWARD_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8223
#define RSFPL_ATTR_PHASE_NOISE_SELECT_PRESET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8224
#define RSFPL_ATTR_PHASE_NOISE_GLOBAL_RBW                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8225
#define RSFPL_ATTR_PHASE_NOISE_GLOBAL_AVERAGE_COUNT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8226
#define RSFPL_ATTR_PHASE_NOISE_MULTIPLIER_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8227
#define RSFPL_ATTR_PHASE_NOISE_MULTIPLIER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8228
#define RSFPL_ATTR_PHASE_NOISE_GLOBAL_SWEEP_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8229
#define RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_RBW                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8230
#define RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_SWEEP_MODE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8231
#define RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_AVERAGE_COUNT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8232
#define RSFPL_ATTR_PHASE_NOISE_HALF_DECADE_WINDOW_TYPE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8233
#define RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_SCOPE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8234
#define RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_START                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8235
#define RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_STOP                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8236
#define RSFPL_ATTR_PHASE_NOISE_TRACE_X_AXIS_HALF_DECADE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8237
#define RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_SCALING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8238
#define RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_RANGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8240
#define RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_BOTTOM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8241
#define RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8242
#define RSFPL_ATTR_PHASE_NOISE_TRACE_SPUR_REMOVAL_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8244
#define RSFPL_ATTR_PHASE_NOISE_TRACE_SMOOTHING_METHOD                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8245
#define RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_AUTO_SCALE_ONCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8243
#define RSFPL_ATTR_PHASE_NOISE_TRACE_Y_AXIS_TOP                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8239
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_TYPE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8246
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SELECTED_TRACES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8247
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SELECTED_TRACE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8248
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_NOISE_FLOOR                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8250
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_CHECK_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8251
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SEGMENT_CORNER_FREQUENCY                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8252
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_SEGMENT_LEFT_SLOPE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8253
#define RSFPL_ATTR_PHASE_NOISE_LIMIT_LINE_COPY_TO_USER_LIMIT_LINE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8254
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RANGE_START                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8255
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RANGE_END                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8256
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8257
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_START                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8258
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_END                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8259
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_TRACE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8260
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_DISABLE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8261
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_INTEGRATED_PHASE_NOISE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8262
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RESIDUAL_FM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8263
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RESIDUAL_RMS_JITTER                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8264
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_RESIDUAL_PM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8265
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_INTEGRATED_PHASE_NOISE    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8266
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_FM               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8267
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_RMS_JITTER       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8268
#define RSFPL_ATTR_PHASE_NOISE_RESIDUAL_NOISE_USER_RANGE_RESIDUAL_PM               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8269
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_ON_ALL_DECADE_EDGES_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8270
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DISABLE_ALL_MARKERS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8271
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_EDGE_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8272
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_MARKER_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8273
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_CUSTOM_MARKER_FREQUENCY                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8274
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_VALUES                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8277
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_DECADE_FREQUENCIES                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8278
#define RSFPL_ATTR_PHASE_NOISE_SPOT_NOISE_MARKER_VALUE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8279
#define RSFPL_ATTR_PHASE_NOISE_SPUR_REMOVAL_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8280
#define RSFPL_ATTR_PHASE_NOISE_SPUR_THRESHOLD                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8281
#define RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_RESULTS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8282
#define RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_DISCRETE_JITTER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8284
#define RSFPL_ATTR_PHASE_NOISE_SPUR_LIST_RANDOM_JITTER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8285
#define RSFPL_ATTR_PHASE_NOISE_TRACE_MEASURED_FREQUENCY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8286
#define RSFPL_ATTR_PHASE_NOISE_TRACE_MEASURED_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8287
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8295
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_STOP                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8296
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_SAMPLING_RATE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8297
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_AVERAGES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8298
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_FREQUENCY_DRIFT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8299
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_LEVEL_DRIFT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8300
#define RSFPL_ATTR_PHASE_NOISE_SWEEP_RESULT_LIST_MAX_DRIFT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8301
#define RSFPL_ATTR_DATA_FILE_NAME_COMMENT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7339
#define RSFPL_ATTR_DATA_FILE_NAME                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7340
#define RSFPL_ATTR_DATA_DECIMAL_SEPARATOR                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7341
#define RSFPL_ATTR_DATA_X_VALUE_DISTRIBUTION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8302
#define RSFPL_ATTR_DATA_SELECTED_STORAGE_DEVICE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7342
#define RSFPL_ATTR_DATA_SELECTED_DIRECTORY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7343
#define RSFPL_ATTR_DATA_DELETE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7349
#define RSFPL_ATTR_DATA_CREATE_DIRECTORY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7351
#define RSFPL_ATTR_DATA_DELETE_DIRECTORY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7353
#define RSFPL_ATTR_DATA_NETWORK_UNUSED_DRIVES                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7356
#define RSFPL_ATTR_DATA_NETWORK_USED_DRIVES                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7357
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_DEFAULT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7358
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_ALL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7359
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_NONE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7360
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_HW_SETTINGS_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7361
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_LIMIT_LINES_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7362
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_SPECTROGRAM_DATA_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7363
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_TRACE_DATA_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7364
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_TRANSDUCER_FACTORS_ENABLED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7365
#define RSFPL_ATTR_ITEMS_TO_STORE_INSTRUMENT_SOURCE_CALIBRATION_DATA               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8361
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_DEFAULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7366
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7367
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_NONE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7368
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_HW_SETTINGS_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7369
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_LIMIT_LINES_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7370
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_SPECTROGRAM_DATA_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7371
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_TRACE_DATA_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7372
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_TRANSDUCER_FACTORS_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7373
#define RSFPL_ATTR_ITEMS_TO_STORE_CHANNEL_SOURCE_CALIBRATION_DATA                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8362
#define RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7374
#define RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_DELETE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7375
#define RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_RECALL_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8096
#define RSFPL_ATTR_DATA_INSTRUMENT_SETTINGS_SAVE_NEXT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7380
#define RSFPL_ATTR_HCOPY_ABORT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7381
#define RSFPL_ATTR_HCOPY_MODE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8303
#define RSFPL_ATTR_HCOPY_PRINT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7382
#define RSFPL_ATTR_HCOPY_PRINT_NEXT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7383
#define RSFPL_ATTR_HCOPY_CONTENT_TYPE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7384
#define RSFPL_ATTR_HCOPY_COMMENT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7385
#define RSFPL_ATTR_HCOPY_LOGO_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7386
#define RSFPL_ATTR_HCOPY_PAGE_COUNT_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7387
#define RSFPL_ATTR_HCOPY_DATE_AND_TIME_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7388
#define RSFPL_ATTR_HCOPY_DESTINATION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7391
#define RSFPL_ATTR_HCOPY_SELECTED_PRINTER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7392
#define RSFPL_ATTR_HCOPY_FILE_FORMAT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7393
#define RSFPL_ATTR_HCOPY_FIRST_PRINTER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7394
#define RSFPL_ATTR_HCOPY_NEXT_PRINTER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7395
#define RSFPL_ATTR_HCOPY_PAGE_ORIENTATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7396
#define RSFPL_ATTR_HCOPY_WINDOWS_PER_PAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7397
#define RSFPL_ATTR_HCOPY_PAGE_SCALE_TO_FIT_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7398
#define RSFPL_ATTR_HCOPY_PAGE_MARGIN_UNIT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7399
#define RSFPL_ATTR_HCOPY_PAGE_MARGIN_BOTTOM                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7400
#define RSFPL_ATTR_HCOPY_PAGE_MARGIN_LEFT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7401
#define RSFPL_ATTR_HCOPY_PAGE_MARGIN_RIGHT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7402
#define RSFPL_ATTR_HCOPY_PAGE_MARGIN_TOP                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7403
#define RSFPL_ATTR_HCOPY_COLOR_PRINTING_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7404
#define RSFPL_ATTR_DATA_RESULTS_HEADER_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7406
#define RSFPL_ATTR_DATA_RESULTS_FILE_FORMAT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7407
#define RSFPL_ATTR_DATA_RESULTS_INCLUDED_DATA_EXPORT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7408
#define RSFPL_ATTR_DATA_RESULTS_INCLUDED_DATA_IMPORT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7409
#define RSFPL_ATTR_DATA_RESULTS_EXPORT_MARKER_PEAK_LIST_SPURIOUS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7410
#define RSFPL_ATTR_DATA_RESULTS_EXPORT_SPURIOUS_EMISSION_LIST                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7414
#define RSFPL_ATTR_DATA_RESULTS_EXPORT_MARKER_PEAK_LIST                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7416
#define RSFPL_ATTR_DATA_RESULTS_EXPORT_SPECTROGRAM_DATA                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7418
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_PAGE_FORMAT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8304
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_PRINTER_FRIENDLY_COLORS_ENABLED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8305
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_INCLUDE_DATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8306
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_INCLUDE_PAGE_COUNT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8307
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_RAW_DATA_STORAGE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8308
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LINE_TITLE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8309
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LINE_VALUE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8310
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LINE_VISIBILITY                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8311
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LOGO                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8312
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_TITLE_PAGE_LOGO_VISIBILITY                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8313
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_SELECTED_ITEMS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8314
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_CATALOGUE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8317
#define RSFPL_ATTR_TEST_REPORT_RESTORE_DEFAULTS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8325
#define RSFPL_ATTR_TEST_REPORT_NEW                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8326
#define RSFPL_ATTR_TEST_REPORT_APPEND                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8327
#define RSFPL_ATTR_TEST_REPORT_INCLUDE_USER_DEFINED_INFORMATION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8328
#define RSFPL_ATTR_TEST_REPORT_TEMPLATE_INCLUDE_TITLE_PAGE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8329
#define RSFPL_ATTR_TEST_REPORT_TITLE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8330
#define RSFPL_ATTR_TEST_REPORT_DESCRIPTION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8331
#define RSFPL_ATTR_TEST_REPORT_ITEM_LIST                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8332
#define RSFPL_ATTR_TEST_REPORT_REMOVE_ALL_SETS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8337
#define RSFPL_ATTR_TRANSDUCER_FACTOR_CATALOG                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7420
#define RSFPL_ATTR_SELECTED_TRANSDUCER_FACTOR                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7421
#define RSFPL_ATTR_TRANSDUCER_FACTOR_SCALING                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7423
#define RSFPL_ATTR_TRANSDUCER_FACTOR_UNIT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7424
#define RSFPL_ATTR_TRANSDUCER_FACTOR_COMMENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7425
#define RSFPL_ATTR_TRANSDUCER_FACTOR_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7426
#define RSFPL_ATTR_TRANSDUCER_FACTOR_REF_LEVEL_AUTO_ADJUST_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7427
#define RSFPL_ATTR_TRANSDUCER_FACTOR_IMPORT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7428
#define RSFPL_ATTR_DELETE_TRANSDUCER_FACTOR                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7431
#define RSFPL_ATTR_GENERAL_REFERENCE_OSCILLATOR_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7432
#define RSFPL_ATTR_GENERAL_REFERENCE_OSCILLATOR_OUTPUT_100_MHZ_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8098
#define RSFPL_ATTR_GENERAL_SELF_TEST                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7433
#define RSFPL_ATTR_GENERAL_REMOTE_LOGGING_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7434
#define RSFPL_ATTR_GENERAL_DISPLAY_LANGUAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7435
#define RSFPL_ATTR_GENERAL_RESET_REVISION_TO_FACTORY_DEFAULT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7437
#define RSFPL_ATTR_GENERAL_RESET_IDN_QUERY_TO_FACTORY_DEFAULT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7439
#define RSFPL_ATTR_GENERAL_GET_IDN_STRING_FACTORY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8104
#define RSFPL_ATTR_GENERAL_IDN_QUERY_STRING                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7440
#define RSFPL_ATTR_GENERAL_LOCAL_LOCKOUT_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7441
#define RSFPL_ATTR_GENERAL_LXI_INFO                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7442
#define RSFPL_ATTR_GENERAL_LXI_LAN_RESET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7443
#define RSFPL_ATTR_GENERAL_LXI_INSTRUMENT_DESCRIPTION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7444
#define RSFPL_ATTR_GENERAL_LXI_PASSWORD                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7445
#define RSFPL_ATTR_GENERAL_DISPLAY_UPDATE_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7446
#define RSFPL_ATTR_GENERAL_DISPLAY_LOCK_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8105
#define RSFPL_ATTR_GENERAL_DISPLAY_ERROR_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7447
#define RSFPL_ATTR_GENERAL_SYSTEM_DATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8338
#define RSFPL_ATTR_GENERAL_SYSTEM_TIME                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8339
#define RSFPL_ATTR_SYST_ERR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7511
#define RSFPL_ATTR_CALIBRATE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7448
#define RSFPL_ATTR_CALIBRATE_ALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8342
#define RSFPL_ATTR_CALIBRATION_RESULT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7449
#define RSFPL_ATTR_CALIBRATION_SIGNAL_FREQUENCY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7450
#define RSFPL_ATTR_CALIBRATION_SIGNAL_BANDWIDTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7451
#define RSFPL_ATTR_CALIBRATION_INPUT_SOURCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7452
#define RSFPL_ATTR_CALIBRATION_SIGNAL_FREQUENCY_43_MHZ_PLUS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8106
#define RSFPL_ATTR_CALIBRATION_SIGNAL_PEAK_DISTANCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8107
#define RSFPL_ATTR_CALIBRATION_SIGNAL_FREQUENCY_7_GHZ_PLUS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8108
#define RSFPL_ATTR_CALIBRATION_SIGNAL_WIDEBAND_FREQUENCY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8109
#define RSFPL_ATTR_FRONTEND_TEMPERATURE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7453
#define RSFPL_ATTR_SERVICE_STEST_RESULTS                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7454
#define RSFPL_ATTR_RECALL_CALIBRATION                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8343
#define RSFPL_ATTR_SAVE_CALIBRATION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8344
#define RSFPL_ATTR_CALIBRATION_SAVE_FILE_PATH                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8345
#define RSFPL_ATTR_SERVICE_FUNCTION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7455
#define RSFPL_ATTR_SERVICE_FUNCTION_LAST_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7456
#define RSFPL_ATTR_SERVICE_FUNCTION_DELETE_LAST_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7457
#define RSFPL_ATTR_SERVICE_FUNCTION_RESULT_FILE_NAME                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7458
#define RSFPL_ATTR_SERVICE_SYSTEM_INFORMATION_FILE_PATH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7459
#define RSFPL_ATTR_SERVICE_FUNCTION_RESET_PASSWORD                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7460
#define RSFPL_ATTR_SERVICE_FUNCTION_PASSWORD                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7461
#define RSFPL_ATTR_SYSTEM_BIOS_INFO                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7463
#define RSFPL_ATTR_SYSTEM_HW_INFO                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7464
#define RSFPL_ATTR_SYSTEM_VERSION_INFO                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7465
#define RSFPL_ATTR_SYSTEM_BATTERY_LEVEL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8110
#define RSFPL_ATTR_STATUS_PRESET                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7488
#define RSFPL_ATTR_SYSTEM_CLEAR_ERRORS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8111
#define RSFPL_ATTR_SYSTEM_ERROR_LIST                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8113
#define RSFPL_ATTR_SYSTEM_CLEAR_REMOTE_ERRORS_TABLE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8115
#define RSFPL_ATTR_ID_QUERY_RESPONSE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7489
#define RSFPL_ATTR_SYSTEM_PRESET                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7491
#define RSFPL_ATTR_PRESET_CHANNEL                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7492
#define RSFPL_ATTR_GENERAL_REBOOT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7436
#define RSFPL_ATTR_GENERAL_SHUTDOWN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7438

// RsFpl_rngTraceDetector
#define RSFPL_VAL_DETECTOR_APEAK                                            0
#define RSFPL_VAL_DETECTOR_NEGATIVE                                         1
#define RSFPL_VAL_DETECTOR_POSITIVE                                         2
#define RSFPL_VAL_DETECTOR_SAMPLE                                           3
#define RSFPL_VAL_DETECTOR_RMS                                              4
#define RSFPL_VAL_DETECTOR_AVERAGE                                          5
#define RSFPL_VAL_DETECTOR_QPE                                              6
#define RSFPL_VAL_DETECTOR_CAV                                              7
#define RSFPL_VAL_DETECTOR_CRMS                                             8

// RsFpl_rngAutoMan
#define RSFPL_VAL_AUTO                                                      0
#define RSFPL_VAL_MANUAL                                                    1

// RsFpl_rngTraceAverageMode
#define RSFPL_VAL_TRACE_AVERAGE_LINEAR                                      1
#define RSFPL_VAL_TRACE_AVERAGE_POWER                                       2
#define RSFPL_VAL_TRACE_AVERAGE_LOG                                         3

// RsFpl_rngLinLog
#define RSFPL_VAL_LIN                                                       0
#define RSFPL_VAL_LOG                                                       1

// RsFpl_rngFilterType
#define RSFPL_VAL_FILTER_NORMAL                                             0
#define RSFPL_VAL_FILTER_CFILTER                                            1
#define RSFPL_VAL_FILTER_P5                                                 2
#define RSFPL_VAL_FILTER_RRC                                                3
#define RSFPL_VAL_FILTER_PULSE                                              4
#define RSFPL_VAL_FILTER_MIL                                                5

// RsFpl_rngSweepListInsertMode
#define RSFPL_VAL_INSERT_AFTER                                              0
#define RSFPL_VAL_INSERT_BEFORE                                             1

// RsFpl_rngStartStopFunction
#define RSFPL_VAL_STOP_FUNCTION_OFF                                         0
#define RSFPL_VAL_STOP_FUNCTION_MAX                                         1

// RsFpl_rngPCLLimitCheckMode
#define RSFPL_VAL_PCL_LIMIT_CHECK_ABSOLUTE                                  0
#define RSFPL_VAL_PCL_LIMIT_CHECK_RELATIVE                                  1
#define RSFPL_VAL_PCL_LIMIT_CHECK_AND                                       2
#define RSFPL_VAL_PCL_LIMIT_CHECK_OR                                        3

// RsFpl_rngMLCalculation
#define RSFPL_VAL_MLC_NONE                                                  0
#define RSFPL_VAL_MLC_MAX                                                   1
#define RSFPL_VAL_MLC_SUM                                                   2

// RsFpl_rngPowerReferenceType
#define RSFPL_VAL_POWER_REFERENCE_PEAK                                      0
#define RSFPL_VAL_POWER_REFERENCE_CPOWER                                    1

// RsFpl_rngStepSizeCoupling
#define RSFPL_VAL_STEP_SIZE_COUPLING_SPAN                                   0
#define RSFPL_VAL_STEP_SIZE_COUPLING_RBW                                    1
#define RSFPL_VAL_STEP_SIZE_COUPLING_OFF                                    2

// RsFpl_rngPulsePowerDetector
#define RSFPL_VAL_PULSE_POWER_DETECTOR_MEAN                                 0
#define RSFPL_VAL_PULSE_POWER_DETECTOR_PEAK                                 1

// RsFpl_rngPowerSensorFrequencyCoupling
#define RSFPL_VAL_PMET_COUPLING_OFF                                         0
#define RSFPL_VAL_PMET_COUPLING_CENTER                                      1
#define RSFPL_VAL_PMET_COUPLING_MARKER1                                     2

// RsFpl_rngPowerSensorMeasTime
#define RSFPL_VAL_PMET_MEAS_TIME_SHORT                                      0
#define RSFPL_VAL_PMET_MEAS_TIME_NORMAL                                     1
#define RSFPL_VAL_PMET_MEAS_TIME_LONG                                       2

// RsFpl_rngPowerMeasurement
#define RSFPL_VAL_POWER_MEAS_ACLR                                           0
#define RSFPL_VAL_POWER_MEAS_CHANNEL                                        1
#define RSFPL_VAL_POWER_MEAS_OBW                                            2
#define RSFPL_VAL_POWER_MEAS_CN                                             3
#define RSFPL_VAL_POWER_MEAS_CN0                                            4

// RsFpl_rngReferenceChannelAutoType
#define RSFPL_VAL_REFERENCE_MINIMUM                                         0
#define RSFPL_VAL_REFERENCE_MAXIMUM                                         1
#define RSFPL_VAL_REFERENCE_LHIGHEST                                        2

// RsFpl_rngROSCSource
#define RSFPL_VAL_ROSC_INTERNAL                                             0
#define RSFPL_VAL_ROSC_EXTERNAL                                             1

// RsFpl_rngPosNeg
#define RSFPL_VAL_POSITIVE                                                  0
#define RSFPL_VAL_NEGATIVE                                                  1

// RsFpl_rngGateSource
#define RSFPL_VAL_GATE_EXTERNAL                                             0
#define RSFPL_VAL_GATE_IFPOWER                                              1
#define RSFPL_VAL_GATE_IMMEDIATE                                            2

// RsFpl_rngGateMode
#define RSFPL_VAL_GATE_LEVEL                                                0
#define RSFPL_VAL_GATE_EDGE                                                 1

// RsFpl_rngMeasurementMode
#define RSFPL_VAL_MEASUREMENT_BASIC                                         1
#define RSFPL_VAL_MEASUREMENT_SEM                                           2
#define RSFPL_VAL_MEASUREMENT_SPURIOUS_EMISSION                             0

// RsFpl_rngPartialSpanOptimizationMode
#define RSFPL_VAL_OPTIMIZATION_AUTO                                         0
#define RSFPL_VAL_OPTIMIZATION_DYNAMIC                                      1
#define RSFPL_VAL_OPTIMIZATION_SPEED                                        2

// RsFpl_rngBandPowerMode
#define RSFPL_VAL_BAND_POWER_POWER                                          0
#define RSFPL_VAL_BAND_POWER_REL_POWER                                      1
#define RSFPL_VAL_BAND_POWER_DENSITY                                        2

// RsFpl_rngSpectrogramMarkerSearchArea
#define RSFPL_VAL_SGR_AREA_VISIBLE                                          0
#define RSFPL_VAL_SGR_AREA_MEMORY                                           1

// RsFpl_rngLimitLineDomain
#define RSFPL_VAL_DOMAIN_FREQUENCY                                          0
#define RSFPL_VAL_DOMAIN_TIME                                               1

// RsFpl_rngLimitLineUnit
#define RSFPL_VAL_UNIT_DBM                                                  0
#define RSFPL_VAL_UNIT_V                                                    1
#define RSFPL_VAL_UNIT_A                                                    2
#define RSFPL_VAL_UNIT_W                                                    3
#define RSFPL_VAL_UNIT_DBPW                                                 4
#define RSFPL_VAL_UNIT_DBUV                                                 5
#define RSFPL_VAL_UNIT_DBMV                                                 6
#define RSFPL_VAL_UNIT_DBUA                                                 7
#define RSFPL_VAL_UNIT_DB                                                   8
#define RSFPL_VAL_UNIT_DBUV_MHZ                                             9
#define RSFPL_VAL_UNIT_DBUA_MHZ                                             10
#define RSFPL_VAL_UNIT_PCT                                                  11
#define RSFPL_VAL_UNIT_DBM_MHZ                                              12
#define RSFPL_VAL_UNIT_DBMV_MHZ                                             13
#define RSFPL_VAL_UNIT_DBUV_MMHZ                                            14
#define RSFPL_VAL_UNIT_DBUA_MMHZ                                            15
#define RSFPL_VAL_UNIT_DBPW_MHZ                                             16
#define RSFPL_VAL_UNIT_DBPT_MHZ                                             17
#define RSFPL_VAL_UNIT_DBPT                                                 18

// RsFpl_rngDemodulationMode
#define RSFPL_VAL_DEMODULATION_AM                                           0
#define RSFPL_VAL_DEMODULATION_FM                                           1
#define RSFPL_VAL_DEMODULATION_PM                                           2
#define RSFPL_VAL_DEMODULATION_AC                                           3
#define RSFPL_VAL_DEMODULATION_AUDIO                                        4

// RsFpl_rngPeakListSortMode
#define RSFPL_VAL_SORT_X                                                    0
#define RSFPL_VAL_SORT_Y                                                    1

// RsFpl_rngPowerMeasurementTraceDisplayMode
#define RSFPL_VAL_TRACE_DISPLAY_WRITE                                       0
#define RSFPL_VAL_TRACE_DISPLAY_MAXHOLD                                     1

// RsFpl_rngPowerMeasurementResultType
#define RSFPL_VAL_POWER_RESULT_CHANNEL                                      0
#define RSFPL_VAL_POWER_RESULT_OBW                                          1
#define RSFPL_VAL_POWER_RESULT_CN                                           2
#define RSFPL_VAL_POWER_RESULT_CN0                                          3
#define RSFPL_VAL_POWER_RESULT_PPOWER                                       4
#define RSFPL_VAL_POWER_RESULT_ACLR                                         5

// RsFpl_rngMarkerStepsize
#define RSFPL_VAL_STEPSIZE_STANDARD                                         0
#define RSFPL_VAL_STEPSIZE_POINTS                                           1

// RsFpl_rngTraceMathematicsMode
#define RSFPL_VAL_TRACE_MATH_LINEAR                                         0
#define RSFPL_VAL_TRACE_MATH_LOGARITHMIC                                    1
#define RSFPL_VAL_TRACE_MATH_POWER                                          2

// RsFpl_rngSpectrogramLayout
#define RSFPL_VAL_SGR_LAYOUT_OFF                                            0
#define RSFPL_VAL_SGR_LAYOUT_SPLIT                                          1

// RsFpl_rngCalibrationSignalBandwidth
#define RSFPL_VAL_CAL_SIGNAL_BWID_NARROWBAND                                0
#define RSFPL_VAL_CAL_SIGNAL_BWID_BROADBAND                                 1

// RsFpl_rngDisplayTouchEnabled
#define RSFPL_VAL_TOUCH_OFF                                                 0
#define RSFPL_VAL_TOUCH_ON                                                  1
#define RSFPL_VAL_TOUCH_FRAME                                               2

// RsFpl_rngDateTimeFormat
#define RSFPL_VAL_DATE_TIME_US                                              0
#define RSFPL_VAL_DATE_TIME_DE                                              1

// RsFpl_rngMarkerTableDisplayMode
#define RSFPL_VAL_MARKER_TABLE_OFF                                          0
#define RSFPL_VAL_MARKER_TABLE_ON                                           1
#define RSFPL_VAL_MARKER_TABLE_AUTO                                         2

// RsFpl_rngSpectrogramColorScheme
#define RSFPL_VAL_SGR_SCHEME_HOT                                            0
#define RSFPL_VAL_SGR_SCHEME_COLD                                           1
#define RSFPL_VAL_SGR_SCHEME_RADAR                                          2
#define RSFPL_VAL_SGR_SCHEME_GRAYSCALE                                      3

// RsFpl_rngTraceMode
#define RSFPL_VAL_TRACE_BLANK                                               0
#define RSFPL_VAL_TRACE_WRITE                                               1
#define RSFPL_VAL_TRACE_MAX_HOLD                                            2
#define RSFPL_VAL_TRACE_MIN_HOLD                                            3
#define RSFPL_VAL_TRACE_AVERAGE                                             4
#define RSFPL_VAL_TRACE_VIEW                                                5
#define RSFPL_VAL_TRACE_DENSITY                                             6

// RsFpl_rngAbsRel
#define RSFPL_VAL_ABS                                                       0
#define RSFPL_VAL_REL                                                       1

// RsFpl_rngDecimalSeparator
#define RSFPL_VAL_SEPARATOR_POINT                                           0
#define RSFPL_VAL_SEPARATOR_COMMA                                           1

// RsFpl_rngResultsFileFormat
#define RSFPL_VAL_RESULT_FILE_CSV                                           0
#define RSFPL_VAL_RESULT_FILE_DAT                                           1

// RsFpl_rngIncludedData
#define RSFPL_VAL_INCLUDED_DATA_SINGLE                                      0
#define RSFPL_VAL_INCLUDED_DATA_ALL                                         1

// RsFpl_rngHcopyContent
#define RSFPL_VAL_HCOPY_WINDOWS                                             0
#define RSFPL_VAL_HCOPY_HCOPY                                               1

// RsFpl_rngHcopyFileFormat
#define RSFPL_VAL_HCOPY_FORMAT_GDI                                          0
#define RSFPL_VAL_HCOPY_FORMAT_BMP                                          1
#define RSFPL_VAL_HCOPY_FORMAT_PNG                                          2
#define RSFPL_VAL_HCOPY_FORMAT_JPG                                          3
#define RSFPL_VAL_HCOPY_FORMAT_PDF                                          6
#define RSFPL_VAL_HCOPY_FORMAT_SVG                                          7
#define RSFPL_VAL_HCOPY_FORMAT_DOC                                          8
#define RSFPL_VAL_HCOPY_FORMAT_RTF                                          9

// RsFpl_rngPageMarginUnit
#define RSFPL_VAL_PAGE_MARGIN_MM                                            0
#define RSFPL_VAL_PAGE_MARGIN_IN                                            1

// RsFpl_rngPageOrientation
#define RSFPL_VAL_PAGE_ORIENTATION_LANDSCAPE                                0
#define RSFPL_VAL_PAGE_ORIENTATION_PORTRAIT                                 1

// RsFpl_rngSequencerMode
#define RSFPL_VAL_SEQUENCER_MODE_SINGLE                                     0
#define RSFPL_VAL_SEQUENCER_MODE_CONTINUOUS                                 1
#define RSFPL_VAL_SEQUENCER_MODE_CDEFINED                                   2

// RsFpl_rngInstrumentSettingsMode
#define RSFPL_VAL_SETTINGS_INSTRUMENT                                       0
#define RSFPL_VAL_SETTINGS_CHANNEL                                          1

// RsFpl_rngOutputSource
#define RSFPL_VAL_OUTPUT_SOURCE_IF                                          0
#define RSFPL_VAL_OUTPUT_SOURCE_VIDEO                                       1

// RsFpl_rngOutputSignalPolarity
#define RSFPL_VAL_POLARITY_HIGH                                             0
#define RSFPL_VAL_POLARITY_LOW                                              1

// RsFpl_rngTriggerSource
#define RSFPL_VAL_TRIGGER_IMMEDIATE                                         0
#define RSFPL_VAL_TRIGGER_EXTERNAL                                          1
#define RSFPL_VAL_TRIGGER_VIDEO                                             2
#define RSFPL_VAL_TRIGGER_IFPOWER                                           3
#define RSFPL_VAL_TRIGGER_TIME                                              4
#define RSFPL_VAL_TRIGGER_IQP                                               5
#define RSFPL_VAL_TRIGGER_AF                                                6
#define RSFPL_VAL_TRIGGER_FM                                                7
#define RSFPL_VAL_TRIGGER_AM                                                8
#define RSFPL_VAL_TRIGGER_AMR                                               9
#define RSFPL_VAL_TRIGGER_PM                                                10
#define RSFPL_VAL_TRIGGER_BB_PWR                                            11
#define RSFPL_VAL_TRIGGER_LINE                                              12
#define RSFPL_VAL_TRIGGER_RFPOWER                                           13
#define RSFPL_VAL_TRIGGER_EXTERNAL2                                         14
#define RSFPL_VAL_TRIGGER_EXTERNAL3                                         15
#define RSFPL_VAL_TRIGGER_EXTERNAL4                                         16
#define RSFPL_VAL_TRIGGER_LXI                                               17

// RsFpl_rngPowerSensorRelativePowerUnit
#define RSFPL_VAL_SENSOR_UNIT_DB                                            0
#define RSFPL_VAL_SENSOR_UNIT_PCT                                           1

// RsFpl_rngSweepType
#define RSFPL_VAL_SWEEP_TYPE_NORMAL                                         0
#define RSFPL_VAL_SWEEP_TYPE_FFT                                            1

// RsFpl_rngYAxisScaling
#define RSFPL_VAL_SCALING_LOG                                               0
#define RSFPL_VAL_SCALING_LIN                                               1
#define RSFPL_VAL_SCALING_LDB                                               2
#define RSFPL_VAL_SCALING_PERC                                              3

// RsFpl_rngTraceResultType
#define RSFPL_VAL_TRACE_RESULT_TRACE1                                       0
#define RSFPL_VAL_TRACE_RESULT_TRACE2                                       1
#define RSFPL_VAL_TRACE_RESULT_TRACE3                                       2
#define RSFPL_VAL_TRACE_RESULT_TRACE4                                       3
#define RSFPL_VAL_TRACE_RESULT_TRACE5                                       4
#define RSFPL_VAL_TRACE_RESULT_TRACE6                                       5
#define RSFPL_VAL_TRACE_RESULT_LIST                                         6
#define RSFPL_VAL_TRACE_RESULT_SPURIOUS                                     7
#define RSFPL_VAL_TRACE_RESULT_SPECTROGRAM                                  8

// RsFpl_rngLayoutDirection
#define RSFPL_VAL_DIRECTION_LEFT                                            0
#define RSFPL_VAL_DIRECTION_RIGHT                                           1
#define RSFPL_VAL_DIRECTION_ABOVE                                           2
#define RSFPL_VAL_DIRECTION_BELOW                                           3
#define RSFPL_VAL_DIRECTION_REPLACE                                         4

// RsFpl_rngLayoutWindowType
#define RSFPL_VAL_LAYOUT_TYPE_DIAGRAM                                       0
#define RSFPL_VAL_LAYOUT_TYPE_MTABLE                                        1
#define RSFPL_VAL_LAYOUT_TYPE_PEAKLIST                                      2
#define RSFPL_VAL_LAYOUT_TYPE_RSUMMARY                                      3
#define RSFPL_VAL_LAYOUT_TYPE_SGRAM                                         4

// RsFpl_rngHcopyDevice
#define RSFPL_VAL_HCOPY_DEVICE_MEM                                          0
#define RSFPL_VAL_HCOPY_DEVICE_PRN                                          1
#define RSFPL_VAL_HCOPY_DEVICE_CLP                                          2

// RsFpl_rngStatusRegisterOperation
#define RSFPL_VAL_STAT_REG_QUESTIONABLE                                     1
#define RSFPL_VAL_STAT_REG_OPERATION                                        2

// RsFpl_rngStatusRegister
#define RSFPL_VAL_STAT_REG_MAIN                                             0
#define RSFPL_VAL_STAT_REG_POWER                                            1
#define RSFPL_VAL_STAT_REG_LIMIT                                            2
#define RSFPL_VAL_STAT_REG_LMARGIN                                          3
#define RSFPL_VAL_STAT_REG_SYNC                                             4
#define RSFPL_VAL_STAT_REG_ACP_LIMIT                                        5
#define RSFPL_VAL_STAT_REG_FREQUENCY                                        6
#define RSFPL_VAL_STAT_REG_DIQ                                              7
#define RSFPL_VAL_STAT_REG_EXTENDED                                         8
#define RSFPL_VAL_STAT_REG_EXTENDED_INFO                                    9
#define RSFPL_VAL_STAT_REG_TEMP                                             10
#define RSFPL_VAL_STAT_REG_TIME                                             11
#define RSFPL_VAL_STAT_REG_POW_DCPN                                         12
#define RSFPL_VAL_STAT_REG_PNOISE                                           13
#define RSFPL_VAL_STAT_REG_CORRECTION                                       14

// RsFpl_rngStatusRegisterQuery
#define RSFPL_VAL_STAT_REG_QUERY_COND                                       0
#define RSFPL_VAL_STAT_REG_QUERY_EVENT                                      1
#define RSFPL_VAL_STAT_REG_QUERY_ENABLE                                     2
#define RSFPL_VAL_STAT_REG_QUERY_PTRANSITION                                3
#define RSFPL_VAL_STAT_REG_QUERY_NTRANSITION                                4

// RsFpl_rngTFacUnit
#define RSFPL_VAL_TFAC_UNIT_DB                                              0
#define RSFPL_VAL_TFAC_UNIT_DBM                                             1
#define RSFPL_VAL_TFAC_UNIT_DBMV                                            2
#define RSFPL_VAL_TFAC_UNIT_DBUV                                            3
#define RSFPL_VAL_TFAC_UNIT_DBUVM                                           4
#define RSFPL_VAL_TFAC_UNIT_DBUA                                            5
#define RSFPL_VAL_TFAC_UNIT_DBUAM                                           6
#define RSFPL_VAL_TFAC_UNIT_DBPW                                            7
#define RSFPL_VAL_TFAC_UNIT_DBPT                                            8

// RsFpl_rngCalibrationInputSource
#define RSFPL_VAL_INPUT_RF                                                  0
#define RSFPL_VAL_INPUT_CAL                                                 1

// RsFpl_rngGeneralDisplayLanguage
#define RSFPL_VAL_DISPLAY_LANGUAGE_EN                                       0
#define RSFPL_VAL_DISPLAY_LANGUAGE_ZH_CH                                    1
#define RSFPL_VAL_DISPLAY_LANGUAGE_ZH_TW                                    2
#define RSFPL_VAL_DISPLAY_LANGUAGE_JA                                       3
#define RSFPL_VAL_DISPLAY_LANGUAGE_KO                                       4
#define RSFPL_VAL_DISPLAY_LANGUAGE_RU                                       5

// RsFpl_rngACLRMeasurementConfiguration
#define RSFPL_VAL_ACLR_CONFIG_NONE                                          0
#define RSFPL_VAL_ACLR_CONFIG_EUTRA                                         1
#define RSFPL_VAL_ACLR_CONFIG_REUT                                          2
#define RSFPL_VAL_ACLR_CONFIG_WCDMA_FWD                                     3
#define RSFPL_VAL_ACLR_CONFIG_WCDMA_REV                                     4
#define RSFPL_VAL_ACLR_CONFIG_WLAN_A                                        5
#define RSFPL_VAL_ACLR_CONFIG_WLAN_B                                        6
#define RSFPL_VAL_ACLR_CONFIG_RFID                                          7
#define RSFPL_VAL_ACLR_CONFIG_TETRA                                         8

// RsFpl_rngChannelType
#define RSFPL_VAL_CHAN_TYPE_SANALYZER                                       0
#define RSFPL_VAL_CHAN_TYPE_ADEM                                            1
#define RSFPL_VAL_CHAN_TYPE_IQ                                              2
#define RSFPL_VAL_CHAN_TYPE_NOISE                                           3
#define RSFPL_VAL_CHAN_TYPE_VSA                                             4

// RsFpl_rngIQRBWMode
#define RSFPL_VAL_IQ_RBW_MODE_AUTO                                          0
#define RSFPL_VAL_IQ_RBW_MODE_MANUAL                                        1
#define RSFPL_VAL_IQ_RBW_MODE_FFT                                           2

// RsFpl_rngTransformationAlgorithm
#define RSFPL_VAL_FFT_SINGLE                                                0
#define RSFPL_VAL_FFT_AVERAGE                                               1

// RsFpl_rngWindowFunction
#define RSFPL_VAL_WINDOW_BLACKHARRIS                                        0
#define RSFPL_VAL_WINDOW_FLATTOP                                            1
#define RSFPL_VAL_WINDOW_GAUSSIAN                                           2
#define RSFPL_VAL_WINDOW_RECTANGULAR                                        3
#define RSFPL_VAL_WINDOW_P5                                                 4

// RsFpl_rngBranchForPeaksearch
#define RSFPL_VAL_BRANCH_REAL                                               0
#define RSFPL_VAL_BRANCH_IMAG                                               1
#define RSFPL_VAL_BRANCH_MAGN                                               2

// RsFpl_rngDisplaySize
#define RSFPL_VAL_DISPLAY_LARGE                                             0
#define RSFPL_VAL_DISPLAY_SMALL                                             1

// RsFpl_rngIQFormat
#define RSFPL_VAL_IQ_FORMAT_FLOAT32                                         0
#define RSFPL_VAL_IQ_FORMAT_INT32                                           1

// RsFpl_rngIQDataFormat
#define RSFPL_VAL_IQ_DATA_COMPLEX                                           0
#define RSFPL_VAL_IQ_DATA_REAL                                              1

// RsFpl_rngIQAnalyzerGatingMode
#define RSFPL_VAL_IQ_ANALYZER_GATING_LEVEL                                  0
#define RSFPL_VAL_IQ_ANALYZER_GATING_EDGE                                   1

// RsFpl_rngIQAnalyzerLayoutWindowType
#define RSFPL_VAL_IQ_ANALYZER_LAYOUT_TYPE_SPECTRUM                          0
#define RSFPL_VAL_IQ_ANALYZER_LAYOUT_TYPE_MTABLE                            1
#define RSFPL_VAL_IQ_ANALYZER_LAYOUT_TYPE_PEAKLIST                          2
#define RSFPL_VAL_IQ_ANALYZER_LAYOUT_TYPE_MAGNITUDE                         3
#define RSFPL_VAL_IQ_ANALYZER_LAYOUT_TYPE_RIMAG                             4
#define RSFPL_VAL_IQ_ANALYZER_LAYOUT_TYPE_VECTOR                            5

// RsFpl_rngNoisePowerMeasurement
#define RSFPL_VAL_NOISE_MEASUREMENT_HOT                                     0
#define RSFPL_VAL_NOISE_MEASUREMENT_COLD                                    1

// RsFpl_rngNoiseDUTType
#define RSFPL_VAL_NOISE_DUT_DIRECT                                          0
#define RSFPL_VAL_NOISE_DUT_DOWNCONV                                        1
#define RSFPL_VAL_NOISE_DUT_UPCONV                                          2

// RsFpl_rngNoiseSourceMode
#define RSFPL_VAL_NOISE_SOURCE_MODE_SPOT                                    0
#define RSFPL_VAL_NOISE_SOURCE_MODE_TABLE                                   1

// RsFpl_rngENRNoiseSource
#define RSFPL_VAL_ENR_NOISE_SOURCE_DIODE                                    0
#define RSFPL_VAL_ENR_NOISE_SOURCE_RESISTOR                                 1
#define RSFPL_VAL_ENR_NOISE_SOURCE_SMART                                    2

// RsFpl_rngNoiseResultType
#define RSFPL_VAL_NOISE_RESULT_NOISE                                        0
#define RSFPL_VAL_NOISE_RESULT_GAIN                                         1
#define RSFPL_VAL_NOISE_RESULT_TEMPERATURE                                  2
#define RSFPL_VAL_NOISE_RESULT_YFACTOR                                      3
#define RSFPL_VAL_NOISE_RESULT_PHOT                                         4
#define RSFPL_VAL_NOISE_RESULT_PCOLD                                        5
#define RSFPL_VAL_NOISE_RESULT_CYF                                          6
#define RSFPL_VAL_NOISE_RESULT_CPH                                          7
#define RSFPL_VAL_NOISE_RESULT_CPC                                          8
#define RSFPL_VAL_NOISE_RESULT_NUNC                                         9

// RsFpl_rngNoiseDisplayFrequencyType
#define RSFPL_VAL_FREQUENCY_RF                                              0
#define RSFPL_VAL_FREQUENCY_IF                                              1

// RsFpl_rngNoiseLayoutWindowType
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_NOISE                                   0
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_GAIN                                    1
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_TEMPERATURE                             2
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_YFACTOR                                 3
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_PHOT                                    4
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_PCOLD                                   5
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_CYFACTOR                                6
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_CPHOT                                   7
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_CPCOLD                                  8
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_RESULTS                                 9
#define RSFPL_VAL_NOISE_LAYOUT_TYPE_MTABLE                                  10

// RsFpl_rngADEMSummaryResultType
#define RSFPL_VAL_ADEM_SUMMARY_RESULT_IMMEDIATE                             0
#define RSFPL_VAL_ADEM_SUMMARY_RESULT_AVERAGE                               1

// RsFpl_rngADEMMode
#define RSFPL_VAL_ADEM_DEMOD_RESULT_MODE_WRITE                              0
#define RSFPL_VAL_ADEM_DEMOD_RESULT_MODE_AVERAGE                            1
#define RSFPL_VAL_ADEM_DEMOD_RESULT_MODE_MAXHOLD                            2

// RsFpl_rngADEMFilterType
#define RSFPL_VAL_ADEM_FILTER_FLAT                                          0
#define RSFPL_VAL_ADEM_FILTER_GAUSS                                         1

// RsFpl_rngADEMAFCoupling
#define RSFPL_VAL_COUPLING_AC                                               0
#define RSFPL_VAL_COUPLING_DC                                               1

// RsFpl_rngADEMAFHighPassFilterType
#define RSFPL_VAL_ADEM_HP_20HZ                                              0
#define RSFPL_VAL_ADEM_HP_50HZ                                              1
#define RSFPL_VAL_ADEM_HP_300HZ                                             2

// RsFpl_rngADEMAFLowPassFilterTypeRelative
#define RSFPL_VAL_ADEM_LP_RELATIVE_5PCT                                     0
#define RSFPL_VAL_ADEM_LP_RELATIVE_10PCT                                    1
#define RSFPL_VAL_ADEM_LP_RELATIVE_25PCT                                    2

// RsFpl_rngADEMAFLowPassFilterTypeAbsolute
#define RSFPL_VAL_ADEM_LP_ABSOLUTE_3KHZ                                     0
#define RSFPL_VAL_ADEM_LP_ABSOLUTE_15KHZ                                    1
#define RSFPL_VAL_ADEM_LP_ABSOLUTE_150KHZ                                   2

// RsFpl_rngADEMAFDeemphasisFilterType
#define RSFPL_VAL_ADEM_DEEMPHASIS_25US                                      0
#define RSFPL_VAL_ADEM_DEEMPHASIS_50US                                      1
#define RSFPL_VAL_ADEM_DEEMPHASIS_75US                                      2
#define RSFPL_VAL_ADEM_DEEMPHASIS_750US                                     3

// RsFpl_rngADEMEvaluationMethod
#define RSFPL_VAL_ADEM_EVAL_RF_TIME_DOMAIN                                  0
#define RSFPL_VAL_ADEM_EVAL_RF_SPECTRUM                                     1
#define RSFPL_VAL_ADEM_EVAL_AM_TIME_DOMAIN                                  2
#define RSFPL_VAL_ADEM_EVAL_AM_SPECTRUM                                     3
#define RSFPL_VAL_ADEM_EVAL_FM_TIME_DOMAIN                                  4
#define RSFPL_VAL_ADEM_EVAL_FM_SPECTRUM                                     5
#define RSFPL_VAL_ADEM_EVAL_PM_TIME_DOMAIN                                  6
#define RSFPL_VAL_ADEM_EVAL_PM_SPECTRUM                                     7
#define RSFPL_VAL_ADEM_EVAL_RF_AM_ABS_SPECTRUM                              8
#define RSFPL_VAL_ADEM_EVAL_RF_AF_SPECTRUM                                  9
#define RSFPL_VAL_ADEM_EVAL_RF_ACV_TIMEDOMAIN                               10

// RsFpl_rngADEMRelativeDemodulationResultsUnit
#define RSFPL_VAL_ADEM_UNIT_PCT                                             0
#define RSFPL_VAL_ADEM_UNIT_DB                                              1

// RsFpl_rngADEMType
#define RSFPL_VAL_ADEM_AM                                                   0
#define RSFPL_VAL_ADEM_FM                                                   1
#define RSFPL_VAL_ADEM_PM                                                   2

// RsFpl_rngADEMSummaryMeasType
#define RSFPL_VAL_ADEM_SUMMARY_TYPE_PPE                                     0
#define RSFPL_VAL_ADEM_SUMMARY_TYPE_NPE                                     1
#define RSFPL_VAL_ADEM_SUMMARY_TYPE_AVERAGE                                 2
#define RSFPL_VAL_ADEM_SUMMARY_TYPE_RMS                                     3

// RsFpl_rngADEMLayoutWindowType
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_RSUM                                     0
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_MTABLE                                   1
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_PEAKLIST                                 2
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_RF_TDOM                                  3
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_AM_TDOM                                  4
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_AM_SPECTRUM                              5
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_FM_TDOM                                  6
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_FM_SPECTRUM                              7
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_PM_TDOM                                  8
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_PM_SPECTRUM                              9
#define RSFPL_VAL_ADEM_LAYOUT_TYPE_RF_SPECTRUM                              10

// RsFpl_rngADEMUnitAngle
#define RSFPL_VAL_UNIT_ANGLE_RAD                                            0
#define RSFPL_VAL_UNIT_ANGLE_DEG                                            1

// RsFpl_rngADEMUnitTHD
#define RSFPL_VAL_UNIT_THD_DB                                               0
#define RSFPL_VAL_UNIT_THD_PCT                                              1

// RsFpl_rngVSANormalizeEvm
#define RSFPL_VAL_VSA_NORMALIZE_EVM_SIGNAL                                  0
#define RSFPL_VAL_VSA_NORMALIZE_EVM_SYMBOL                                  1
#define RSFPL_VAL_VSA_NORMALIZE_EVM_MECPOWER                                2
#define RSFPL_VAL_VSA_NORMALIZE_EVM_MACPOWER                                3

// RsFpl_rngVSAEqFileFormat
#define RSFPL_VAL_VSA_EQ_FILE_FORMAT_VAE                                    0
#define RSFPL_VAL_VSA_EQ_FILE_FORMAT_FRES                                   1

// RsFpl_rngVSADemodulationEQMode
#define RSFPL_VAL_VSA_DEMODULATION_EQ_MODE_NORMAL                           0
#define RSFPL_VAL_VSA_DEMODULATION_EQ_MODE_TRACKING                         1
#define RSFPL_VAL_VSA_DEMODULATION_EQ_MODE_FREEZE                           2
#define RSFPL_VAL_VSA_DEMODULATION_EQ_MODE_USER                             3
#define RSFPL_VAL_VSA_DEMODULATION_EQ_MODE_AVERAGING                        4

// RsFpl_rngVSARestoresFactory
#define RSFPL_VAL_VSA_RESTORES_FACTORY_ALL                                  0
#define RSFPL_VAL_VSA_RESTORES_FACTORY_STANDARD                             1
#define RSFPL_VAL_VSA_RESTORES_FACTORY_PATTERN                              2

// RsFpl_rngVSAModulationType
#define RSFPL_VAL_VSA_MODULATION_TYPE_QPSK                                  0
#define RSFPL_VAL_VSA_MODULATION_TYPE_PSK                                   1
#define RSFPL_VAL_VSA_MODULATION_TYPE_MSK                                   2
#define RSFPL_VAL_VSA_MODULATION_TYPE_QAM                                   3
#define RSFPL_VAL_VSA_MODULATION_TYPE_FSK                                   4
#define RSFPL_VAL_VSA_MODULATION_TYPE_ASK                                   5
#define RSFPL_VAL_VSA_MODULATION_TYPE_APSK                                  6
#define RSFPL_VAL_VSA_MODULATION_TYPE_UQAM                                  7

// RsFpl_rngVSAFineSyncMode
#define RSFPL_VAL_VSA_FINE_SYNC_KDATA                                       0
#define RSFPL_VAL_VSA_FINE_SYNC_PATTERN                                     1
#define RSFPL_VAL_VSA_FINE_SYNC_DDATA                                       2

// RsFpl_rngVSAModulationMSKFormat
#define RSFPL_VAL_VSA_MODULATION_MSK_FORMAT_TYPE1                           0
#define RSFPL_VAL_VSA_MODULATION_MSK_FORMAT_TYPE2                           1

// RsFpl_rngVSADemodulationOptimization
#define RSFPL_VAL_VSA_DEMODULATION_OPTIMIZATION_RMS_MIN                     0
#define RSFPL_VAL_VSA_DEMODULATION_OPTIMIZATION_EVM_MIN                     1

// RsFpl_rngVSAFrameModulation
#define RSFPL_VAL_VSA_FRAME_MODULATION_DATA                                 0
#define RSFPL_VAL_VSA_FRAME_MODULATION_PATTERN                              1
#define RSFPL_VAL_VSA_FRAME_MODULATION_AUTO                                 2

// RsFpl_rngVSAFrameMode
#define RSFPL_VAL_VSA_FRAME_MODE_DEFAULT                                    0
#define RSFPL_VAL_VSA_FRAME_MODE_USER                                       1

// RsFpl_rngVSAPSKFormat
#define RSFPL_VAL_VSA_PSK_FORMAT_NORMAL                                     0
#define RSFPL_VAL_VSA_PSK_FORMAT_DIFFERENTIAL                               1

// RsFpl_rngVSAModulationQAMFormat
#define RSFPL_VAL_VSA_MODULATION_NORMAL                                     0
#define RSFPL_VAL_VSA_MODULATION_DIFFERENTIAL                               1
#define RSFPL_VAL_VSA_MODULATION_NPI4                                       2
#define RSFPL_VAL_VSA_MODULATION_MNPI4                                      3

// RsFpl_rngVSAModulationQPSKFormat
#define RSFPL_VAL_VSA_MODULATION_DPI4                                       3
#define RSFPL_VAL_VSA_MODULATION_OFFSET                                     4
#define RSFPL_VAL_VSA_MODULATION_N3PI4                                      5
#define RSFPL_VAL_VSA_MODULATION_SOFFSET                                    6

// RsFpl_rngVSABurstSearchMeas
#define RSFPL_VAL_VSA_BURST_SEARCH_MEAS_MEAS                                0
#define RSFPL_VAL_VSA_BURST_SEARCH_MEAS_BURST                               1

// RsFpl_rngVSAStructurePatterns
#define RSFPL_VAL_VSA_STRUCTURE_PATT_CURRENT                                0
#define RSFPL_VAL_VSA_STRUCTURE_PATT_ALL                                    1

// RsFpl_rngVSAPatternSyncMode
#define RSFPL_VAL_VSA_PATTERN_SYNC_MODE_MEAS                                0
#define RSFPL_VAL_VSA_PATTERN_SYNC_MODE_SYNC                                1

// RsFpl_rngVSAResultType
#define RSFPL_VAL_VSA_RESULT_TYPE_MAGNITUDE                                 0
#define RSFPL_VAL_VSA_RESULT_TYPE_PHASE                                     1
#define RSFPL_VAL_VSA_RESULT_TYPE_UPHASE                                    2
#define RSFPL_VAL_VSA_RESULT_TYPE_RIMAG                                     3
#define RSFPL_VAL_VSA_RESULT_TYPE_FREQUENCY                                 4
#define RSFPL_VAL_VSA_RESULT_TYPE_COMP                                      5
#define RSFPL_VAL_VSA_RESULT_TYPE_CONS                                      6
#define RSFPL_VAL_VSA_RESULT_TYPE_IEYE                                      7
#define RSFPL_VAL_VSA_RESULT_TYPE_QEYE                                      8
#define RSFPL_VAL_VSA_RESULT_TYPE_FEYE                                      9
#define RSFPL_VAL_VSA_RESULT_TYPE_CONF                                      10
#define RSFPL_VAL_VSA_RESULT_TYPE_COVF                                      11
#define RSFPL_VAL_VSA_RESULT_TYPE_RCONSTELLATION                            12
#define RSFPL_VAL_VSA_RESULT_TYPE_RSUMMARY                                  13
#define RSFPL_VAL_VSA_RESULT_TYPE_BERATE                                    14
#define RSFPL_VAL_VSA_RESULT_TYPE_GDELAY                                    15
#define RSFPL_VAL_VSA_RESULT_TYPE_BIN                                       16
#define RSFPL_VAL_VSA_RESULT_TYPE_OCT                                       17
#define RSFPL_VAL_VSA_RESULT_TYPE_DEC                                       18
#define RSFPL_VAL_VSA_RESULT_TYPE_HEX                                       19

// RsFpl_rngVSAMeasResultType
#define RSFPL_VAL_VSA_MEAS_RESULT_TYPE_NONE                                 0
#define RSFPL_VAL_VSA_MEAS_RESULT_TYPE_AVG                                  1
#define RSFPL_VAL_VSA_MEAS_RESULT_TYPE_RPE                                  2
#define RSFPL_VAL_VSA_MEAS_RESULT_TYPE_SDEV                                 3
#define RSFPL_VAL_VSA_MEAS_RESULT_TYPE_PCTL                                 4

// RsFpl_rngVSAResultOversampling
#define RSFPL_VAL_VSA_RESULT_OVERSAMPLING_SONLY                             0
#define RSFPL_VAL_VSA_RESULT_OVERSAMPLING_INFINITE                          1

// RsFpl_rngVSAResultAlignment
#define RSFPL_VAL_VSA_RESULT_ALIGNEMNT_LEFT                                 0
#define RSFPL_VAL_VSA_RESULT_ALIGNEMNT_CENTER                               1
#define RSFPL_VAL_VSA_RESULT_ALIGNEMNT_RIGHT                                2

// RsFpl_rngVSAResultReference
#define RSFPL_VAL_VSA_RESULT_REFERENCE_TRIGGER                              0
#define RSFPL_VAL_VSA_RESULT_REFERENCE_BURST                                1
#define RSFPL_VAL_VSA_RESULT_REFERENCE_PATTERN                              2

// RsFpl_rngVSATraceSymbolSelection
#define RSFPL_VAL_VSA_TRACE_SYMBOL_SEL_ALL                                  0
#define RSFPL_VAL_VSA_TRACE_SYMBOL_SEL_PATTERN                              1
#define RSFPL_VAL_VSA_TRACE_SYMBOL_SEL_DATA                                 2

// RsFpl_rngVSAResultIndividual
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_ALL                                 0
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_EVMR                                1
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_EVMP                                2
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_PERM                                3
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_PEP                                 4
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_MERM                                5
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_MEP                                 6
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_CFER                                7
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_RHO                                 8
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_IQOF                                9
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_FERM                                10
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_FEP                                 11
#define RSFPL_VAL_VSA_RESULT_INDIVIDUAL_FDER                                12

// RsFpl_rngVSAOptimizationMode
#define RSFPL_VAL_VSA_OPTIMIZATION_MODE_LNOISE                              0
#define RSFPL_VAL_VSA_OPTIMIZATION_MODE_LDISTORTION                         1

// RsFpl_rngVSAPadType
#define RSFPL_VAL_VSA_PAD_TYPE_SRESISTOR                                    0
#define RSFPL_VAL_VSA_PAD_TYPE_MLPAD                                        1

// RsFpl_rngVSAIQFormat
#define RSFPL_VAL_VSA_IQ_FORMAT_FLOAT32                                     0
#define RSFPL_VAL_VSA_IQ_FORMAT_INT32                                       1

// RsFpl_rngVSAIQDataFormat
#define RSFPL_VAL_VSA_IQ_DATA_FORMAT_COMPLEX                                0
#define RSFPL_VAL_VSA_IQ_DATA_FORMAT_REAL                                   1

// RsFpl_rngVSAModulationAPSKNState
#define RSFPL_VAL_VSA_MODULATION_APSKN16                                    16
#define RSFPL_VAL_VSA_MODULATION_APSKN32                                    32

// RsFpl_rngVSAModulationASKNState
#define RSFPL_VAL_VSA_MODULATION_ASKN2                                      2
#define RSFPL_VAL_VSA_MODULATION_ASKN4                                      4

// RsFpl_rngVSAModulationPSKFormat
#define RSFPL_VAL_VSA_MODULATION_PSK_FORMAT_NORMAL                          0
#define RSFPL_VAL_VSA_MODULATION_PSK_FORMAT_DIFF                            1
#define RSFPL_VAL_VSA_MODULATION_PSK_FORMAT_N3PI8                           2
#define RSFPL_VAL_VSA_MODULATION_PSK_FORMAT_PI8D8PSK                        3
#define RSFPL_VAL_VSA_MODULATION_PSK_FORMAT_NPI2                            4
#define RSFPL_VAL_VSA_MODULATION_PSK_FORMAT_DPI2                            5
#define RSFPL_VAL_VSA_MODULATION_PSK_FORMAT_MNPI2                           6

// RsFpl_rngVSASignalStructureType
#define RSFPL_VAL_VSA_STRUCTURE_TYPE_CONTINUOUS                             0
#define RSFPL_VAL_VSA_STRUCTURE_TYPE_BURSTED                                1

// RsFpl_rngVSAKDataSource
#define RSFPL_VAL_KDATA_SOURCE_FILE                                         0
#define RSFPL_VAL_KDATA_SOURCE_PRBS                                         1

// RsFpl_rngVSAScallingTimeUnit
#define RSFPL_VAL_VSA_TIME_UNIT_S                                           0
#define RSFPL_VAL_VSA_TIME_UNIT_SYM                                         1

// RsFpl_rngVSADemodulationCoarse
#define RSFPL_VAL_VSA_DEMODULATION_COARSE_AUTO                              0
#define RSFPL_VAL_VSA_DEMODULATION_COARSE_MANUAL                            1

// RsFpl_rngVSATraceEvaluation
#define RSFPL_VSA_TRACE_EVALUATION_MEAS                                     0
#define RSFPL_VSA_TRACE_EVALUATION_REF                                      1
#define RSFPL_VSA_TRACE_EVALUATION_ERR                                      2
#define RSFPL_VSA_TRACE_EVALUATION_TCAP                                     3

// RsFpl_rngVSAMACCLimitResult
#define RSFPL_VAL_VSA_MACC_LIMIT_RESULT_NONE                                -1
#define RSFPL_VAL_VSA_MACC_LIMIT_RESULT_PASS                                0
#define RSFPL_VAL_VSA_MACC_LIMIT_RESULT_FAIL                                1
#define RSFPL_VAL_VSA_MACC_LIMIT_RESULT_MARGIN                              2

// RsFpl_rngVSASearchAuto
#define RSFPL_VAL_VSA_SEARCH_AUTO                                           0
#define RSFPL_VAL_VSA_SEARCH_MANUAL                                         1

// RsFpl_rngVSAResultPPSMode
#define RSFPL_VAL_VSA_RESULT_PPS_MODE_AUTO                                  0
#define RSFPL_VAL_VSA_RESULT_PPS_MODE_MANUAL                                1

// RsFpl_rngVSAMeasResultTypeEVM
#define RSFPL_VAL_VSA_RESULTS_EVM_NONE                                      0
#define RSFPL_VAL_VSA_RESULTS_EVM_AVG                                       1
#define RSFPL_VAL_VSA_RESULTS_EVM_PAVG                                      2
#define RSFPL_VAL_VSA_RESULTS_EVM_PCTL                                      3
#define RSFPL_VAL_VSA_RESULTS_EVM_PEAK                                      4
#define RSFPL_VAL_VSA_RESULTS_EVM_PPCT                                      5
#define RSFPL_VAL_VSA_RESULTS_EVM_PSD                                       6
#define RSFPL_VAL_VSA_RESULTS_EVM_RPE                                       7
#define RSFPL_VAL_VSA_RESULTS_EVM_SDEV                                      8
#define RSFPL_VAL_VSA_RESULTS_EVM_TPE                                       9

// RsFpl_rngVSAStatusRegister
#define RSFPL_VAL_VSA_STATUS_REGISTER_MAIN                                  0
#define RSFPL_VAL_VSA_STATUS_REGISTER_CFR                                   1
#define RSFPL_VAL_VSA_STATUS_REGISTER_EVM                                   2
#define RSFPL_VAL_VSA_STATUS_REGISTER_FSK                                   3
#define RSFPL_VAL_VSA_STATUS_REGISTER_IQRH                                  4
#define RSFPL_VAL_VSA_STATUS_REGISTER_MAGN                                  5
#define RSFPL_VAL_VSA_STATUS_REGISTER_PHAS                                  6

// RsFpl_rngEMIMeasurementDetector
#define RSFPL_VAL_EMI_DETECTOR_TYPE_OFF                                     0
#define RSFPL_VAL_EMI_DETECTOR_TYPE_AVER                                    1
#define RSFPL_VAL_EMI_DETECTOR_TYPE_CAV                                     2
#define RSFPL_VAL_EMI_DETECTOR_TYPE_CRMS                                    3
#define RSFPL_VAL_EMI_DETECTOR_TYPE_POS                                     4
#define RSFPL_VAL_EMI_DETECTOR_TYPE_QPE                                     5

// RsFpl_rngProbability
#define RSFPL_VAL_PROBABILITY_0_01                                          0
#define RSFPL_VAL_PROBABILITY_0_1                                           1
#define RSFPL_VAL_PROBABILITY_1                                             2
#define RSFPL_VAL_PROBABILITY_10                                            3

// RsFpl_rngCalibrationSignalPeakDistance
#define RSFPL_VAL_PEAK_DISTANCE_SMALL                                       0
#define RSFPL_VAL_PEAK_DISTANCE_WIDE                                        1

// RsFpl_rngRFInputSAWFilter
#define RSFPL_VAL_SAW_FILTER_OFF                                            0
#define RSFPL_VAL_SAW_FILTER_AUTO                                           1

// RsFpl_rngEMILISNVNetworkPhase
#define RSFPL_VAL_EMI_VNET_PHAS_N                                           0
#define RSFPL_VAL_EMI_VNET_PHAS_L1                                          1
#define RSFPL_VAL_EMI_VNET_PHAS_L2                                          2
#define RSFPL_VAL_EMI_VNET_PHAS_L3                                          3

// RsFpl_rngEMILISNVNetworkType
#define RSFPL_VAL_EMI_VNET_TYPE_OFF                                         0
#define RSFPL_VAL_EMI_VNET_TYPE_216                                         1
#define RSFPL_VAL_EMI_VNET_TYPE_4200                                        2
#define RSFPL_VAL_EMI_VNET_TYPE_ESH2Z5                                      3
#define RSFPL_VAL_EMI_VNET_TYPE_ESH3Z5                                      4
#define RSFPL_VAL_EMI_VNET_TYPE_TWO                                         5
#define RSFPL_VAL_EMI_VNET_TYPE_FOUR                                        6
#define RSFPL_VAL_EMI_VNET_TYPE_ENV432                                      8

// RsFpl_rngRFInput
#define RSFPL_VAL_RF_INPUT_RF                                               0

// RsFpl_rngDataInstrumentSettingsRecallMode
#define RSFPL_VAL_RECALL_MODE_NEW                                           0
#define RSFPL_VAL_RECALL_MODE_REPLACE                                       1

// RsFpl_rngMessageType
#define RSFPL_VAL_MESSAGE_TYPE_ALL                                          0
#define RSFPL_VAL_MESSAGE_TYPE_INFO                                         1
#define RSFPL_VAL_MESSAGE_TYPE_WARNING                                      2
#define RSFPL_VAL_MESSAGE_TYPE_FATAL                                        3
#define RSFPL_VAL_MESSAGE_TYPE_ERROR                                        4
#define RSFPL_VAL_MESSAGE_TYPE_MESSAGE                                      5

// RsFpl_rngSignalStatisticScalingYUnit
#define RSFPL_VAL_PCT                                                       1

// RsFpl_rngStatisticResultType
#define RSFPL_VAL_STAT_RESULT_MEAN                                          0
#define RSFPL_VAL_STAT_RESULT_PEAK                                          1
#define RSFPL_VAL_STAT_RESULT_CFAC                                          2
#define RSFPL_VAL_STAT_RESULT_ALL                                           3

// RsFpl_rngSpectrogramSelectTrace
#define RSFPL_VAL_TRACE_1                                                   1
#define RSFPL_VAL_TRACE_2                                                   2
#define RSFPL_VAL_TRACE_3                                                   3
#define RSFPL_VAL_TRACE_4                                                   4
#define RSFPL_VAL_TRACE_5                                                   5
#define RSFPL_VAL_TRACE_6                                                   6

// RsFpl_rngLinkedMarkers
#define RSFPL_VAL_LINKED_MARKERS_NONE                                       0
#define RSFPL_VAL_LINKED_MARKERS_TIME                                       1
#define RSFPL_VAL_LINKED_MARKERS_SPECTRUM                                   2
#define RSFPL_VAL_LINKED_MARKERS_BOTH                                       3

// RsFpl_rngDisplayFormat
#define RSFPL_VAL_DISPLAY_FORMAT_SINGLE                                     0
#define RSFPL_VAL_DISPLAY_FORMAT_SPLIT                                      1

// RsFpl_rngLOType
#define RSFPL_VAL_LO_TYPE_FIXED                                             0
#define RSFPL_VAL_LO_TYPE_VARIABLE                                          1

// RsFpl_rngCalibrationMode
#define RSFPL_VAL_CALIBRATION_MODE_SHORT_OR_THROUGH                         0
#define RSFPL_VAL_CALIBRATION_MODE_OPEN                                     1

// RsFpl_rngCalibrationMethod
#define RSFPL_VAL_CALIBRATION_METHOD_REFLECTION                             0
#define RSFPL_VAL_CALIBRATION_METHOD_TRANSMISSION                           1

// RsFpl_rngXDistribution
#define RSFPL_VAL_X_DISTRIBUTION_BINCENTERED                                0
#define RSFPL_VAL_X_DISTRIBUTION_START_STOP                                 1

// RsFpl_rngHCopyMode
#define RSFPL_VAL_HCOPY_MODE_SCREEN                                         0
#define RSFPL_VAL_HCOPY_MODE_REPORT                                         1

// RsFpl_rngHeaderLineVisibility
#define RSFPL_VAL_TEST_REPORT_ITEM_VISIBILITY_NEVER                         0
#define RSFPL_VAL_TEST_REPORT_ITEM_VISIBILITY_GLOBAL                        1
#define RSFPL_VAL_TEST_REPORT_ITEM_VISIBILITY_SUBREPORT                     2

// RsFpl_rngPageFormat
#define RSFPL_VAL_PAGE_FORMAT_A4                                            0
#define RSFPL_VAL_PAGE_FORMAT_US_LETTER                                     1

// RsFpl_rngGeneratorPowerMode
#define RSFPL_VAL_GENERATOR_POWER_MODE_FIXED                                0
#define RSFPL_VAL_GENERATOR_POWER_MODE_SWEEP                                1

// RsFpl_rngTestReportChannelType
#define RSFPL_VAL_TEST_REPORT_CHANNEL_ACTIVE                                0
#define RSFPL_VAL_TEST_REPORT_CHANNEL_SPECTRUM                              1

// RsFpl_rngTestReportItems
#define RSFPL_VAL_TEST_REPORT_ITEM_DIAGRAM                                  0
#define RSFPL_VAL_TEST_REPORT_ITEM_MARKERS                                  1
#define RSFPL_VAL_TEST_REPORT_ITEM_PEAK_LIST                                2
#define RSFPL_VAL_TEST_REPORT_ITEM_RESULT_SUMMARY                           3
#define RSFPL_VAL_TEST_REPORT_ITEM_SETTINGS                                 4
#define RSFPL_VAL_TEST_REPORT_ITEM_SPECTROGRAM                              5
#define RSFPL_VAL_TEST_REPORT_ITEM_RESULT_LIST                              6
#define RSFPL_VAL_TEST_REPORT_ITEM_TRANSDUCER                               7
#define RSFPL_VAL_TEST_REPORT_ITEM_LISN                                     8

// RsFpl_rngPhaseNoiseSweepMode
#define RSFPL_VAL_PHASE_NOISE_SWEEP_MODE_NORMAL                             0
#define RSFPL_VAL_PHASE_NOISE_SWEEP_MODE_IQ_FFT                             1

// RsFpl_rngPhaseNoiseLimitLineType
#define RSFPL_VAL_PHASE_NOISE_LIMIT_NONE                                    0
#define RSFPL_VAL_PHASE_NOISE_LIMIT_FC1                                     1
#define RSFPL_VAL_PHASE_NOISE_LIMIT_FC2                                     2
#define RSFPL_VAL_PHASE_NOISE_LIMIT_FC3                                     3
#define RSFPL_VAL_PHASE_NOISE_LIMIT_FC4                                     4
#define RSFPL_VAL_PHASE_NOISE_LIMIT_FC5                                     5

// RsFpl_rngPhaseNoiseTraceSmoothing
#define RSFPL_VAL_PHASE_NOISE_SMOOTHING_LINEAR                              0
#define RSFPL_VAL_PHASE_NOISE_SMOOTHING_LOGARITHMIC                         1
#define RSFPL_VAL_PHASE_NOISE_SMOOTHING_MEDIAN                              2

// RsFpl_rngPhaseNoiseHalfDecade
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_1HZ_3HZ                           0
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_3HZ_10HZ                          1
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_10HZ_30HZ                         2
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_30HZ_100HZ                        3
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_100HZ_300HZ                       4
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_300HZ_1KHZ                        5
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_1KHZ_3KHZ                         6
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_3KHZ_10KHZ                        7
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_10KHZ_30KHZ                       8
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_30KHZ_100KHZ                      9
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_100KHZ_300KHZ                     10
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_300KHZ_1MHZ                       11
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_1MHZ_3MHZ                         12
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_3MHZ_10MHZ                        13
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_10MHZ_30MHZ                       14
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_30MHZ_100MHZ                      15
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_100MHZ_300MHZ                     16
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_300MHZ_1GHZ                       17
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_1GHZ_3GHZ                         18
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_3GHZ_10GHZ                        19
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_10GHZ_30GHZ                       20
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_30GHZ_100GHZ                      21
#define RSFPL_VAL_PHASE_NOISE_HALF_DECADE_100GHZ_300GHZ                     22

// RsFpl_rngPhaseNoiseXAxisScope
#define RSFPL_VAL_PHASE_NOISE_X_AXIS_SCOPE_MEAS_RANGE                       0
#define RSFPL_VAL_PHASE_NOISE_X_AXIS_SCOPE_HALF_DECADE                      1
#define RSFPL_VAL_PHASE_NOISE_X_AXIS_SCOPE_MANUAL                           2

// RsFpl_rngPhaseNoiseScalingMethod
#define RSFPL_VAL_PHASE_NOISE_SCALING_AUTO                                  0
#define RSFPL_VAL_PHASE_NOISE_SCALING_TOP_BOTTOM                            1
#define RSFPL_VAL_PHASE_NOISE_SCALING_TOP_RANGE                             2
#define RSFPL_VAL_PHASE_NOISE_SCALING_BOTTOM_RANGE                          3

// RsFpl_rngPhaseNoiseOnVerifyFailed
#define RSFPL_VAL_PHASE_NOISE_ON_VERIFY_FAILED_RESTART                      0
#define RSFPL_VAL_PHASE_NOISE_ON_VERIFY_FAILED_STOP                         1
#define RSFPL_VAL_PHASE_NOISE_ON_VERIFY_FAILED_AUTO                         2

// RsFpl_rngPhaseNoiseMeasPreset
#define RSFPL_VAL_PHASE_NOISE_PRESET_NORMAL                                 0
#define RSFPL_VAL_PHASE_NOISE_PRESET_AVERAGE                                1
#define RSFPL_VAL_PHASE_NOISE_PRESET_FAST                                   2

// RsFpl_rngPhaseNoiseHalfDecadeWindowType
#define RSFPL_VAL_PHASE_NOISE_WINDOW_TYPE_RECTANGULAR                       0
#define RSFPL_VAL_PHASE_NOISE_WINDOW_TYPE_GAUSSIAN                          1
#define RSFPL_VAL_PHASE_NOISE_WINDOW_TYPE_CHEBYSHEV                         2
#define RSFPL_VAL_PHASE_NOISE_WINDOW_TYPE_BLACKMAN_HARRIS                   3
#define RSFPL_VAL_PHASE_NOISE_WINDOW_TYPE_SWEPT                             4

// RsFpl_rngPhaseNoiseUserRangeTrace
#define RSFPL_VAL_TRACE_NONE                                                0

// RsFpl_rngWindowTracePreset
#define RSFPL_VAL_TRACES_PRESET_ALL                                         0
#define RSFPL_VAL_TRACES_PRESET_MAX_AVG_MIN                                 1
#define RSFPL_VAL_TRACES_PRESET_MAX_CLRWRITE_MIN                            2

// RsFpl_rngVSABitErrorRateResult
#define RSFPL_VAL_VSA_BER_CURRENT                                           0
#define RSFPL_VAL_VSA_BER_MIN                                               1
#define RSFPL_VAL_VSA_BER_MAX                                               2
#define RSFPL_VAL_VSA_BER_TOTAL                                             3
#define RSFPL_VAL_VSA_BER_CURRENT_BIT_COUNT                                 4
#define RSFPL_VAL_VSA_BER_MIN_BIT_COUNT                                     5
#define RSFPL_VAL_VSA_BER_MAX_BIT_COUNT                                     6
#define RSFPL_VAL_VSA_BER_TOTAL_BIT_COUNT                                   7
#define RSFPL_VAL_VSA_BER_CURRENT_BIT_ERRORS                                8
#define RSFPL_VAL_VSA_BER_MIN_BIT_ERRORS                                    9
#define RSFPL_VAL_VSA_BER_MAX_BIT_ERRORS                                    10
#define RSFPL_VAL_VSA_BER_TOTAL_BIT_ERRORS                                  11
#define RSFPL_VAL_VSA_BER_CURRENT_SYMBOL_ERRORS                             12
#define RSFPL_VAL_VSA_BER_MIN_SYMBOL_ERRORS                                 13
#define RSFPL_VAL_VSA_BER_MAX_SYMBOL_ERRORS                                 14
#define RSFPL_VAL_VSA_BER_TOTAL_SYMBOL_ERRORS                               15

// RsFpl_rngSweepTypeMode
#define RSFPL_VAL_SWEEP_TYPE_MODE_AUTO                                      0
#define RSFPL_VAL_SWEEP_TYPE_MODE_FFT                                       1
#define RSFPL_VAL_SWEEP_TYPE_MODE_SWE                                       2

// RsFpl_rngFrequencyAnnotationMode
#define RSFPL_VAL_FREQUENCY_ANNOTATION_MODE_CSP                             0
#define RSFPL_VAL_FREQUENCY_ANNOTATION_MODE_SST                             1

// RsFpl_rngADEMZeroPhaseReferencePositionMode
#define RSFPL_VAL_ADEM_ZERO_PHASE_REFERENCE_POSITION_MODE_MANUAL            0
#define RSFPL_VAL_ADEM_ZERO_PHASE_REFERENCE_POSITION_MODE_RIGHT             1

// RsFpl_rngAnalogDemodulationResultRangeResultType
#define RSFPL_VAL_ANALOG_DEMODULATION_RESULT_RANGE_RESULT_TYPE_PHAS         0
#define RSFPL_VAL_ANALOG_DEMODULATION_RESULT_RANGE_RESULT_TYPE_UPH          1

// RsFpl_rngAnalogDemodulationUnitTHD
#define RSFPL_VAL_ANALOG_DEMODULATION_UNIT_THD_DB                           0
#define RSFPL_VAL_ANALOG_DEMODULATION_UNIT_THD_PCT                          1

//repcap Window
#define RSFPL_REPCAP_WINDOW_WIN0                                            0
#define RSFPL_REPCAP_WINDOW_WIN1                                            1
#define RSFPL_REPCAP_WINDOW_WIN2                                            2
#define RSFPL_REPCAP_WINDOW_WIN3                                            3
#define RSFPL_REPCAP_WINDOW_WIN4                                            4
#define RSFPL_REPCAP_WINDOW_WIN5                                            5
#define RSFPL_REPCAP_WINDOW_WIN6                                            6
#define RSFPL_REPCAP_WINDOW_WIN7                                            7
#define RSFPL_REPCAP_WINDOW_WIN8                                            8
#define RSFPL_REPCAP_WINDOW_WIN9                                            9
#define RSFPL_REPCAP_WINDOW_WIN10                                           10
#define RSFPL_REPCAP_WINDOW_WIN11                                           11
#define RSFPL_REPCAP_WINDOW_WIN12                                           12
#define RSFPL_REPCAP_WINDOW_WIN13                                           13
#define RSFPL_REPCAP_WINDOW_WIN14                                           14
#define RSFPL_REPCAP_WINDOW_WIN15                                           15
#define RSFPL_REPCAP_WINDOW_WIN16                                           16

//repcap Marker
#define RSFPL_REPCAP_MARKER_M1                                              1
#define RSFPL_REPCAP_MARKER_M2                                              2
#define RSFPL_REPCAP_MARKER_M3                                              3
#define RSFPL_REPCAP_MARKER_M4                                              4
#define RSFPL_REPCAP_MARKER_M5                                              5
#define RSFPL_REPCAP_MARKER_M6                                              6
#define RSFPL_REPCAP_MARKER_M7                                              7
#define RSFPL_REPCAP_MARKER_M8                                              8
#define RSFPL_REPCAP_MARKER_M9                                              9
#define RSFPL_REPCAP_MARKER_M10                                             10
#define RSFPL_REPCAP_MARKER_M11                                             11
#define RSFPL_REPCAP_MARKER_M12                                             12
#define RSFPL_REPCAP_MARKER_M13                                             13
#define RSFPL_REPCAP_MARKER_M14                                             14
#define RSFPL_REPCAP_MARKER_M15                                             15
#define RSFPL_REPCAP_MARKER_M16                                             16

//repcap Trace
#define RSFPL_REPCAP_TRACE_TR1                                              1
#define RSFPL_REPCAP_TRACE_TR2                                              2
#define RSFPL_REPCAP_TRACE_TR3                                              3
#define RSFPL_REPCAP_TRACE_TR4                                              4
#define RSFPL_REPCAP_TRACE_TR5                                              5
#define RSFPL_REPCAP_TRACE_TR6                                              6

//repcap PowerSensor
#define RSFPL_REPCAP_POWERSENSOR_PMET1                                      1
#define RSFPL_REPCAP_POWERSENSOR_PMET2                                      2
#define RSFPL_REPCAP_POWERSENSOR_PMET3                                      3
#define RSFPL_REPCAP_POWERSENSOR_PMET4                                      4

//repcap Subblock
#define RSFPL_REPCAP_SUBBLOCK_SB1                                           1
#define RSFPL_REPCAP_SUBBLOCK_SB2                                           2
#define RSFPL_REPCAP_SUBBLOCK_SB3                                           3

//repcap Channel
#define RSFPL_REPCAP_CHANNEL_CH1                                            1
#define RSFPL_REPCAP_CHANNEL_CH2                                            2
#define RSFPL_REPCAP_CHANNEL_CH3                                            3
#define RSFPL_REPCAP_CHANNEL_CH4                                            4
#define RSFPL_REPCAP_CHANNEL_CH5                                            5
#define RSFPL_REPCAP_CHANNEL_CH6                                            6
#define RSFPL_REPCAP_CHANNEL_CH7                                            7
#define RSFPL_REPCAP_CHANNEL_CH8                                            8
#define RSFPL_REPCAP_CHANNEL_CH9                                            9
#define RSFPL_REPCAP_CHANNEL_CH10                                           10
#define RSFPL_REPCAP_CHANNEL_CH11                                           11
#define RSFPL_REPCAP_CHANNEL_CH12                                           12
#define RSFPL_REPCAP_CHANNEL_CH13                                           13
#define RSFPL_REPCAP_CHANNEL_CH14                                           14
#define RSFPL_REPCAP_CHANNEL_CH15                                           15
#define RSFPL_REPCAP_CHANNEL_CH16                                           16
#define RSFPL_REPCAP_CHANNEL_CH17                                           17
#define RSFPL_REPCAP_CHANNEL_CH18                                           18

//repcap AlternateChannel
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT1                                  1
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT2                                  2
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT3                                  3
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT4                                  4
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT5                                  5
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT6                                  6
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT7                                  7
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT8                                  8
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT9                                  9
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT10                                 10
#define RSFPL_REPCAP_ALTERNATECHANNEL_ALT11                                 11

//repcap Range
#define RSFPL_REPCAP_RANGE_R1                                               1
#define RSFPL_REPCAP_RANGE_R2                                               2
#define RSFPL_REPCAP_RANGE_R3                                               3
#define RSFPL_REPCAP_RANGE_R4                                               4
#define RSFPL_REPCAP_RANGE_R5                                               5
#define RSFPL_REPCAP_RANGE_R6                                               6
#define RSFPL_REPCAP_RANGE_R7                                               7
#define RSFPL_REPCAP_RANGE_R8                                               8
#define RSFPL_REPCAP_RANGE_R9                                               9
#define RSFPL_REPCAP_RANGE_R10                                              10
#define RSFPL_REPCAP_RANGE_R11                                              11
#define RSFPL_REPCAP_RANGE_R12                                              12
#define RSFPL_REPCAP_RANGE_R13                                              13
#define RSFPL_REPCAP_RANGE_R14                                              14
#define RSFPL_REPCAP_RANGE_R15                                              15
#define RSFPL_REPCAP_RANGE_R16                                              16
#define RSFPL_REPCAP_RANGE_R17                                              17
#define RSFPL_REPCAP_RANGE_R18                                              18
#define RSFPL_REPCAP_RANGE_R19                                              19
#define RSFPL_REPCAP_RANGE_R20                                              20
#define RSFPL_REPCAP_RANGE_R21                                              21
#define RSFPL_REPCAP_RANGE_R22                                              22
#define RSFPL_REPCAP_RANGE_R23                                              23
#define RSFPL_REPCAP_RANGE_R24                                              24
#define RSFPL_REPCAP_RANGE_R25                                              25
#define RSFPL_REPCAP_RANGE_R26                                              26
#define RSFPL_REPCAP_RANGE_R27                                              27
#define RSFPL_REPCAP_RANGE_R28                                              28
#define RSFPL_REPCAP_RANGE_R29                                              29
#define RSFPL_REPCAP_RANGE_R30                                              30

//repcap PowerClass
#define RSFPL_REPCAP_POWERCLASS_PCL1                                        1
#define RSFPL_REPCAP_POWERCLASS_PCL2                                        2
#define RSFPL_REPCAP_POWERCLASS_PCL3                                        3
#define RSFPL_REPCAP_POWERCLASS_PCL4                                        4

//repcap ZoomWindow
#define RSFPL_REPCAP_ZOOMWINDOW_Z1                                          1
#define RSFPL_REPCAP_ZOOMWINDOW_Z2                                          2
#define RSFPL_REPCAP_ZOOMWINDOW_Z3                                          3
#define RSFPL_REPCAP_ZOOMWINDOW_Z4                                          4

//repcap DisplayLine
#define RSFPL_REPCAP_DISPLAYLINE_DL1                                        1
#define RSFPL_REPCAP_DISPLAYLINE_DL2                                        2

//repcap LimitLine
#define RSFPL_REPCAP_LIMITLINE_LL1                                          1
#define RSFPL_REPCAP_LIMITLINE_LL2                                          2
#define RSFPL_REPCAP_LIMITLINE_LL3                                          3
#define RSFPL_REPCAP_LIMITLINE_LL4                                          4
#define RSFPL_REPCAP_LIMITLINE_LL5                                          5
#define RSFPL_REPCAP_LIMITLINE_LL6                                          6
#define RSFPL_REPCAP_LIMITLINE_LL7                                          7
#define RSFPL_REPCAP_LIMITLINE_LL8                                          8

//repcap ADEMDetector
#define RSFPL_REPCAP_ADEMDETECTOR_DETPOSITIVEPEAK                           1
#define RSFPL_REPCAP_ADEMDETECTOR_DETNEGATIVEPEAK                           2
#define RSFPL_REPCAP_ADEMDETECTOR_DETAVERAGE                                3
#define RSFPL_REPCAP_ADEMDETECTOR_DETRMS                                    4

//repcap StartStop
#define RSFPL_REPCAP_STARTSTOP_SS1                                          1
#define RSFPL_REPCAP_STARTSTOP_SS2                                          2

//repcap EGateRange
#define RSFPL_REPCAP_EGATERANGE_ER1                                         1
#define RSFPL_REPCAP_EGATERANGE_ER2                                         2
#define RSFPL_REPCAP_EGATERANGE_ER3                                         3

//repcap Subwindow
#define RSFPL_REPCAP_SUBWINDOW_SW0                                          0
#define RSFPL_REPCAP_SUBWINDOW_SW1                                          1
#define RSFPL_REPCAP_SUBWINDOW_SW2                                          2
#define RSFPL_REPCAP_SUBWINDOW_SW3                                          3
#define RSFPL_REPCAP_SUBWINDOW_SW4                                          4
#define RSFPL_REPCAP_SUBWINDOW_SW5                                          5
#define RSFPL_REPCAP_SUBWINDOW_SW6                                          6
#define RSFPL_REPCAP_SUBWINDOW_SW7                                          7
#define RSFPL_REPCAP_SUBWINDOW_SW8                                          8
#define RSFPL_REPCAP_SUBWINDOW_SW9                                          9
#define RSFPL_REPCAP_SUBWINDOW_SW10                                         10
#define RSFPL_REPCAP_SUBWINDOW_SW11                                         11
#define RSFPL_REPCAP_SUBWINDOW_SW12                                         12
#define RSFPL_REPCAP_SUBWINDOW_SW13                                         13
#define RSFPL_REPCAP_SUBWINDOW_SW14                                         14
#define RSFPL_REPCAP_SUBWINDOW_SW15                                         15
#define RSFPL_REPCAP_SUBWINDOW_SW16                                         16
#define RSFPL_REPCAP_SUBWINDOW_SW17                                         17
#define RSFPL_REPCAP_SUBWINDOW_SW18                                         18
#define RSFPL_REPCAP_SUBWINDOW_SW19                                         19
#define RSFPL_REPCAP_SUBWINDOW_SW20                                         20
#define RSFPL_REPCAP_SUBWINDOW_SW21                                         21
#define RSFPL_REPCAP_SUBWINDOW_SW22                                         22
#define RSFPL_REPCAP_SUBWINDOW_SW23                                         23
#define RSFPL_REPCAP_SUBWINDOW_SW24                                         24
#define RSFPL_REPCAP_SUBWINDOW_SW25                                         25
#define RSFPL_REPCAP_SUBWINDOW_SW26                                         26
#define RSFPL_REPCAP_SUBWINDOW_SW27                                         27
#define RSFPL_REPCAP_SUBWINDOW_SW28                                         28
#define RSFPL_REPCAP_SUBWINDOW_SW29                                         29
#define RSFPL_REPCAP_SUBWINDOW_SW30                                         30
#define RSFPL_REPCAP_SUBWINDOW_SW31                                         31
#define RSFPL_REPCAP_SUBWINDOW_SW32                                         32
#define RSFPL_REPCAP_SUBWINDOW_SW33                                         33
#define RSFPL_REPCAP_SUBWINDOW_SW34                                         34
#define RSFPL_REPCAP_SUBWINDOW_SW35                                         35
#define RSFPL_REPCAP_SUBWINDOW_SW36                                         36
#define RSFPL_REPCAP_SUBWINDOW_SW37                                         37
#define RSFPL_REPCAP_SUBWINDOW_SW38                                         38
#define RSFPL_REPCAP_SUBWINDOW_SW39                                         39
#define RSFPL_REPCAP_SUBWINDOW_SW40                                         40
#define RSFPL_REPCAP_SUBWINDOW_SW41                                         41
#define RSFPL_REPCAP_SUBWINDOW_SW42                                         42
#define RSFPL_REPCAP_SUBWINDOW_SW43                                         43
#define RSFPL_REPCAP_SUBWINDOW_SW44                                         44
#define RSFPL_REPCAP_SUBWINDOW_SW45                                         45
#define RSFPL_REPCAP_SUBWINDOW_SW46                                         46
#define RSFPL_REPCAP_SUBWINDOW_SW47                                         47
#define RSFPL_REPCAP_SUBWINDOW_SW48                                         48
#define RSFPL_REPCAP_SUBWINDOW_SW49                                         49
#define RSFPL_REPCAP_SUBWINDOW_SW50                                         50
#define RSFPL_REPCAP_SUBWINDOW_SW51                                         51
#define RSFPL_REPCAP_SUBWINDOW_SW52                                         52
#define RSFPL_REPCAP_SUBWINDOW_SW53                                         53
#define RSFPL_REPCAP_SUBWINDOW_SW54                                         54
#define RSFPL_REPCAP_SUBWINDOW_SW55                                         55
#define RSFPL_REPCAP_SUBWINDOW_SW56                                         56
#define RSFPL_REPCAP_SUBWINDOW_SW57                                         57
#define RSFPL_REPCAP_SUBWINDOW_SW58                                         58
#define RSFPL_REPCAP_SUBWINDOW_SW59                                         59
#define RSFPL_REPCAP_SUBWINDOW_SW60                                         60
#define RSFPL_REPCAP_SUBWINDOW_SW61                                         61
#define RSFPL_REPCAP_SUBWINDOW_SW62                                         62
#define RSFPL_REPCAP_SUBWINDOW_SW63                                         63
#define RSFPL_REPCAP_SUBWINDOW_SW64                                         64

//repcap TriggerPort
#define RSFPL_REPCAP_TRIGGERPORT_PORT1                                      1
#define RSFPL_REPCAP_TRIGGERPORT_PORT2                                      2

//repcap HeaderLine
#define RSFPL_REPCAP_HEADERLINE_HLINE1                                      1
#define RSFPL_REPCAP_HEADERLINE_HLINE2                                      2
#define RSFPL_REPCAP_HEADERLINE_HLINE3                                      3
#define RSFPL_REPCAP_HEADERLINE_HLINE4                                      4
#define RSFPL_REPCAP_HEADERLINE_HLINE5                                      5
#define RSFPL_REPCAP_HEADERLINE_HLINE6                                      6

//repcap PNRange
#define RSFPL_REPCAP_PNRANGE_PNR1_3                                         1
#define RSFPL_REPCAP_PNRANGE_PNR3_10                                        2
#define RSFPL_REPCAP_PNRANGE_PNR10_30                                       3
#define RSFPL_REPCAP_PNRANGE_PNR30_100                                      4
#define RSFPL_REPCAP_PNRANGE_PNR100_300                                     5
#define RSFPL_REPCAP_PNRANGE_PNR300_1K                                      6
#define RSFPL_REPCAP_PNRANGE_PNR1K_3K                                       7
#define RSFPL_REPCAP_PNRANGE_PNR3K_10K                                      8
#define RSFPL_REPCAP_PNRANGE_PNR10K_30K                                     9
#define RSFPL_REPCAP_PNRANGE_PNR30K_100K                                    10
#define RSFPL_REPCAP_PNRANGE_PNR100K_300K                                   11
#define RSFPL_REPCAP_PNRANGE_PNR300K_1M                                     12
#define RSFPL_REPCAP_PNRANGE_PNR1M_3M                                       13
#define RSFPL_REPCAP_PNRANGE_PNR3M_10M                                      14
#define RSFPL_REPCAP_PNRANGE_PNR10M_30M                                     15
#define RSFPL_REPCAP_PNRANGE_PNR30M_100M                                    16
#define RSFPL_REPCAP_PNRANGE_PNR100M_300M                                   17
#define RSFPL_REPCAP_PNRANGE_PNR300M_1G                                     18
#define RSFPL_REPCAP_PNRANGE_PNR1G_3G                                       19
#define RSFPL_REPCAP_PNRANGE_PNR3G_10G                                      20
#define RSFPL_REPCAP_PNRANGE_PNR10G_30G                                     21
#define RSFPL_REPCAP_PNRANGE_PNR30G_100G                                    22
#define RSFPL_REPCAP_PNRANGE_PNR100G_300G                                   23

//repcap PNLimSeg
#define RSFPL_REPCAP_PNLIMSEG_PNLIMSEG1                                     1
#define RSFPL_REPCAP_PNLIMSEG_PNLIMSEG2                                     2
#define RSFPL_REPCAP_PNLIMSEG_PNLIMSEG3                                     3
#define RSFPL_REPCAP_PNLIMSEG_PNLIMSEG4                                     4
#define RSFPL_REPCAP_PNLIMSEG_PNLIMSEG5                                     5

//repcap SNMarker
#define RSFPL_REPCAP_SNMARKER_SNM1                                          1
#define RSFPL_REPCAP_SNMARKER_SNM2                                          2
#define RSFPL_REPCAP_SNMARKER_SNM3                                          3
#define RSFPL_REPCAP_SNMARKER_SNM4                                          4
#define RSFPL_REPCAP_SNMARKER_SNM5                                          5

//repcap RNRange
#define RSFPL_REPCAP_RNRANGE_RNRANGE1                                       1
#define RSFPL_REPCAP_RNRANGE_RNRANGE2                                       2
#define RSFPL_REPCAP_RNRANGE_RNRANGE3                                       3
#define RSFPL_REPCAP_RNRANGE_RNRANGE4                                       4
#define RSFPL_REPCAP_RNRANGE_RNRANGE5                                       5
#define RSFPL_REPCAP_RNRANGE_RNRANGE6                                       6
#define RSFPL_REPCAP_RNRANGE_RNRANGE7                                       7
#define RSFPL_REPCAP_RNRANGE_RNRANGE8                                       8
#define RSFPL_REPCAP_RNRANGE_RNRANGE9                                       9
#define RSFPL_REPCAP_RNRANGE_RNRANGE10                                      10

//repcap DeltaMarker
#define RSFPL_REPCAP_DELTAMARKER_DM1                                        1
#define RSFPL_REPCAP_DELTAMARKER_DM2                                        2
#define RSFPL_REPCAP_DELTAMARKER_DM3                                        3
#define RSFPL_REPCAP_DELTAMARKER_DM4                                        4
#define RSFPL_REPCAP_DELTAMARKER_DM5                                        5
#define RSFPL_REPCAP_DELTAMARKER_DM6                                        6
#define RSFPL_REPCAP_DELTAMARKER_DM7                                        7
#define RSFPL_REPCAP_DELTAMARKER_DM8                                        8
#define RSFPL_REPCAP_DELTAMARKER_DM9                                        9
#define RSFPL_REPCAP_DELTAMARKER_DM10                                       10
#define RSFPL_REPCAP_DELTAMARKER_DM11                                       11
#define RSFPL_REPCAP_DELTAMARKER_DM12                                       12
#define RSFPL_REPCAP_DELTAMARKER_DM13                                       13
#define RSFPL_REPCAP_DELTAMARKER_DM14                                       14
#define RSFPL_REPCAP_DELTAMARKER_DM15                                       15
#define RSFPL_REPCAP_DELTAMARKER_DM16                                       16
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSFPL_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSFPL_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSFPL_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSFPL_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

 /****************************************************************************
  *------------------------ Attribute Value Defines -------------------------*
  ****************************************************************************/

  /*- Instrument specific attribute value definitions --------------------*/
#define RSFPL_VAL_TDOM_POWER_RMS   0
#define RSFPL_VAL_TDOM_POWER_MEAN  1
#define RSFPL_VAL_TDOM_POWER_PPEAK 2
#define RSFPL_VAL_TDOM_POWER_SDEV  3

#define RSFPL_VAL_LIMIT_LOWER 0
#define RSFPL_VAL_LIMIT_UPPER 1
#define RSFPL_VAL_LIMIT_CONTROL 2

#define RSFPL_VAL_LIMIT_MOVE_OFFSET 0
#define RSFPL_VAL_LIMIT_MOVE_SHIFT  1

/* Fast Mode */
#define RSFPL_VAL_FAST_SWEEP_NORMAL 0
#define RSFPL_VAL_FAST_SWEEP_FAST   1

#define RSFPL_VAL_MARKER_SEARCH_HIGHEST            1
#define RSFPL_VAL_MARKER_SEARCH_MINIMUM            2
#define RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK          3
#define RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT     4
#define RSFPL_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT    5
#define RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM       6
#define RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT  7
#define RSFPL_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT 8

#define RSFPL_VAL_MEAS_POW_ACP   0
#define RSFPL_VAL_MEAS_POW_CPOW  1
#define RSFPL_VAL_MEAS_POW_MCAC  2
#define RSFPL_VAL_MEAS_POW_OBAN  3
#define RSFPL_VAL_MEAS_POW_CN    4
#define RSFPL_VAL_MEAS_POW_CN0   5
#define RSFPL_VAL_MEAS_POW_AOB   6
#define RSFPL_VAL_MEAS_POW_PPOW  7
#define RSFPL_VAL_MEAS_POW_GACL  8
#define RSFPL_VAL_MEAS_POW_MACM  9
#define RSFPL_VAL_MEAS_POW_COB  10

#define RSFPL_VAL_STAT_CURRENT 0
#define RSFPL_VAL_STAT_AVERAGE 1
#define RSFPL_VAL_STAT_MAX     2

#define RSFPL_VAL_SGR_MARKER_SEARCH_HIGHEST            1
#define RSFPL_VAL_SGR_MARKER_SEARCH_MINIMUM            2
#define RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK          3
#define RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK_ABOVE    4
#define RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_PEAK_BELOW    5
#define RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM       6
#define RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM_ABOVE 7
#define RSFPL_VAL_SGR_MARKER_SEARCH_NEXT_MINIMUM_BELOW 8
#define RSFPL_VAL_SGR_MARKER_SEARCH_LEVEL_MINIMUM      9
#define RSFPL_VAL_SGR_MARKER_SEARCH_LEVEL_MAXIMUM      10

#define RSFPL_VAL_STATS_APD  0
#define RSFPL_VAL_STATS_CCDF 1

#define RSFPL_VAL_STATS_SETTINGS_DEFAULT 0
#define RSFPL_VAL_STATS_SETTINGS_AUTO    1

#define RSFPL_VAL_VSA_STANDARD_LOAD   0
#define RSFPL_VAL_VSA_STANDARD_SAVE   1
#define RSFPL_VAL_VSA_STANDARD_DELETE 2

#define RSFPL_VAL_VSA_FRAME_TYPE_NEXT 0
#define RSFPL_VAL_VSA_FRAME_TYPE_PREV 1

#define RSFPL_VAL_SYNC_PATT_ADD        0
#define RSFPL_VAL_SYNC_PATT_SELECT     1
#define RSFPL_VAL_SYNC_PATT_DELETE     2
#define RSFPL_VAL_SYNC_PATT_REMOVE     3
#define RSFPL_VAL_SYNC_PATT_COPY       4

#define RSFPL_VAL_VSA_EQUALIZER_LOAD   0
#define RSFPL_VAL_VSA_EQUALIZER_SAVE   1
#define RSFPL_VAL_VSA_EQUALIZER_RESET  2

#define RSFPL_VAL_VSA_STRUCTURE_PATT_CURRENT 0
#define RSFPL_VAL_VSA_STRUCTURE_PATT_ALL     1

#define RSFPL_VAL_VSA_RESULT_EVM             0
#define RSFPL_VAL_VSA_RESULT_PHASE_ERROR     1
#define RSFPL_VAL_VSA_RESULT_MAGNITUDE_ERROR 2
#define RSFPL_VAL_VSA_RESULT_CF_ERROR        3
#define RSFPL_VAL_VSA_RESULT_RHO             4
#define RSFPL_VAL_VSA_RESULT_IQ_OFFSET       5
#define RSFPL_VAL_VSA_RESULT_FREQ_ERROR      6
#define RSFPL_VAL_VSA_RESULT_FSK_DEV_ERROR   7

#define RSFPL_VAL_VSA_LIMIT_CURRENT      0
#define RSFPL_VAL_VSA_LIMIT_MEAN         1
#define RSFPL_VAL_VSA_LIMIT_PEAK         2
#define RSFPL_VAL_VSA_LIMIT_CURRENT_RMS  3
#define RSFPL_VAL_VSA_LIMIT_MEAN_RMS     4
#define RSFPL_VAL_VSA_LIMIT_PEAK_RMS     5
#define RSFPL_VAL_VSA_LIMIT_CURRENT_PEAK 6
#define RSFPL_VAL_VSA_LIMIT_MEAN_PEAK    7
#define RSFPL_VAL_VSA_LIMIT_PEAK_PEAK    8

/* Internal Generator option B9 */
#define RSFPL_VAL_INTERNAL_GENERATOR_USAGE_CW_SOURCE          0
#define RSFPL_VAL_INTERNAL_GENERATOR_USAGE_TRACKING_GENERATOR 1
#define RSFPL_VAL_INTERNAL_GENERATOR_USAGE_POWER_SWEEP        2

/* Noise Figure Uncertainty K30 */
#define RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_VSWR  0
#define RSFPL_VAL_NOISE_FIGURE_UNCERTAINTY_IO_MATCH_RL    1

/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rsfpl_init(ViRsrc     resourceName,
                             ViBoolean  IDQuery,
                             ViBoolean  resetDevice,
                             ViSession* instrSession);

ViStatus _VI_FUNC rsfpl_InitWithOptions(ViRsrc     resourceName,
                                        ViBoolean  IDQuery,
                                        ViBoolean  resetDevice,
                                        ViString   optionString,
                                        ViSession* instrSession);

ViStatus _VI_FUNC rsfpl_SetSANMode(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyStartStop(ViSession instrSession,
                                                    ViReal64  startFrequency,
                                                    ViReal64  stopFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyCenterSpan(ViSession instrSession,
                                                     ViReal64  centerFrequency,
                                                     ViReal64  span);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyCenter(ViSession instrSession,
                                                 ViReal64  centerFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyOffset(ViSession instrSession,
                                                 ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsfpl_ConfigureOperatingModeMultiviewEnabled(
                            ViSession instrSession,
                            ViBoolean modeMultiviewEnabled);

ViStatus _VI_FUNC rsfpl_AddMeasurementChannel(ViSession instrSession,
                                              ViString  channelType,
                                              ViString  channelName);

ViStatus _VI_FUNC rsfpl_SelectMeasurementChannel(ViSession instrSession,
                                                 ViString  channelName);

ViStatus _VI_FUNC rsfpl_SelectMeasurementChannelByType(ViSession instrSession,
                                                       ViInt32   channelType);

ViStatus _VI_FUNC rsfpl_DuplicateMeasurementChannel(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ReplaceMeasurementChannel(ViSession instrSession,
                                                  ViString  oldChannelName,
                                                  ViString  newChannelType,
                                                  ViString  newChannelName);

ViStatus _VI_FUNC rsfpl_RenameMeasurementChannel(ViSession instrSession,
                                                 ViString  oldChannelName,
                                                 ViString  newChannelName);

ViStatus _VI_FUNC rsfpl_DeleteMeasurementChannel(ViSession instrSession,
                                                 ViString  channelName);

ViStatus _VI_FUNC rsfpl_QueryAllMeasurementChannels(ViSession instrSession,
                                                    ViUInt32  arraySize,
                                                    ViChar    channelCatalog[],
                                                    ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureSequencerEnabled(ViSession instrSession,
                                                  ViBoolean sequencerEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSequencerMode(ViSession instrSession,
                                               ViInt32   sequencerMode);

ViStatus _VI_FUNC rsfpl_SequencerInit(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SequencerAbort(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureRFInputSAWFilter(ViSession instrSession,
                                                  ViInt32   RFInputSAWFilter);

ViStatus _VI_FUNC rsfpl_ConfigureRFInputImpedance(ViSession instrSession,
                                                  ViInt32   RFInputImpedance);

ViStatus _VI_FUNC rsfpl_RFInputProtectionReset(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorEnabled(ViSession instrSession,
                                                    ViInt32   sensor,
                                                    ViBoolean powerSensorEnabled);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorContinuousUpdateEnabled(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean continuousUpdateEnabled);

ViStatus _VI_FUNC rsfpl_QueryPowerSensorCount(ViSession instrSession,
                                              ViInt32*  powerSensorCount);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorAssignment(ViSession instrSession,
                                                       ViInt32   sensor,
                                                       ViBoolean autoAssignEnabled,
                                                       ViString  placeholder,
                                                       ViString  type,
                                                       ViString  powerSensorInterface,
                                                       ViString  serialNumber);

ViStatus _VI_FUNC rsfpl_PowerSensorMeasurementToReference(ViSession instrSession,
                                                          ViInt32   sensor);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorReferenceValue(ViSession instrSession,
                                                           ViInt32   sensor,
                                                           ViReal64  referenceValue);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorRelativePowerEnabled(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean relativePowerEnabled);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorExternalTrigger(ViSession instrSession,
                                                            ViInt32   sensor,
                                                            ViBoolean externalTriggerEnabled,
                                                            ViReal64  level,
                                                            ViReal64  hysteresis,
                                                            ViReal64  dropoutTime,
                                                            ViReal64  holdoffTime,
                                                            ViInt32   slope);

ViStatus _VI_FUNC rsfpl_PowerSensorZero(ViSession instrSession,
                                        ViInt32   sensor);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorFrequency(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViInt32   frequencyCoupling,
                                                      ViReal64  frequencyManual);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorUseRefLevelOffsetEnabled(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean useRefLevelOffsetEnabled);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorSensorLevelOffset(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViReal64  sensorLevelOffset);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorAbsolutePowerUnit(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViInt32   absolutePowerUnit);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorRelativePowerUnit(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViInt32   relativePowerUnit);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorAveraging(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViBoolean averagingEnabled,
                                                      ViInt32   numberOfReadings);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorMeasTime(ViSession instrSession,
                                                     ViInt32   sensor,
                                                     ViInt32   measTime);

ViStatus _VI_FUNC rsfpl_ConfigurePowerSensorDutyCycle(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViBoolean dutyCycleEnabled,
                                                      ViReal64  dutyCycleValue);

ViStatus _VI_FUNC rsfpl_PowerSensorRead(ViSession instrSession,
                                        ViInt32   sensor,
                                        ViReal64* measuredValue);

ViStatus _VI_FUNC rsfpl_PowerSensorFetch(ViSession instrSession,
                                         ViInt32   sensor,
                                         ViReal64* measuredValue);

ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorState(ViSession instrSession,
                                                        ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorUsage(ViSession instrSession,
                                                        ViInt32   usage);

ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorIndependentCWSource(
                            ViSession instrSession,
                            ViReal64  level,
                            ViReal64  offset,
                            ViReal64  CWFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorTrackingGenerator(
                            ViSession instrSession,
                            ViReal64  level,
                            ViReal64  levelOffset);

ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorPowerSweep(ViSession instrSession,
                                                             ViReal64  offset,
                                                             ViReal64  PSFrequency,
                                                             ViReal64  frequencyOffset,
                                                             ViReal64  levelStart,
                                                             ViReal64  levelStop);

ViStatus _VI_FUNC rsfpl_ConfigureInternalGeneratorFrequencyOffset(
                            ViSession instrSession,
                            ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsfpl_InternalGeneratorCalibrateTransmission(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_InternalGeneratorCalibrateReflection(ViSession instrSession,
                                                             ViInt32   circuitType);

ViStatus _VI_FUNC rsfpl_InternalGeneratorRecallCalibrationSettings(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_InternalGeneratorSaveCalibrationSettings(
                            ViSession instrSession,
                            ViString  saveFilePath);

ViStatus _VI_FUNC rsfpl_InternalGeneratorQueryCalibrationSaveFilePath(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    savedFilePath[]);

ViStatus _VI_FUNC rsfpl_InternalGeneratorSaveAsTransducerFactor(
                            ViSession instrSession,
                            ViString  name);

ViStatus _VI_FUNC rsfpl_ConfigureOutputNoiseSource(ViSession instrSession,
                                                   ViBoolean noiseSourceEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureOutputSource(ViSession instrSession,
                                              ViInt32   outputSource,
                                              ViReal64  frequency);

ViStatus _VI_FUNC rsfpl_ConfigureOutputUserPortOutput(ViSession instrSession,
                                                      ViBoolean outputEnabled,
                                                      ViInt32   outputValue);

ViStatus _VI_FUNC rsfpl_ConfigureOutputSignalPolarity(ViSession instrSession,
                                                      ViInt32   outputSignalPolarity);

ViStatus _VI_FUNC rsfpl_ConfigureOutputLoudspeaker(ViSession instrSession,
                                                   ViBoolean enabled,
                                                   ViReal64  volume);

ViStatus _VI_FUNC rsfpl_OutputLoudspeakerMute(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryUserPortValue(ViSession instrSession,
                                           ViInt32*  userPortValue);

ViStatus _VI_FUNC rsfpl_AdjustAll(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_AdjustFrequency(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_AdjustLevel(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureAdjustLengthOfMeasurement(ViSession instrSession,
                                                           ViInt32   mode,
                                                           ViReal64  manualLength);

ViStatus _VI_FUNC rsfpl_ConfigureAdjustLevelHysteresis(ViSession instrSession,
                                                       ViReal64  lower,
                                                       ViReal64  upper);

ViStatus _VI_FUNC rsfpl_ConfigureAdjustWaitForTriggerEnabled(ViSession instrSession,
                                                             ViBoolean waitForTriggerEnabled);

ViStatus _VI_FUNC rsfpl_MarkerToCenterFrequency(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencySpanFull(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyZeroSpan(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyStepSize(ViSession instrSession,
                                                   ViReal64  frequencyStepSize);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyStepSizeCoupling(ViSession instrSession,
                                                           ViInt32   stepSizeCouplingMode,
                                                           ViReal64  frequencyStepSizeFactor);

ViStatus _VI_FUNC rsfpl_MarkerToStepSizeFrequency(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker);

ViStatus _VI_FUNC rsfpl_FrequencyCenterUp(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_FrequencyCenterDown(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureHorizontalScale(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   horizontalScale);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowHorizontalScale(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   subwindow,
                                                          ViInt32   horizontalScale);

ViStatus _VI_FUNC rsfpl_ConfigureSignalTracking(ViSession instrSession,
                                                ViBoolean signalTrackingEnabled,
                                                ViReal64  bandwidth,
                                                ViReal64  threshold,
                                                ViInt32   trace);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyAnnotationMode(ViSession instrSession,
                                                         ViInt32   mode);

ViStatus _VI_FUNC rsfpl_ConfigurePreamplifier(ViSession instrSession,
                                              ViBoolean preamplifierEnabled,
                                              ViReal64  gain);

ViStatus _VI_FUNC rsfpl_ConfigureAttenuation(ViSession instrSession,
                                             ViBoolean attenuationCouplingEnabled,
                                             ViReal64  attenuation);

ViStatus _VI_FUNC rsfpl_MarkerToReferenceLevel(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   marker);

ViStatus _VI_FUNC rsfpl_ConfigureAmplitudeUnit(ViSession instrSession,
                                               ViInt32   amplitudeUnit);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceLevel(ViSession instrSession,
                                                ViReal64  referenceLevel);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceLevelOffset(ViSession instrSession,
                                                      ViReal64  referenceLevelOffset);

ViStatus _VI_FUNC rsfpl_YAxisAutoscaleOnce(ViSession instrSession,
                                           ViInt32   window);

ViStatus _VI_FUNC rsfpl_ConfigureYAxisRange(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   subwindow,
                                            ViReal64  range);

ViStatus _VI_FUNC rsfpl_ConfigureYAxisGridSpacing(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   subwindow,
                                                  ViReal64  gridSpacing);

ViStatus _VI_FUNC rsfpl_ConfigureYAxisReferenceValue(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViReal64  referenceValue);

ViStatus _VI_FUNC rsfpl_ConfigureYAxisScaling(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   scaling,
                                              ViInt32   scalingMode);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowYAxisReferenceValue(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViReal64  referenceValue);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowYAxisScaling(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow,
                                                       ViInt32   scaling,
                                                       ViInt32   scalingMode);

ViStatus _VI_FUNC rsfpl_ConfigureYAxisReferenceLevelPosition(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   subwindow,
                                                             ViReal64  referenceLevelPosition);

ViStatus _VI_FUNC rsfpl_ConfigureRBW(ViSession instrSession,
                                     ViReal64  RBW);

ViStatus _VI_FUNC rsfpl_ConfigureRBWCoupling(ViSession instrSession,
                                             ViBoolean couplingEnabled,
                                             ViReal64  RBWToSpanRatio);

ViStatus _VI_FUNC rsfpl_ConfigureRBWFilterType(ViSession instrSession,
                                               ViInt32   RBWFilterType);

ViStatus _VI_FUNC rsfpl_ConfigureVBW(ViSession instrSession,
                                     ViReal64  VBW);

ViStatus _VI_FUNC rsfpl_ConfigureVBWCoupling(ViSession instrSession,
                                             ViBoolean couplingEnabled,
                                             ViInt32   RBWToVBWRatio);

ViStatus _VI_FUNC rsfpl_ConfigureVBWFilterMode(ViSession instrSession,
                                               ViInt32   VBWFilterMode);

ViStatus _VI_FUNC rsfpl_ConfigureSweepModeContinuous(ViSession instrSession,
                                                     ViBoolean sweepModeContinuousEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSweepCount(ViSession instrSession,
                                            ViInt32   sweepCount);

ViStatus _VI_FUNC rsfpl_ConfigureSweepPoints(ViSession instrSession,
                                             ViInt32   sweepPoints);

ViStatus _VI_FUNC rsfpl_ConfigureWindowSweepPoints(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   sweepPoints);

ViStatus _VI_FUNC rsfpl_ConfigurePartialSpanOptimizationMode(ViSession instrSession,
                                                             ViInt32   partialSpanOptimizationMode);

ViStatus _VI_FUNC rsfpl_ConfigureSweepTime(ViSession instrSession,
                                           ViBoolean sweepTimeAutoEnabled,
                                           ViReal64  sweepTimeManual);

ViStatus _VI_FUNC rsfpl_QueryCurrentNumberOfSweeps(ViSession instrSession,
                                                   ViInt32*  currentNumberOfSweeps);

ViStatus _VI_FUNC rsfpl_QueryCaptureTime(ViSession instrSession,
                                         ViReal64* captureTime);

ViStatus _VI_FUNC rsfpl_QuerySweepType(ViSession instrSession,
                                       ViInt32*  sweepType);

ViStatus _VI_FUNC rsfpl_ConfigureSweepTypeMode(ViSession instrSession,
                                               ViInt32   mode);

ViStatus _VI_FUNC rsfpl_QuerySweepFFTSubspan(ViSession instrSession,
                                             ViInt32*  FFTSubspan);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerSource(ViSession instrSession,
                                               ViInt32   triggerSource);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerLevelIFP(ViSession instrSession,
                                                 ViReal64  triggerLevelIFP);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerLevelIQPower(ViSession instrSession,
                                                     ViReal64  triggerLevelIQPower);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerDropOutTime(ViSession instrSession,
                                                    ViReal64  triggerDropOutTime);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerOffset(ViSession instrSession,
                                               ViReal64  triggerOffset);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerSlope(ViSession instrSession,
                                              ViInt32   triggerSlope);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerHysteresis(ViSession instrSession,
                                                   ViReal64  triggerHysteresis);

ViStatus _VI_FUNC rsfpl_ConfigureTriggerHoldoff(ViSession instrSession,
                                                ViReal64  triggerHoldoff);

ViStatus _VI_FUNC rsfpl_ConfigureExternalTriggerLevel(ViSession instrSession,
                                                      ViInt32   triggerPort,
                                                      ViReal64  externalTriggerLevel);

ViStatus _VI_FUNC rsfpl_ConfigureGate(ViSession instrSession,
                                      ViBoolean gateEnabled,
                                      ViInt32   source,
                                      ViInt32   mode,
                                      ViReal64  delay,
                                      ViReal64  length,
                                      ViInt32   gatePolarity);

ViStatus _VI_FUNC rsfpl_ConfigureMarker(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   marker,
                                        ViBoolean markerEnabled,
                                        ViInt32   trace);

ViStatus _VI_FUNC rsfpl_QueryMarker(ViSession instrSession,
                                    ViInt32   window,
                                    ViInt32   marker,
                                    ViReal64* markerPosition,
                                    ViReal64* markerAmplitude);

ViStatus _VI_FUNC rsfpl_AllMarkersOff(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerGeneralSettings(ViSession instrSession,
                                                       ViInt32   markerTableDisplayMode,
                                                       ViBoolean markerInfoEnabled,
                                                       ViInt32   markerStepsize);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerPosition(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViReal64  markerPosition);

ViStatus _VI_FUNC rsfpl_LinkToAnotherMarker(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   firstMarker,
                                            ViInt32   secondMarker,
                                            ViBoolean linkEnabled);

ViStatus _VI_FUNC rsfpl_LinkToAnotherDeltaMarker(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   firstMarker,
                                                 ViInt32   secondMarker,
                                                 ViBoolean linkEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerLinking(ViSession instrSession,
                                               ViInt32   markerLinking);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerExcludeLOEnabled(ViSession instrSession,
                                                        ViBoolean excludeLOEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerPeakExcursion(ViSession instrSession,
                                                     ViReal64  peakExcursion);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerSearchLimits(ViSession instrSession,
                                                    ViBoolean searchLimitsEnabled,
                                                    ViReal64  searchLimitLeft,
                                                    ViReal64  searchLimitRight);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerSearchZoomLimitsEnabled(
                            ViSession instrSession,
                            ViBoolean searchZoomLimitsEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerSearchThreshold(ViSession instrSession,
                                                       ViBoolean thresholdEnabled,
                                                       ViReal64  thresholdValue);

ViStatus _VI_FUNC rsfpl_DeactivateAllSearchLimits(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerAutoPeakSearch(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViBoolean autoMaxPeakSearchEnabled,
                                                      ViBoolean autoMinPeakSearchEnabled);

ViStatus _VI_FUNC rsfpl_MarkerSearch(ViSession instrSession,
                                     ViInt32   window,
                                     ViInt32   marker,
                                     ViInt32   markerSearch);

ViStatus _VI_FUNC rsfpl_QueryMarkerAmplitude(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   marker,
                                             ViReal64* amplitude);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerProbability(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViReal64  markerProbability);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerZoomFactor(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViReal64  markerZoomFactor);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerFrequencyCounter(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViBoolean signalCount,
                                                        ViReal64  resolution);

ViStatus _VI_FUNC rsfpl_QueryFrequencyCounterResult(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViReal64* counterFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerPeakListEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker,
                                                       ViBoolean peakListEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerPeakList(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   sortMode,
                                                ViInt32   maximumNumberOfPeaks,
                                                ViBoolean labelsEnabled);

ViStatus _VI_FUNC rsfpl_MarkerPeakListSearchPeaks(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViInt32   numberOfPeaks);

ViStatus _VI_FUNC rsfpl_QueryMarkerPeakListNumberOfFoundPeaks(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32*  numberOfFoundPeaks);

ViStatus _VI_FUNC rsfpl_QueryMarkerPeakListXPositions(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViUInt32  arraySize,
                                                      ViReal64  peakList[],
                                                      ViUInt32* numberOfResults);

ViStatus _VI_FUNC rsfpl_QueryMarkerPeakListYPositions(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViUInt32  arraySize,
                                                      ViReal64  peakList[],
                                                      ViUInt32* numberOfResults);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementMarker(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViBoolean noiseMeasurementEnabled);

ViStatus _VI_FUNC rsfpl_NoiseMeasurementMarkerAllOff(ViSession instrSession,
                                                     ViInt32   window);

ViStatus _VI_FUNC rsfpl_QueryNoiseMeasurementMarkerResult(ViSession instrSession,
                                                          ViInt32   marker,
                                                          ViInt32   window,
                                                          ViReal64* measurementResult);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseMeasurementMarker(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViBoolean phaseNoiseMarkerEnabled);

ViStatus _VI_FUNC rsfpl_PhaseNoiseMeasurementMarkerAllOff(ViSession instrSession,
                                                          ViInt32   window);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseMeasurementMarkerResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViReal64* phaseNoiseResult);

ViStatus _VI_FUNC rsfpl_ConfigureBandPowerMarker(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViBoolean bandPowerMarkerEnabled,
                                                 ViInt32   mode,
                                                 ViReal64  span);

ViStatus _VI_FUNC rsfpl_BandPowerMarkerAllOff(ViSession instrSession,
                                              ViInt32   window);

ViStatus _VI_FUNC rsfpl_QueryBandPowerMarkerResult(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViReal64* bandPowerMarkerResult);

ViStatus _VI_FUNC rsfpl_ConfigurendBDown(ViSession instrSession,
                                         ViBoolean nDBDownEnabled,
                                         ViReal64  nDBDownDistance);

ViStatus _VI_FUNC rsfpl_QueryndBDownResult(ViSession instrSession,
                                           ViReal64* nDBDownResult);

ViStatus _VI_FUNC rsfpl_QueryndBDownFrequencies(ViSession instrSession,
                                                ViReal64* left,
                                                ViReal64* right);

ViStatus _VI_FUNC rsfpl_QueryndBDownQFactor(ViSession instrSession,
                                            ViReal64* nDBDownQFactor);

ViStatus _VI_FUNC rsfpl_QueryndBDownTime(ViSession instrSession,
                                         ViReal64* left,
                                         ViReal64* right);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerDemodulation(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViBoolean demodulationEnabled,
                                                    ViInt32   demodulationMode);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerDemodulationContinuous(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean continuousEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureMarkerDemodulationStopTime(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   marker,
                                                            ViReal64  stopTime);

ViStatus _VI_FUNC rsfpl_ConfigureSelectiveDemodulation(ViSession instrSession,
                                                       ViBoolean enabled,
                                                       ViReal64  selectiveThreshold);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramMarkerFramePosition(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   spectrogramMarkerFrame);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramMarkerSearchArea(ViSession instrSession,
                                                             ViInt32   searchArea);

ViStatus _VI_FUNC rsfpl_SpectrogramMarkerSearch(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   markerSearch);

ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarker(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   marker,
                                             ViBoolean state,
                                             ViInt32   trace);

ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarkerMode(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   deltaMarkerMode);

ViStatus _VI_FUNC rsfpl_AllDeltaMarkersOff(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_DeltaMarkerSearch(ViSession instrSession,
                                          ViInt32   window,
                                          ViInt32   marker,
                                          ViInt32   markerSearch);

ViStatus _VI_FUNC rsfpl_QueryDeltaMarker(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   marker,
                                         ViReal64* deltaMarkerRelativePosition,
                                         ViReal64* deltaMarkerAmplitude);

ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarkerPosition(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViReal64  deltaMarkerPosition);

ViStatus _VI_FUNC rsfpl_DeltaMarkerLinkToAnotherMarker(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   firstMarker,
                                                       ViInt32   secondMarker,
                                                       ViBoolean linkEnabled);

ViStatus _VI_FUNC rsfpl_DeltaMarkerLinkToAnotherDeltaMarker(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   firstMarker,
                                                            ViInt32   secondMarker,
                                                            ViBoolean linkEnabled);

ViStatus _VI_FUNC rsfpl_DeltaMarkerLinkToMarker1(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViBoolean deltaMarkerLinkToMarker1);

ViStatus _VI_FUNC rsfpl_ConfigureDeltaMarkerReference(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViBoolean fixedReferenceEnabled,
                                                      ViInt32   deltaMarkerReference);

ViStatus _VI_FUNC rsfpl_ConfigureFixedReferenceMarkerEnabled(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViBoolean fixedReferenceEnabled);

ViStatus _VI_FUNC rsfpl_FixedReferenceToMaxPeak(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   window);

ViStatus _VI_FUNC rsfpl_ConfigureFixedReferencePoint(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViReal64  referencePointHorizontal,
                                                     ViReal64  referencePointVertical,
                                                     ViReal64  referencePointLevelOffset);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseMeasurementDeltaMarkerEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean phaseNoiseEnabled);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseMeasurementDeltaMarkerAutoPeakSearchEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean autoPeakSearchEnabled);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseMeasurementDeltaMarkerResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViReal64* deltaMarkerResult);

ViStatus _VI_FUNC rsfpl_ConfigureBandPowerDeltaMarker(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViBoolean bandPowerEnabled,
                                                      ViInt32   bandPowerMode,
                                                      ViReal64  span);

ViStatus _VI_FUNC rsfpl_QueryBandPowerDeltaMarkerResult(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViReal64* bandPowerResult);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramDeltaMarkerFrame(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViInt32   spectrogramDeltaMarkerFrame);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramDeltaMarkerSearchArea(
                            ViSession instrSession,
                            ViInt32   deltaMarkerSearchArea);

ViStatus _VI_FUNC rsfpl_SpectrogramDeltaMarkerSearch(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViInt32   markerSearch);

ViStatus _VI_FUNC rsfpl_ConfigureTrace(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   subwindow,
                                       ViInt32   trace,
                                       ViInt32   traceMode);

ViStatus _VI_FUNC rsfpl_ConfigureTraceDetector(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViBoolean traceDetectorAutoEnabled,
                                               ViInt32   traceDetector);

ViStatus _VI_FUNC rsfpl_ConfigureTraceAverageEnabled(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViBoolean traceAverageEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureTraceAverageCount(ViSession instrSession,
                                                   ViInt32   traceAverageCount);

ViStatus _VI_FUNC rsfpl_ConfigureTraceAverageMode(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   traceAverageMode);

ViStatus _VI_FUNC rsfpl_ConfigureTraceSmoothing(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   trace,
                                                ViBoolean traceSmoothingEnabled,
                                                ViInt32   apertureValue);

ViStatus _VI_FUNC rsfpl_ConfigureTraceHoldEnabled(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   trace,
                                                  ViBoolean traceHoldEnabled);

ViStatus _VI_FUNC rsfpl_CopyTrace(ViSession instrSession,
                                  ViInt32   window,
                                  ViInt32   destinationTrace,
                                  ViInt32   sourceTrace);

ViStatus _VI_FUNC rsfpl_ConfigureSubtractTraces(ViSession instrSession,
                                                ViInt32   window,
                                                ViReal64  position,
                                                ViBoolean traceMathEnabled,
                                                ViInt32   mode);

ViStatus _VI_FUNC rsfpl_SubtractTraces(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   trace2);

ViStatus _VI_FUNC rsfpl_ApplyWindowPresetTraces(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   trace,
                                                ViInt32   presetTraces);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowTraceEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow,
                                                       ViInt32   trace,
                                                       ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowTraceHold(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   subwindow,
                                                    ViInt32   trace,
                                                    ViBoolean holdEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowTraceSmoothing(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   subwindow,
                                                         ViInt32   trace,
                                                         ViBoolean smoothingEnabled,
                                                         ViInt32   smoothingValue);

ViStatus _VI_FUNC rsfpl_ApplySubwindowPresetTraces(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   subwindow,
                                                   ViInt32   trace,
                                                   ViInt32   presetTraces);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramEnabled(ViSession instrSession,
                                                    ViBoolean spectrogramEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogram3DEnabled(ViSession instrSession,
                                                      ViBoolean _3DEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramTimeStampEnabled(ViSession instrSession,
                                                             ViBoolean spectrogramTimeStampEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramSelectedFrame(ViSession instrSession,
                                                          ViInt32   spectrogramSelectedFrame);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramHistoryDepth(ViSession instrSession,
                                                         ViInt32   spectrogramHistoryDepth);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramSweep(ViSession instrSession,
                                                  ViBoolean continousSweepEnabled,
                                                  ViInt32   numberOfFrames);

ViStatus _VI_FUNC rsfpl_SpectrogramNumberOfFrames(ViSession instrSession,
                                                  ViInt32   spectrogramNumberOfFrames);

ViStatus _VI_FUNC rsfpl_QuerySpectrogramTimeStampData(ViSession instrSession,
                                                      ViInt32*  seconds,
                                                      ViInt32*  nanoseconds,
                                                      ViInt32*  reserved1,
                                                      ViInt32*  reserved2);

ViStatus _VI_FUNC rsfpl_QuerySpectrogramTimeStampDataAll(ViSession instrSession,
                                                         ViUInt32  arraySize,
                                                         ViInt32   seconds[],
                                                         ViInt32   nanoseconds[],
                                                         ViInt32   reserved1[],
                                                         ViInt32   reserved2[]);

ViStatus _VI_FUNC rsfpl_SpectrogramClear(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramLayout(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   spectrogramLayout);

ViStatus _VI_FUNC rsfpl_ConfigureSpectrogramSelectTrace(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   trace);

ViStatus _VI_FUNC rsfpl_ColorMapRestoreDefault(ViSession instrSession,
                                               ViInt32   window);

ViStatus _VI_FUNC rsfpl_ConfigureColorMapScheme(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   colorMapScheme);

ViStatus _VI_FUNC rsfpl_ConfigureColorMap(ViSession instrSession,
                                          ViInt32   window,
                                          ViReal64  startingPoint,
                                          ViReal64  endPoint,
                                          ViReal64  curveShape);

ViStatus _VI_FUNC rsfpl_WindowSetFocus(ViSession instrSession,
                                       ViInt32   window);

ViStatus _VI_FUNC rsfpl_SubwindowSetFocus(ViSession instrSession,
                                          ViInt32   window,
                                          ViInt32   subwindow);

ViStatus _VI_FUNC rsfpl_ConfigureDisplay(ViSession instrSession,
                                         ViBoolean statusBarEnabled,
                                         ViBoolean softkeyBarEnabled,
                                         ViBoolean toolbarEnabled,
                                         ViInt32   touchEnabled,
                                         ViBoolean channelInformationEnabled,
                                         ViBoolean xAxisLabelEnabled,
                                         ViBoolean dateAndTimeEnabled,
                                         ViBoolean frontPanelKeysEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureDisplayBrightness(ViSession instrSession,
                                                   ViInt32   displayBrightness);

ViStatus _VI_FUNC rsfpl_ConfigureDisplayDateAndTimeFormat(ViSession instrSession,
                                                          ViInt32   dateAndTimeFormat);

ViStatus _VI_FUNC rsfpl_ConfigureDisplayTemporarySize(ViSession instrSession,
                                                      ViInt32   displayTemporarySize);

ViStatus _VI_FUNC rsfpl_ConfigureDisplayFormat(ViSession instrSession,
                                               ViInt32   displayFormat);

ViStatus _VI_FUNC rsfpl_QueryColorsAndThemesCatalog(ViSession instrSession,
                                                    ViUInt32  arraySize,
                                                    ViChar    catalog[],
                                                    ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureColorsAndThemesSelectedTheme(
                            ViSession instrSession,
                            ViString  selectedTheme);

ViStatus _VI_FUNC rsfpl_ResetColorScheme(ViSession instrSession,
                                         ViInt32   color);

ViStatus _VI_FUNC rsfpl_LayoutAddWindow(ViSession instrSession,
                                        ViString  windowName,
                                        ViInt32   direction,
                                        ViInt32   windowType,
                                        ViUInt32  arraySize,
                                        ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsfpl_LayoutReplaceWindow(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32   windowType);

ViStatus _VI_FUNC rsfpl_LayoutMoveWindow(ViSession instrSession,
                                         ViString  sourceWindowName,
                                         ViString  destinationWindowName,
                                         ViInt32   direction);

ViStatus _VI_FUNC rsfpl_LayoutRemoveWindow(ViSession instrSession,
                                           ViString  window);

ViStatus _VI_FUNC rsfpl_QueryLayoutWindowList(ViSession instrSession,
                                              ViUInt32  arraySize,
                                              ViChar    catalog[],
                                              ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_LayoutIdentifyWindow(ViSession instrSession,
                                             ViString  windowName,
                                             ViInt32*  index);

ViStatus _VI_FUNC rsfpl_ConfigureLayoutSplitterPosition(ViSession instrSession,
                                                        ViString  window1,
                                                        ViString  window2,
                                                        ViInt32   position);

ViStatus _VI_FUNC rsfpl_ConfigureLayoutWindowType(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViString  type);

ViStatus _VI_FUNC rsfpl_ConfigureDisplaySingleZoomEnabled(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViBoolean displaySingleZoomEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSingleZoom(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   lowerLeftX,
                                            ViInt32   lowerLeftY,
                                            ViInt32   upperRightX,
                                            ViInt32   upperRightY);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowDisplaySingleZoomEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean displaySingleZoomEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowSingleZoom(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   subwindow,
                                                     ViInt32   lowerLeftX,
                                                     ViInt32   lowerLeftY,
                                                     ViInt32   upperRightX,
                                                     ViInt32   upperRightY);

ViStatus _VI_FUNC rsfpl_ConfigureMultipleZoomEnabled(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   zoomWindow,
                                                     ViBoolean multipleZoomEnabled);

ViStatus _VI_FUNC rsfpl_MultipleZoom(ViSession instrSession,
                                     ViInt32   window,
                                     ViInt32   zoomWindow,
                                     ViInt32   lowerLeftX,
                                     ViInt32   lowerLeftY,
                                     ViInt32   upperRightX,
                                     ViInt32   upperRightY);

ViStatus _VI_FUNC rsfpl_ConfigureSubwindowMultipleZoomEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   zoomWindow,
                            ViBoolean multipleZoomEnabled);

ViStatus _VI_FUNC rsfpl_SubwindowMultipleZoom(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   subwindow,
                                              ViInt32   zoomWindow,
                                              ViInt32   lowerLeftX,
                                              ViInt32   lowerLeftY,
                                              ViInt32   upperRightX,
                                              ViInt32   upperRightY);

ViStatus _VI_FUNC rsfpl_ConfigureFrequencyDisplayLine(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   line,
                                                      ViBoolean displayLineEnabled,
                                                      ViReal64  displayLinePosition);

ViStatus _VI_FUNC rsfpl_ConfigureTimeDisplayLine(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   line,
                                                 ViBoolean displayLineEnabled,
                                                 ViReal64  displayLinePosition);

ViStatus _VI_FUNC rsfpl_ConfigurePowerDisplayLine(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   line,
                                                  ViBoolean displayLineEnabled,
                                                  ViReal64  displayLinePosition);

ViStatus _VI_FUNC rsfpl_ConfigureHorizontalDisplayLine(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   line,
                                                       ViBoolean displayLineEnabled,
                                                       ViReal64  displayLinePosition);

ViStatus _VI_FUNC rsfpl_CreateLimitLine(ViSession instrSession,
                                        ViInt32   limit,
                                        ViString  name,
                                        ViInt32   domain,
                                        ViString  comment,
                                        ViBoolean deleteExistingLimitLine);

ViStatus _VI_FUNC rsfpl_ConfigureLimitLinesAssignTraceEnabled(
                            ViSession instrSession,
                            ViInt32   limit,
                            ViInt32   trace,
                            ViBoolean assignTraceEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureLimitLine(ViSession instrSession,
                                           ViInt32   limit,
                                           ViInt32   type,
                                           ViInt32   units,
                                           ViInt32   xAxisSpacing,
                                           ViInt32   yAxisInterpolation,
                                           ViInt32   xAxisScaling,
                                           ViInt32   yAxisScaling,
                                           ViReal64  margin,
                                           ViReal64  threshold);

ViStatus _VI_FUNC rsfpl_DefineLimitLine(ViSession instrSession,
                                        ViInt32   limit,
                                        ViInt32   type,
                                        ViUInt32  count,
                                        ViReal64  xAxis[],
                                        ViReal64  amplitude[]);

ViStatus _VI_FUNC rsfpl_ConfigureLimitLineCheckEnabled(ViSession instrSession,
                                                       ViInt32   limit,
                                                       ViBoolean limitLineCheckEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureLimitLineEnabled(ViSession instrSession,
                                                  ViInt32   limit,
                                                  ViInt32   type,
                                                  ViBoolean limitEnabled);

ViStatus _VI_FUNC rsfpl_MoveLimitLine(ViSession instrSession,
                                      ViInt32   limit,
                                      ViInt32   type,
                                      ViInt32   method,
                                      ViReal64  offsetValue);

ViStatus _VI_FUNC rsfpl_CopyLimitLine(ViSession instrSession,
                                      ViInt32   limit,
                                      ViInt32   copyTo);

ViStatus _VI_FUNC rsfpl_DeleteLimitLine(ViSession instrSession,
                                        ViInt32   limit);

ViStatus _VI_FUNC rsfpl_DeleteLimitResults(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryLimitNamesOfActiveLimitLines(ViSession instrSession,
                                                          ViUInt32  arraySize,
                                                          ViChar    catalog[],
                                                          ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_LoadLimitLine(ViSession instrSession,
                                      ViString  fileName);

ViStatus _VI_FUNC rsfpl_ExportLimitLine(ViSession instrSession,
                                        ViString  fileName,
                                        ViString  limitLineName);

ViStatus _VI_FUNC rsfpl_ConfigureDataSelectedStorageDevice(ViSession instrSession,
                                                           ViString  selectedStorageDevice);

ViStatus _VI_FUNC rsfpl_ConfigureDataSelectedDirectory(ViSession instrSession,
                                                       ViString  selectedDirectory);

ViStatus _VI_FUNC rsfpl_DataCreateDirectory(ViSession instrSession,
                                            ViString  directory);

ViStatus _VI_FUNC rsfpl_QueryDataDirectoryContents(ViSession instrSession,
                                                   ViString  directory,
                                                   ViUInt32  arraySize,
                                                   ViChar    contents[],
                                                   ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_QueryDataDirectoryAdditionalContents(ViSession instrSession,
                                                             ViString  directory,
                                                             ViUInt32  arraySize,
                                                             ViChar    contents[],
                                                             ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_DataDeleteDirectory(ViSession instrSession,
                                            ViString  directory);

ViStatus _VI_FUNC rsfpl_ConfigureDataFileName(ViSession instrSession,
                                              ViString  dataFileName);

ViStatus _VI_FUNC rsfpl_ConfigureDataFileNameComment(ViSession instrSession,
                                                     ViString  comment);

ViStatus _VI_FUNC rsfpl_ConfigureDataDecimalSeparator(ViSession instrSession,
                                                      ViInt32   decimalSeparator);

ViStatus _VI_FUNC rsfpl_ConfigureDataXValueDistribution(ViSession instrSession,
                                                        ViInt32   distribution);

ViStatus _VI_FUNC rsfpl_DataCopyFile(ViSession instrSession,
                                     ViString  sourceFile,
                                     ViString  destinationFile);

ViStatus _VI_FUNC rsfpl_DataMoveFile(ViSession instrSession,
                                     ViString  sourceFile,
                                     ViString  newFileName);

ViStatus _VI_FUNC rsfpl_DataDeleteFile(ViSession instrSession,
                                       ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataNetworkMapDrive(ViSession instrSession,
                                            ViString  drive,
                                            ViString  hostName,
                                            ViString  userName,
                                            ViString  password,
                                            ViBoolean reconnect);

ViStatus _VI_FUNC rsfpl_DataNetworkDisconnectDrive(ViSession instrSession,
                                                   ViString  drive,
                                                   ViBoolean force);

ViStatus _VI_FUNC rsfpl_QueryDataNetworkUnusedDrives(ViSession instrSession,
                                                     ViUInt32  arraySize,
                                                     ViChar    unusedDrives[],
                                                     ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_QueryDataNetworkUsedDrives(ViSession instrSession,
                                                   ViUInt32  arraySize,
                                                   ViChar    usedDrives[],
                                                   ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureDataInstrumentSettingsMode(ViSession instrSession,
                                                            ViInt32   settingsMode);

ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsDeleteAll(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsDelete(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsStartupRecall(ViSession instrSession,
                                                            ViString  fileName);

ViStatus _VI_FUNC rsfpl_ConfigureDataInstrumentSettingsRecallMode(
                            ViSession instrSession,
                            ViInt32   settingsRecallMode);

ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsRecall(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsSave(ViSession instrSession,
                                                   ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataInstrumentSettingsSaveNext(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ItemsToStoreInstrumentDefault(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ItemsToStoreInstrumentAll(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ItemsToStoreInstrumentNone(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreInstrument(ViSession instrSession,
                                                        ViBoolean HWSettingsEnabled,
                                                        ViBoolean limitLinesEnabled,
                                                        ViBoolean spectrogramDataEnabled,
                                                        ViBoolean traceDataEnabled,
                                                        ViBoolean transducerFactorsEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreInstrumentSourceCalibrationData(
                            ViSession instrSession,
                            ViBoolean sourceCalibrationData);

ViStatus _VI_FUNC rsfpl_ItemsToStoreChannelDefault(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ItemsToStoreChannelAll(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ItemsToStoreChannelNone(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreChannel(ViSession instrSession,
                                                     ViBoolean HWSettingsEnabled,
                                                     ViBoolean limitLinesEnabled,
                                                     ViBoolean spectrogramDataEnabled,
                                                     ViBoolean traceDataEnabled,
                                                     ViBoolean transducerFactorsEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureItemsToStoreChannelSourceCalibrationData(
                            ViSession instrSession,
                            ViBoolean sourceCalibrationData);

ViStatus _VI_FUNC rsfpl_HcopyAbort(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_HcopyMode(ViSession instrSession,
                                  ViInt32   mode);

ViStatus _VI_FUNC rsfpl_HcopyPrint(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_HcopyPrintNext(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyContent(ViSession instrSession,
                                              ViInt32   contentType,
                                              ViString  comment,
                                              ViBoolean logoEnabled,
                                              ViBoolean pageCountEnabled,
                                              ViBoolean dateAndTimeEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyWindowPrintout(ViSession instrSession,
                                                     ViString  channelName,
                                                     ViString  windowName,
                                                     ViBoolean windowEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyChannelPrintout(ViSession instrSession,
                                                      ViString  channelName,
                                                      ViBoolean channelEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyDestination(ViSession instrSession,
                                                  ViInt32   destination);

ViStatus _VI_FUNC rsfpl_ConfigureHcopySelectedPrinter(ViSession instrSession,
                                                      ViString  selectedPrinter);

ViStatus _VI_FUNC rsfpl_HcopyFileFormat(ViSession instrSession,
                                        ViInt32   hcopyFileFormat);

ViStatus _VI_FUNC rsfpl_QueryHcopyFirstPrinter(ViSession instrSession,
                                               ViUInt32  arraySize,
                                               ViChar    firstPrinter[],
                                               ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_QueryHcopyNextPrinter(ViSession instrSession,
                                              ViUInt32  arraySize,
                                              ViChar    nextPrinter[],
                                              ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyPageOrientation(ViSession instrSession,
                                                      ViInt32   pageOrientation);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyWindowsPerPage(ViSession instrSession,
                                                     ViInt32   windowsPerPage);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyPageScaleToFitEnabled(ViSession instrSession,
                                                            ViBoolean scaleToFitEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyPageMargin(ViSession instrSession,
                                                 ViInt32   unit,
                                                 ViReal64  bottom,
                                                 ViReal64  left,
                                                 ViReal64  right,
                                                 ViReal64  top);

ViStatus _VI_FUNC rsfpl_ConfigureHcopyColorPrinting(ViSession instrSession,
                                                    ViBoolean colorPrintingEnabled);

ViStatus _VI_FUNC rsfpl_ResetHcopyColorScheme(ViSession instrSession,
                                              ViInt32   color);

ViStatus _VI_FUNC rsfpl_ConfigureDataResultsFileFormat(ViSession instrSession,
                                                       ViInt32   fileFormat);

ViStatus _VI_FUNC rsfpl_ConfigureDataResultsHeaderEnabled(ViSession instrSession,
                                                          ViBoolean headerEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureDataResultsIncludedDataExport(
                            ViSession instrSession,
                            ViInt32   resultsIncludedDataExport);

ViStatus _VI_FUNC rsfpl_ConfigureDataResultsIncludedDataImport(
                            ViSession instrSession,
                            ViInt32   resultsIncludedDataImport);

ViStatus _VI_FUNC rsfpl_DataResultsExportMarkerPeakListSpurious(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataResultsExportTrace(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataResultsImportTrace(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViChar    fileName[]);

ViStatus _VI_FUNC rsfpl_DataResultsExportSpuriousEmissionList(
                            ViSession instrSession,
                            ViInt32   window,
                            ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataResultsExportMarkerPeakList(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViString  fileName);

ViStatus _VI_FUNC rsfpl_DataResultsExportSpectrogramData(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViString  fileName);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateGeneralSettings(
                            ViSession instrSession,
                            ViInt32   pageFormat,
                            ViInt32   fileFormat,
                            ViBoolean printerFriendlyColors,
                            ViBoolean includeDate,
                            ViBoolean includePageCount);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateRawDataStorage(
                            ViSession instrSession,
                            ViString  path);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateTitlePageLine(
                            ViSession instrSession,
                            ViInt32   headerLine,
                            ViInt32   visibility,
                            ViString  title,
                            ViString  value);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateLogo(ViSession instrSession,
                                                        ViInt32   visibility,
                                                        ViString  logoPath);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateSelectedItems(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   arraySize,
                            ViInt32   items[]);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportTemplateSelectedItemsString(
                            ViSession instrSession,
                            ViString  channelType,
                            ViString  items);

ViStatus _VI_FUNC rsfpl_QueryTestReportTemplateCatalogue(ViSession instrSession,
                                                         ViInt32   bufferSize,
                                                         ViChar    templateCatalogue[],
                                                         ViInt32*  bytesRead);

ViStatus _VI_FUNC rsfpl_SaveTestReportTemplate(ViSession instrSession,
                                               ViString  templateName);

ViStatus _VI_FUNC rsfpl_LoadTestReportTemplate(ViSession instrSession,
                                               ViString  templateName);

ViStatus _VI_FUNC rsfpl_DeleteTestReportTemplate(ViSession instrSession,
                                                 ViString  templateName);

ViStatus _VI_FUNC rsfpl_TestReportRestoreDefaults(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_TestReportNew(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_TestReportAppend(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_TestReportRemoveSet(ViSession instrSession,
                                            ViInt32   dataset);

ViStatus _VI_FUNC rsfpl_TestReportRemoveAllSets(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportTitlePage(ViSession instrSession,
                                                     ViBoolean includeTitlePage,
                                                     ViString  title,
                                                     ViString  abstract);

ViStatus _VI_FUNC rsfpl_ConfigureTestReportIncludeUserDefinedInformation(
                            ViSession instrSession,
                            ViBoolean includeInformation);

ViStatus _VI_FUNC rsfpl_QueryTestReportItemList(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    testReportItemList[],
                                                ViInt32*  bytesRead);

ViStatus _VI_FUNC rsfpl_TestReportSave(ViSession instrSession,
                                       ViInt32   fileType,
                                       ViString  reportPath);

ViStatus _VI_FUNC rsfpl_QueryTransducerFactorCatalog(ViSession instrSession,
                                                     ViUInt32  arraySize,
                                                     ViChar    catalog[],
                                                     ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureSelectedTransducerFactor(ViSession instrSession,
                                                          ViString  selectedTransducerFactor);

ViStatus _VI_FUNC rsfpl_ConfigureTransducerFactor(ViSession instrSession,
                                                  ViInt32   scaling,
                                                  ViInt32   unit,
                                                  ViString  comment);

ViStatus _VI_FUNC rsfpl_TransducerFactorWriteData(ViSession instrSession,
                                                  ViUInt32  arraySize,
                                                  ViReal64  frequency[],
                                                  ViReal64  level[]);

ViStatus _VI_FUNC rsfpl_ConfigureTransducerFactorEnabled(ViSession instrSession,
                                                         ViBoolean transducerFactorEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureTransducerFactorRefLevelAutoAdjustEnabled(
                            ViSession instrSession,
                            ViBoolean levelAutoAdjustEnabled);

ViStatus _VI_FUNC rsfpl_TransducerFactorImport(ViSession instrSession,
                                               ViString  transducerFactorImport);

ViStatus _VI_FUNC rsfpl_TransducerFactorExport(ViSession instrSession,
                                               ViString  fileName,
                                               ViString  transducerName);

ViStatus _VI_FUNC rsfpl_DeleteTransducerFactor(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceOscillatorSource(ViSession instrSession,
                                                           ViInt32   referenceOscillatorSource);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceOscillator100MHzEnabled(
                            ViSession instrSession,
                            ViBoolean output100MHzEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureRemoteLoggingEnabled(ViSession instrSession,
                                                      ViBoolean remoteLoggingEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureRemoteDisplay(ViSession instrSession,
                                               ViBoolean displayUpdateEnabled,
                                               ViBoolean displayErrorEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureRemoteDisplayLockEnabled(ViSession instrSession,
                                                          ViBoolean lockEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureLocalLockout(ViSession instrSession,
                                              ViBoolean generalLocalLockoutEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureDisplayLanguage(ViSession instrSession,
                                                 ViInt32   displayLanguage);

ViStatus _VI_FUNC rsfpl_ConfigureSystemDateTime(ViSession instrSession,
                                                ViInt32   year,
                                                ViInt32   month,
                                                ViInt32   day,
                                                ViInt32   hour,
                                                ViInt32   minute,
                                                ViInt32   second);

ViStatus _VI_FUNC rsfpl_QuerySystemDateTime(ViSession instrSession,
                                            ViInt32*  year,
                                            ViInt32*  month,
                                            ViInt32*  day,
                                            ViInt32*  hour,
                                            ViInt32*  minute,
                                            ViInt32*  second);

ViStatus _VI_FUNC rsfpl_GeneralResetRevisionToFactoryDefault(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_GeneralResetIDNQueryToFactoryDefault(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryFactoryDefaultIDN(ViSession instrSession,
                                               ViInt32*  factoryDefaultIDN);

ViStatus _VI_FUNC rsfpl_ConfigureIDNQueryString(ViSession instrSession,
                                                ViString  IDNQueryString);

ViStatus _VI_FUNC rsfpl_QueryLXIInfo(ViSession instrSession,
                                     ViUInt32  arraySize,
                                     ViChar    LXIInfo[],
                                     ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_LXILANReset(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureLXIInstrumentDescription(ViSession instrSession,
                                                          ViString  LXIInstrumentDescription);

ViStatus _VI_FUNC rsfpl_ConfigureLXIPassword(ViSession instrSession,
                                             ViString  LXIPassword);

ViStatus _VI_FUNC rsfpl_Calibrate(ViSession instrSession,
                                  ViInt32*  calibrate);

ViStatus _VI_FUNC rsfpl_CalibrateAll(ViSession  instrSession,
                                     ViUInt32   timeout,
                                     ViBoolean* calibrationSuccessful);

ViStatus _VI_FUNC rsfpl_QueryCalibrationResult(ViSession instrSession,
                                               ViUInt32  arraySize,
                                               ViChar    calibrationResult[],
                                               ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalFrequency(ViSession instrSession,
                                                            ViReal64  calibrationSignalFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalBandwidth(ViSession instrSession,
                                                            ViInt32   calibrationSignalBandwidth);

ViStatus _VI_FUNC rsfpl_ConfigureCalibrationInputSource(ViSession instrSession,
                                                        ViInt32   calibrationInputSource);

ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalFrequency43MHzPlus(
                            ViSession instrSession,
                            ViReal64  calibrationSignalFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalFrequency7GHzPlus(
                            ViSession instrSession,
                            ViReal64  calibrationSignalFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalWidebandFrequency(
                            ViSession instrSession,
                            ViReal64  calibrationSignalFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureCalibrationSignalPeakDistance(
                            ViSession instrSession,
                            ViInt32   peakDistance);

ViStatus _VI_FUNC rsfpl_QueryFrontendTemperature(ViSession instrSession,
                                                 ViReal64* frontendTemperature);

ViStatus _VI_FUNC rsfpl_QuerySelfTestResults(ViSession instrSession,
                                             ViUInt32  arraySize,
                                             ViChar    selfTestResults[],
                                             ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_RecallCalibration(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SaveCalibration(ViSession instrSession,
                                        ViString  filePath);

ViStatus _VI_FUNC rsfpl_QueryCalibrationSaveFilePath(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    filePath[]);

ViStatus _VI_FUNC rsfpl_StartServiceFunction(ViSession instrSession,
                                             ViString  serviceFunction);

ViStatus _VI_FUNC rsfpl_QueryServiceFunctionLastResult(ViSession instrSession,
                                                       ViUInt32  arraySize,
                                                       ViChar    lastResult[],
                                                       ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ServiceFunctionDeleteLastResult(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SaveServiceFunctionResult(ViSession instrSession,
                                                  ViString  fileName);

ViStatus _VI_FUNC rsfpl_QueryServiceSystemInformationFilePath(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    filePath[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ServiceFunctionResetPassword(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ProvideServiceFunctionPassword(ViSession instrSession,
                                                       ViString  serviceFunctionPassword);

ViStatus _VI_FUNC rsfpl_ReadYTrace(ViSession instrSession,
                                   ViInt32   window,
                                   ViInt32   trace,
                                   ViUInt32  maximumTime,
                                   ViUInt32  arrayLength,
                                   ViUInt32* actualPoints,
                                   ViReal64  amplitude[]);

ViStatus _VI_FUNC rsfpl_FetchYTrace(ViSession instrSession,
                                    ViInt32   window,
                                    ViInt32   trace,
                                    ViUInt32  arrayLength,
                                    ViUInt32* actualPoints,
                                    ViReal64  amplitude[]);

ViStatus _VI_FUNC rsfpl_FetchYTracePrevious(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViInt32   offsetSweepPoint,
                                            ViInt32   numberOfSweeps,
                                            ViUInt32  arrayLength,
                                            ViUInt32* actualPoints,
                                            ViReal64  amplitude[]);

ViStatus _VI_FUNC rsfpl_FetchXTrace(ViSession instrSession,
                                    ViInt32   window,
                                    ViInt32   trace,
                                    ViUInt32  arrayLength,
                                    ViUInt32* actualPoints,
                                    ViReal64  x[]);

ViStatus _VI_FUNC rsfpl_GetLimitCheckResult(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   limit,
                                            ViInt32*  state);

ViStatus _VI_FUNC rsfpl_ConfigureMeasurementMode(ViSession instrSession,
                                                 ViInt32   measurementMode);

ViStatus _VI_FUNC rsfpl_QueryPowerResults(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViInt32   powerMeasurement,
                                          ViUInt32  arraySize,
                                          ViReal64  results[],
                                          ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_DisablePowerMeasurement(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigurePowerMeasurementTrace(ViSession instrSession,
                                                       ViInt32   trace,
                                                       ViInt32   displayMode);

ViStatus _VI_FUNC rsfpl_PowerMeasurementAdjustSettings(ViSession instrSession,
                                                       ViInt32   measurement);

ViStatus _VI_FUNC rsfpl_PowerMeasurementsFindReference(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_PowerMeasurementSelectAndEnable(ViSession instrSession,
                                                        ViInt32   measurement);

ViStatus _VI_FUNC rsfpl_QueryPowerMeasurementResult(ViSession instrSession,
                                                    ViInt32   resultType,
                                                    ViReal64* resultValue);

ViStatus _VI_FUNC rsfpl_ConfigureCompressionPointMeasurementEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureCompressionPointMeasurementReference(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViReal64  value);

ViStatus _VI_FUNC rsfpl_ConfigureCompressionPointMeasurementSelectCompressionPoints(
                            ViSession instrSession,
                            ViBoolean _1DBPointEnabled,
                            ViBoolean _3DBPointEnabled,
                            ViBoolean nDBPointEnabled,
                            ViReal64  nDBValue);

ViStatus _VI_FUNC rsfpl_QueryCompressionPointMeasurement1dBResult(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  results[]);

ViStatus _VI_FUNC rsfpl_QueryCompressionPointMeasurement3dBResult(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  results[]);

ViStatus _VI_FUNC rsfpl_QueryCompressionPointMeasurementNdBResult(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViReal64  results[]);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerDisplayMode(ViSession instrSession,
                                                         ViInt32   channelPowerDisplayMode);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAsDensityEnabled(
                            ViSession instrSession,
                            ViBoolean powerAsDensityEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerNumberOfTransmissionChannels(
                            ViSession instrSession,
                            ViInt32   numberOfTransmissionChannels);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerChannelPairs(ViSession instrSession,
                                                          ViInt32   channelPowerChannelPairs);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerWeightingFilterAll(
                            ViSession instrSession,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerACLRLimitCheckEnabled(
                            ViSession instrSession,
                            ViBoolean limitCheckEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureACLRSelectedMeasurementConfiguration(
                            ViSession instrSession,
                            ViInt32   configuration);

ViStatus _VI_FUNC rsfpl_QueryACLRMeasurementConfigurationsCatalog(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    configurationsCatalog[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ACLRMeasurementConfigurationsLoad(ViSession instrSession,
                                                          ViString  configurationName);

ViStatus _VI_FUNC rsfpl_ACLRMeasurementConfigurationsSave(ViSession instrSession,
                                                          ViString  configurationName);

ViStatus _VI_FUNC rsfpl_ACLRMeasurementConfigurationsDelete(ViSession instrSession,
                                                            ViString  configurationName);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerTxChannel(ViSession instrSession,
                                                       ViInt32   channelNumber,
                                                       ViString  name,
                                                       ViReal64  bandwidth,
                                                       ViReal64  spacing);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerTxChannelWeightingFilter(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentChannel(ViSession instrSession,
                                                             ViString  name,
                                                             ViReal64  bandwidth,
                                                             ViReal64  spacing);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentChannelWeightingFilter(
                            ViSession instrSession,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentLimitAbsolute(
                            ViSession instrSession,
                            ViBoolean limitCheckEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAdjacentLimitRelative(
                            ViSession instrSession,
                            ViBoolean lowerEnabled,
                            ViBoolean upperEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsfpl_QueryChannelPowerAdjacentLimitCheckResult(
                            ViSession instrSession,
                            ViInt32*  lower,
                            ViInt32*  upper);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateChannel(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViString  channelName,
                            ViReal64  bandwidth,
                            ViReal64  spacing);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateChannelWeightingFilter(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean filterEnabled,
                            ViReal64  filterValue);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateLimitAbsolute(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean lowerEnabled,
                            ViBoolean upperEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsfpl_ConfigureChannelPowerAlternateLimitRelative(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViBoolean lowerEnabled,
                            ViBoolean upperEnabled,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsfpl_QueryChannelPowerAlternateLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   channelNumber,
                            ViInt32*  lower,
                            ViInt32*  upper);

ViStatus _VI_FUNC rsfpl_ReferenceChannelSetCPReference(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceChannelNumberAutoType(
                            ViSession instrSession,
                            ViInt32   channelNumberAutoType);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceChannelNumber(ViSession instrSession,
                                                        ViInt32   referenceChannelNumber);

ViStatus _VI_FUNC rsfpl_ConfigurePercentOfPowerBandwidth(ViSession instrSession,
                                                         ViReal64  percentOfPowerBandwidth);

ViStatus _VI_FUNC rsfpl_SEMInitiate(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureSEMNumberOfSubBlocks(ViSession instrSession,
                                                      ViInt32   SEMNumberOfSubBlocks);

ViStatus _VI_FUNC rsfpl_SEMConfigurationsRestoreDefault(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SEMConfigurationsLoad(ViSession instrSession,
                                              ViString  configurationName);

ViStatus _VI_FUNC rsfpl_SEMConfigurationsSave(ViSession instrSession,
                                              ViString  configurationName);

ViStatus _VI_FUNC rsfpl_SEMConfigurationsRestoreLimits(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureSEMSubBlockFrequency(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViReal64  frequency);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListFastSEMEnabled(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViBoolean sweepListFastSEMEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListSymmetricalSetupEnabled(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_QuerySweepListNumberOfRanges(ViSession instrSession,
                                                     ViInt32   subBlock,
                                                     ViInt32*  sweepListNumberOfRanges);

ViStatus _VI_FUNC rsfpl_QuerySEMPowerMeasurementResult(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViInt32   resultType,
                                                       ViUInt32  arraySize,
                                                       ViReal64  results[],
                                                       ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesMode(ViSession instrSession,
                                                     ViInt32   subBlock,
                                                     ViInt32   SEMPowerClassesMode);

ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesNumberOfClasses(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   numberOfClasses);

ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesManualLimitLine(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViReal64  manualLimitLine);

ViStatus _VI_FUNC rsfpl_ConfigureSEMPowerClassesRangeDefinition(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViUInt32  arraySize,
                            ViReal64  ranges[]);

ViStatus _VI_FUNC rsfpl_SweepListRangeInsert(ViSession instrSession,
                                             ViInt32   subBlock,
                                             ViInt32   range,
                                             ViInt32   insertMode);

ViStatus _VI_FUNC rsfpl_SweepListRangeDelete(ViSession instrSession,
                                             ViInt32   subBlock,
                                             ViInt32   range);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeStartStop(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViInt32   range,
                                                         ViReal64  start,
                                                         ViReal64  stop);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeFilterType(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32   range,
                                                          ViInt32   filterType);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeRBW(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   range,
                                                   ViReal64  RBW);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeVBW(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   range,
                                                   ViReal64  VBW);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeSweepTime(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViInt32   range,
                                                         ViBoolean sweepTimeAutoEnabled,
                                                         ViReal64  sweepTimeManual);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeReferenceLevel(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViReal64  referenceLevel);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeAttenuation(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   range,
                                                           ViBoolean attenuationAutoEnabled,
                                                           ViReal64  attenuationManual);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangePreamplifierEnabled(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViBoolean preamplifierEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangePreamplifierLevel(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViReal64  level);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeTransducerFileName(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViString  transducerFileName);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeMultiLimitCalculation(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   multiLimitCalculation);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListRangeMinSweepPoints(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   minSweepPoints);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassEnabled(ViSession instrSession,
                                                            ViInt32   subBlock,
                                                            ViInt32   powerClass,
                                                            ViBoolean powerClassEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitCheckMode(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   powerClass,
                            ViInt32   limitCheckMode);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitCheckModeAllRanges(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   powerClass,
                            ViInt32   limitCheckModeAllRanges);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimit(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32   powerClass,
                                                          ViReal64  min,
                                                          ViReal64  max);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitAbsolute(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViReal64  absoluteStart,
                            ViReal64  absoluteStop);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitRelative(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViReal64  relativeStart,
                            ViReal64  relativeStop);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitRelativeStartFunction(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViInt32   relativeStartFunction,
                            ViReal64  maxStartFunctionLimit);

ViStatus _VI_FUNC rsfpl_ConfigureSweepListPowerClassLimitRelativeStopFunction(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViInt32   relativeStopFunction,
                            ViReal64  maxStopFunctionLimit);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceRangePowerReferenceType(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   referenceType);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceRangeTxBandwidth(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViReal64  txBandwidth);

ViStatus _VI_FUNC rsfpl_ConfigureReferenceRangeRRCFilter(ViSession instrSession,
                                                         ViInt32   subBlock,
                                                         ViBoolean RRCFilterEnabled,
                                                         ViReal64  alpha);

ViStatus _VI_FUNC rsfpl_QueryReferenceRangeNumber(ViSession instrSession,
                                                  ViInt32   subBlock,
                                                  ViInt32*  referenceRangeNumber);

ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationEnabled(ViSession instrSession,
                                                          ViBoolean evaluationEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationAllDetectedPeaksIncluded(
                            ViSession instrSession,
                            ViBoolean allDetectedPeaksIncluded);

ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationThreshold(ViSession instrSession,
                                                            ViReal64  threshold);

ViStatus _VI_FUNC rsfpl_ConfigureSEMListEvaluationPeakLabelsEnabled(
                            ViSession instrSession,
                            ViBoolean evaluationPeakLabelsEnabled);

ViStatus _VI_FUNC rsfpl_SEMListEvaluationInitiate(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SpuriousEmissionsInitiate(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangesEnabled(
                            ViSession instrSession,
                            ViBoolean rangesEnabled);

ViStatus _VI_FUNC rsfpl_QuerySpuriousEmissionsNumberOfRanges(ViSession instrSession,
                                                             ViInt32*  numberOfRanges);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsBreakMeasurementEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_AdjustSpuriousEmissionsXAxis(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SpuriousEmissionsRangeDelete(ViSession instrSession,
                                                     ViInt32   range);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeStartStop(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  start,
                            ViReal64  stop);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeFilterType(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   filterType);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeRBW(ViSession instrSession,
                                                           ViInt32   range,
                                                           ViReal64  RBW);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeVBW(ViSession instrSession,
                                                           ViInt32   range,
                                                           ViReal64  VBW);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeSweepTime(
                            ViSession instrSession,
                            ViInt32   range,
                            ViBoolean sweepTimeAutoEnabled,
                            ViReal64  sweepTimeManual);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeDetector(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   detector);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeReferenceLevel(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  referenceLevel);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeAttenuation(
                            ViSession instrSession,
                            ViInt32   range,
                            ViBoolean attenuationAutoEnabled,
                            ViReal64  attenuationManual);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangePreamplifier(
                            ViSession instrSession,
                            ViInt32   range,
                            ViBoolean preamplifierEnabled,
                            ViReal64  preamplificationLevel);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeSweepPoints(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   sweepPoints);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeTransducerFileName(
                            ViSession instrSession,
                            ViInt32   range,
                            ViString  transducerFileName);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsRangeAbsLimit(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  start,
                            ViReal64  stop);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationEnabled(
                            ViSession instrSession,
                            ViBoolean listEvaluationEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationNumberOfPeaks(
                            ViSession instrSession,
                            ViInt32   numberOfPeaks);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationAllDetectedPeaksIncluded(
                            ViSession instrSession,
                            ViBoolean allDetectedPeaksIncluded);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationThreshold(
                            ViSession instrSession,
                            ViReal64  threshold);

ViStatus _VI_FUNC rsfpl_ConfigureSpuriousEmissionsListEvaluationPeakLabelsEnabled(
                            ViSession instrSession,
                            ViBoolean labelsEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureStatisticalMeasurement(ViSession instrSession,
                                                        ViBoolean statisticsEnabled,
                                                        ViInt32   measurement,
                                                        ViInt32   numberOfSamples);

ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticScaling(ViSession instrSession,
                                                        ViReal64  xRefLevel,
                                                        ViReal64  xRange,
                                                        ViInt32   yUnit,
                                                        ViReal64  yMinimum,
                                                        ViReal64  yMaximum);

ViStatus _VI_FUNC rsfpl_SetSignalStatisticScalingSettings(ViSession instrSession,
                                                          ViInt32   configuration);

ViStatus _VI_FUNC rsfpl_QueryStatisticalMeasurementResults(ViSession instrSession,
                                                           ViInt32   trace,
                                                           ViInt32   resultType,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedResults);

ViStatus _VI_FUNC rsfpl_QueryStatisticalMeasurementCCDFLevel(ViSession instrSession,
                                                             ViInt32   trace,
                                                             ViInt32   probability,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticGatePeriod(ViSession instrSession,
                                                           ViInt32   trace,
                                                           ViReal64  gatePeriod);

ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticGateComment(ViSession instrSession,
                                                            ViInt32   trace,
                                                            ViString  gateComment);

ViStatus _VI_FUNC rsfpl_ConfigureSignalStatisticGateRange(ViSession instrSession,
                                                          ViInt32   trace,
                                                          ViInt32   gate,
                                                          ViBoolean gateRangeEnabled,
                                                          ViReal64  gateRangeStart,
                                                          ViReal64  gateRangeStop);

ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerEnabled(ViSession instrSession,
                                                        ViBoolean timeDomainPowerEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerAveragingEnabled(
                            ViSession instrSession,
                            ViBoolean averagingEnabled);

ViStatus _VI_FUNC rsfpl_TimeDomainPowerAllOff(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerPeakHoldEnabled(
                            ViSession instrSession,
                            ViBoolean peakHoldEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureTimeDomainPowerMeasurementEnabled(
                            ViSession instrSession,
                            ViInt32   timeDomainPower,
                            ViBoolean measurementEnabled);

ViStatus _VI_FUNC rsfpl_QueryTimeDomainPowerResult(ViSession instrSession,
                                                   ViInt32   timeDomainPower,
                                                   ViInt32   statisticalType,
                                                   ViReal64* result);

ViStatus _VI_FUNC rsfpl_ConfigureHarmonicDistortionEnabled(ViSession instrSession,
                                                           ViBoolean harmonicDistortionEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureHarmonicDistortionNumberOfHarmonics(
                            ViSession instrSession,
                            ViInt32   numberOfHarmonics);

ViStatus _VI_FUNC rsfpl_ConfigureHarmonicDistortionRBWAutoEnabled(
                            ViSession instrSession,
                            ViBoolean RBWAutoEnabled);

ViStatus _VI_FUNC rsfpl_HarmonicDistortionInitiate(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryHarmonicDistortionTHDResult(ViSession instrSession,
                                                         ViReal64* THDInPct,
                                                         ViReal64* THDInDB);

ViStatus _VI_FUNC rsfpl_QueryHarmonicDistortionHarmonicsPositions(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  harmonicsList[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureThirdOrderInterceptEnabled(ViSession instrSession,
                                                            ViBoolean thirdOrderInterceptEnabled);

ViStatus _VI_FUNC rsfpl_ThirdOrderInterceptSearchSignal(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryThirdOrderInterceptResult(ViSession instrSession,
                                                       ViReal64* thirdOrderInterceptResult);

ViStatus _VI_FUNC rsfpl_QueryThirdOrderInterceptResultMaximum(
                            ViSession instrSession,
                            ViReal64* maximum);

ViStatus _VI_FUNC rsfpl_QueryThirdOrderInterceptResultMinimum(
                            ViSession instrSession,
                            ViReal64* minimum);

ViStatus _VI_FUNC rsfpl_ConfigureAMModulationDepthEnabled(ViSession instrSession,
                                                          ViBoolean AMModulationDepthEnabled);

ViStatus _VI_FUNC rsfpl_AMModulationDepthSearchSignal(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryAMModulationDepthResult(ViSession instrSession,
                                                     ViReal64* AMModulationDepthResult);

ViStatus _VI_FUNC rsfpl_DefineListEvaluationParameters(ViSession instrSession,
                                                       ViBoolean peakPower,
                                                       ViBoolean RMSPower,
                                                       ViBoolean AVGPower,
                                                       ViString  triggerSource,
                                                       ViInt32   triggerSlope,
                                                       ViReal64  triggerOffset,
                                                       ViReal64  gateLength);

ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurement(
                            ViSession instrSession,
                            ViInt32   noOfListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  RFAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  RBW[],
                            ViReal64  VBW[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[]);

ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurementElectronic(
                            ViSession instrSession,
                            ViInt32   noOfListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  RFAttenuation[],
                            ViReal64  electronicAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  RBW[],
                            ViReal64  VBW[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[]);

ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurementSynchronized(
                            ViSession instrSession,
                            ViInt32   noOfListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  RFAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  RBW[],
                            ViReal64  VBW[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[],
                            ViUInt32  timeout);

ViStatus _VI_FUNC rsfpl_ConfigureAndInitiateListEvaluationMeasurementElectronicSynchronized(
                            ViSession instrSession,
                            ViInt32   noOfListItems,
                            ViReal64  frequency[],
                            ViReal64  referenceLevel[],
                            ViReal64  RFAttenuation[],
                            ViReal64  electronicAttenuation[],
                            ViInt32   filterType[],
                            ViReal64  RBW[],
                            ViReal64  VBW[],
                            ViReal64  measTime[],
                            ViReal64  triggerLevel[],
                            ViUInt32  timeout);

ViStatus _VI_FUNC rsfpl_QueryListEvaluationResults(ViSession instrSession,
                                                   ViUInt32  noOfResults,
                                                   ViReal64  listPowerResults[],
                                                   ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ReadListEvaluationMeasurement(ViSession instrSession,
                                                      ViUInt32  noOfListItems,
                                                      ViReal64  frequency[],
                                                      ViReal64  referenceLevel[],
                                                      ViReal64  RFAttenuation[],
                                                      ViInt32   filterType[],
                                                      ViReal64  RBW[],
                                                      ViReal64  VBW[],
                                                      ViReal64  measTime[],
                                                      ViReal64  triggerLevel[],
                                                      ViUInt32  arraySize,
                                                      ViReal64  results[],
                                                      ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_DisableListEvaluation(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigurePulsePowerMeasurementFilterType(
                            ViSession instrSession,
                            ViInt32   powerMeasurementFilterType);

ViStatus _VI_FUNC rsfpl_QueryPulsePowerMeasurementMinimum(ViSession instrSession,
                                                          ViReal64* powerMeasurementMinimum);

ViStatus _VI_FUNC rsfpl_QueryPulsePowerMeasurementListOfPulsePowers(
                            ViSession instrSession,
                            ViUInt32  noOfResults,
                            ViReal64  listPowerResults[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigurePulsePowerMeasurement(ViSession instrSession,
                                                       ViReal64  timeOffset,
                                                       ViReal64  measTime,
                                                       ViReal64  pulsePeriod,
                                                       ViInt32   numberOfPulses);

ViStatus _VI_FUNC rsfpl_ConfigureAndInitiatePulsePowerMeasurement(
                            ViSession instrSession,
                            ViReal64  frequency,
                            ViReal64  RBW,
                            ViReal64  measTime,
                            ViInt32   triggerSource,
                            ViReal64  triggerLevel,
                            ViReal64  triggerOffset,
                            ViInt32   detector,
                            ViInt32   numberOfPulses);

ViStatus _VI_FUNC rsfpl_ConfigureAndInitiatePulsePowerMeasurementSynchronized(
                            ViSession instrSession,
                            ViReal64  frequency,
                            ViReal64  RBW,
                            ViReal64  measTime,
                            ViInt32   triggerSource,
                            ViReal64  triggerLevel,
                            ViReal64  triggerOffset,
                            ViInt32   detector,
                            ViInt32   numberOfPulses);

ViStatus _VI_FUNC rsfpl_ConfigureAndReadPulsePowerMeasurement(
                            ViSession instrSession,
                            ViReal64  frequency,
                            ViReal64  RBW,
                            ViReal64  measTime,
                            ViInt32   triggerSource,
                            ViReal64  triggerLevel,
                            ViReal64  triggerOffset,
                            ViInt32   detector,
                            ViInt32   numberOfPulses,
                            ViUInt32  noOfResults,
                            ViReal64  listPowerResults[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_InitiateSynchronized(ViSession instrSession,
                                             ViUInt32  maximumTime);

ViStatus _VI_FUNC rsfpl_Initiate(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_Abort(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ContinueMeasurement(ViSession instrSession,
                                            ViUInt32  maximumTime);

ViStatus _VI_FUNC rsfpl_SendSWTrigger(ViSession instrSession,
                                      ViUInt32  maximumTime);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerGating(ViSession instrSession,
                                                  ViBoolean gatingEnabled,
                                                  ViInt32   mode,
                                                  ViInt32   gap,
                                                  ViInt32   length,
                                                  ViInt32   numberOfGatePeriods);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerEnabled(ViSession instrSession,
                                                   ViBoolean IQAnalyzerEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerEvaluationEnabled(ViSession instrSession,
                                                             ViBoolean evaluationEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerSettings(ViSession instrSession,
                                                    ViReal64  sampleRate,
                                                    ViInt32   triggerMode,
                                                    ViInt32   triggerSlope,
                                                    ViInt32   pretriggerSamples,
                                                    ViInt32   recordLength);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerSampleRate(ViSession instrSession,
                                                      ViReal64  sampleRate);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerAnalysisBandwidth(ViSession instrSession,
                                                             ViReal64  analysisBandwidth);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerSwapIQEnabled(ViSession instrSession,
                                                         ViBoolean swapIQEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerRecordLength(ViSession instrSession,
                                                        ViInt32   recordLength);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerRBW(ViSession instrSession,
                                               ViInt32   RBWMode,
                                               ViReal64  manualRBW);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerAdvancedFFTMode(ViSession instrSession,
                                                           ViInt32   transformationAlgorithm,
                                                           ViInt32   FFTLength,
                                                           ViInt32   windowFunction,
                                                           ViReal64  windowOverlap,
                                                           ViInt32   windowLength);

ViStatus _VI_FUNC rsfpl_QueryIQAnalyzerTriggerOffset(ViSession instrSession,
                                                     ViReal64* triggerOffset);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerBranchForPeaksearch(
                            ViSession instrSession,
                            ViInt32   branchForPeaksearch);

ViStatus _VI_FUNC rsfpl_IQAnalyzerLayoutAddWindow(ViSession instrSession,
                                                  ViString  windowName,
                                                  ViInt32   direction,
                                                  ViInt32   windowType,
                                                  ViUInt32  arraySize,
                                                  ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsfpl_IQAnalyzerLayoutReplaceWindow(ViSession instrSession,
                                                      ViString  windowName,
                                                      ViInt32   windowType);

ViStatus _VI_FUNC rsfpl_FetchIQData(ViSession instrSession,
                                    ViInt32   offsetSamples,
                                    ViUInt32  numberOfSamples,
                                    ViReal64  realParts_I[],
                                    ViReal64  imaginaryParts_Q[],
                                    ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ReadIQData(ViSession instrSession,
                                   ViUInt32  timeout,
                                   ViUInt32  arraySize,
                                   ViReal64  realParts_I[],
                                   ViReal64  imaginaryParts_Q[],
                                   ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_ReadIQDataToPCFile(ViSession instrSession,
                                           ViUInt32  timeout,
                                           ViString  fileName);

ViStatus _VI_FUNC rsfpl_ConfigureIQAnalyzerIQDataComment(ViSession instrSession,
                                                         ViString  comment);

ViStatus _VI_FUNC rsfpl_SaveIQData(ViSession instrSession,
                                   ViString  fileName);

ViStatus _VI_FUNC rsfpl_LoadIQData(ViSession instrSession,
                                   ViString  fileName);

ViStatus _VI_FUNC rsfpl_SelectIQDataFile(ViSession instrSession,
                                         ViString  filename,
                                         ViReal64  analysisBW);

ViStatus _VI_FUNC rsfpl_SetNoiseMode(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCommonSourceEnabled(ViSession instrSession,
                                                          ViBoolean commonSourceEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseRoomTemperature(ViSession instrSession,
                                                      ViReal64  noiseRoomTemperature);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationNoiseSource(ViSession instrSession,
                                                             ViInt32   calibrationNoiseSource);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationMode(ViSession instrSession,
                                                      ViInt32   noiseCalibrationMode);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationConstantENR(ViSession instrSession,
                                                             ViReal64  constantENR);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationSpot(ViSession instrSession,
                                                      ViReal64  coldTemperature,
                                                      ViReal64  hotTemperature);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationSmartSerialNumber(
                            ViSession instrSession,
                            ViString  serialNumber);

ViStatus _VI_FUNC rsfpl_SelectCalibrationTable(ViSession instrSession,
                                               ViString  tableName);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementNoiseSource(ViSession instrSession,
                                                             ViInt32   measurementNoiseSource);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseSourceMeasurementMode(ViSession instrSession,
                                                            ViInt32   measurementMode);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementConstantENR(ViSession instrSession,
                                                             ViReal64  constantENR);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementSpot(ViSession instrSession,
                                                      ViReal64  coldTemperature,
                                                      ViReal64  hotTemperature);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementSmartSerialNumber(
                            ViSession instrSession,
                            ViString  serialNumber);

ViStatus _VI_FUNC rsfpl_SelectNoiseMeasurementTable(ViSession instrSession,
                                                    ViString  tableName);

ViStatus _VI_FUNC rsfpl_DefineNoiseMeasurementDiodeTable(ViSession instrSession,
                                                         ViUInt32  arraySize,
                                                         ViReal64  frequencies[],
                                                         ViReal64  ENRs[]);

ViStatus _VI_FUNC rsfpl_QueryNoiseMeasurementDiodeTableList(ViSession instrSession,
                                                            ViUInt32  arraySize,
                                                            ViChar    tableList[],
                                                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_NoiseMeasurementDiodeTableDelete(ViSession instrSession,
                                                         ViString  tableName);

ViStatus _VI_FUNC rsfpl_DefineNoiseMeasurementTemperatureTable(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  frequencies[],
                            ViReal64  thot[],
                            ViReal64  tcold[]);

ViStatus _VI_FUNC rsfpl_QueryNoiseMeasurementTemperatureTableList(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    tableList[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_NoiseMeasurementTemperatureTableDelete(
                            ViSession instrSession,
                            ViString  tableName);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseInputLossMode(ViSession instrSession,
                                                    ViInt32   mode);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseInputLossConstant(ViSession instrSession,
                                                        ViReal64  constantLoss);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseInputLossTemperature(ViSession instrSession,
                                                           ViReal64  inputCorrectionTemperature);

ViStatus _VI_FUNC rsfpl_SelectNoiseInputLossTable(ViSession instrSession,
                                                  ViString  tableName);

ViStatus _VI_FUNC rsfpl_DefineNoiseInputLossTable(ViSession instrSession,
                                                  ViUInt32  arraySize,
                                                  ViReal64  frequencies[],
                                                  ViReal64  loss[]);

ViStatus _VI_FUNC rsfpl_DeleteNoiseInputLossTable(ViSession instrSession,
                                                  ViString  tableName);

ViStatus _VI_FUNC rsfpl_QueryNoiseInputLossTableList(ViSession instrSession,
                                                     ViUInt32  bufferSize,
                                                     ViChar    inputLossTableList[],
                                                     ViInt32*  listLength);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseOutputLossMode(ViSession instrSession,
                                                     ViInt32   mode);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseOutputLossConstant(ViSession instrSession,
                                                         ViReal64  constantLoss);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseOutputLossTemperature(ViSession instrSession,
                                                            ViReal64  outputCorrectionTemperature);

ViStatus _VI_FUNC rsfpl_SelectNoiseOutputLossTable(ViSession instrSession,
                                                   ViString  tableName);

ViStatus _VI_FUNC rsfpl_DefineNoiseOutputLossTable(ViSession instrSession,
                                                   ViUInt32  arraySize,
                                                   ViReal64  frequencies[],
                                                   ViReal64  loss[]);

ViStatus _VI_FUNC rsfpl_DeleteNoiseOutputLossTable(ViSession instrSession,
                                                   ViString  tableName);

ViStatus _VI_FUNC rsfpl_QueryNoiseOutputLossTableList(ViSession instrSession,
                                                      ViUInt32  bufferSize,
                                                      ViChar    outputLossTableList[],
                                                      ViInt32*  listLength);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationLossMode(ViSession instrSession,
                                                          ViInt32   mode);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationLossConstant(
                            ViSession instrSession,
                            ViReal64  constantLoss);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseCalibrationLossTemperature(
                            ViSession instrSession,
                            ViReal64  correctionTemperature);

ViStatus _VI_FUNC rsfpl_SelectNoiseCalibrationLossTable(ViSession instrSession,
                                                        ViString  tableName);

ViStatus _VI_FUNC rsfpl_DefineNoiseCalibrationLossTable(ViSession instrSession,
                                                        ViUInt32  arraySize,
                                                        ViReal64  frequencies[],
                                                        ViReal64  loss[]);

ViStatus _VI_FUNC rsfpl_DeleteNoiseCalibrationLossTable(ViSession instrSession,
                                                        ViString  tableName);

ViStatus _VI_FUNC rsfpl_QueryNoiseCalibrationLossTableList(ViSession instrSession,
                                                           ViUInt32  bufferSize,
                                                           ViChar    calibrationLossTableList[],
                                                           ViInt32*  listLength);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseDUTType(ViSession instrSession,
                                              ViInt32   noiseDUTType);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseDUTLOType(ViSession instrSession,
                                                ViInt32   noiseDUTLOType);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseDUTMixer(ViSession instrSession,
                                               ViReal64  LOFrequency,
                                               ViReal64  imageRejection);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementPoints(ViSession instrSession,
                                                        ViInt32   measurementPoints);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequency(ViSession instrSession,
                                                      ViReal64  singleFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequencyCouplingEnabled(
                            ViSession instrSession,
                            ViBoolean frequencyCouplingEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequencyContinuousSweep(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseSingleFrequencySingleSweep(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFrequencyStepsize(ViSession instrSession,
                                                        ViReal64  stepsize);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFrequencyTableData(ViSession instrSession,
                                                         ViUInt32  arraySize,
                                                         ViReal64  data[]);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseListMeasurementContinuousSweep(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseListMeasurementSingleSweep(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryNoiseFrequencyTableData(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViReal64  RFFrequency[],
                                                     ViReal64  LOFrequency[],
                                                     ViReal64  IFFrequency[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsfpl_QueryNoiseData(ViSession instrSession,
                                       ViInt32   bufferSize,
                                       ViReal64  frequency[],
                                       ViReal64  bandwidth[],
                                       ViReal64  sweepTime[],
                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerSecondStageCorrection(
                            ViSession instrSession,
                            ViBoolean secondStageCorrectionEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerSettlingTime(ViSession instrSession,
                                                           ViReal64  settlingTime);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerAutoReferenceLevelEnabled(
                            ViSession instrSession,
                            ViBoolean autoReferenceLevelEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseAnalyzerExpectedDUTGain(
                            ViSession instrSession,
                            ViReal64  expectedDUTGain);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseLimitLineResultType(ViSession instrSession,
                                                          ViInt32   limit,
                                                          ViInt32   resultType);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseDisplayItems(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   items,
                                                   ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseDisplayAutomaticScalingEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean automaticScalingEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseDisplayXAxis(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   frequencyType,
                                                   ViBoolean symbolsEnabled,
                                                   ViReal64  bottom,
                                                   ViReal64  top);

ViStatus _VI_FUNC rsfpl_NoiseLayoutAddWindow(ViSession instrSession,
                                             ViString  windowName,
                                             ViInt32   direction,
                                             ViInt32   windowType,
                                             ViUInt32  arraySize,
                                             ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsfpl_NoiseLayoutReplaceWindow(ViSession instrSession,
                                                 ViString  windowName,
                                                 ViInt32   windowType);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementMode(ViSession instrSession,
                                                      ViInt32   measurementMode);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseMeasurementTypeOfPowerMeasurement(
                            ViSession instrSession,
                            ViInt32   typeOfPowerMeasurement);

ViStatus _VI_FUNC rsfpl_FetchNoiseTraceData(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViInt32   resultType,
                                            ViUInt32  arrayLength,
                                            ViUInt32* actualPoints,
                                            ViReal64  result[]);

ViStatus _VI_FUNC rsfpl_QueryNoiseMarkerResults(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   resultType,
                                                ViReal64* result);

ViStatus _VI_FUNC rsfpl_QueryNoiseDeltaMarkerResults(ViSession instrSession,
                                                     ViInt32   deltaMarker,
                                                     ViInt32   resultType,
                                                     ViReal64* result);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintySource(
                            ViSession instrSession,
                            ViInt32   outputMatch,
                            ViReal64  outputMatchValue,
                            ViReal64  ENRUncertainty);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyResistorTempUncertainty(
                            ViSession instrSession,
                            ViReal64  hotUncertainty,
                            ViReal64  coldUncertainty);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyCalibrationSource(
                            ViSession instrSession,
                            ViBoolean commonSource,
                            ViInt32   outputMatch,
                            ViReal64  outputMatchValue,
                            ViReal64  ENRUncertainty);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyCalibrationResistorTempUncertainty(
                            ViSession instrSession,
                            ViReal64  hotUncertainty,
                            ViReal64  coldUncertainty);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyDUTInputMatch(
                            ViSession instrSession,
                            ViInt32   inputMatch,
                            ViReal64  inputMatchValue);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyDUTOutputMatch(
                            ViSession instrSession,
                            ViInt32   outputMatch,
                            ViReal64  outputMatchValue);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyDUTCharacteristics(
                            ViSession instrSession,
                            ViBoolean useMeasValues,
                            ViReal64  noiseFigure,
                            ViReal64  gain,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsfpl_ConfigureNoiseFigureUncertaintyPreampCharacteristics(
                            ViSession instrSession,
                            ViBoolean preampEnabled,
                            ViReal64  noiseFigure,
                            ViReal64  gain,
                            ViInt32   inputMatch,
                            ViReal64  inputMatchValue);

ViStatus _VI_FUNC rsfpl_QueryNoiseFigureUncertaintyAnalyzerCharacteristics(
                            ViSession instrSession,
                            ViReal64* noiseFigureUncertainty,
                            ViReal64* gainUncertainty);

ViStatus _VI_FUNC rsfpl_QueryNoiseFigureUncertaintyResult(ViSession instrSession,
                                                          ViReal64* uncertaintyResult);

ViStatus _VI_FUNC rsfpl_SetADEMMode(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ADEMLoadConfiguration(ViSession instrSession,
                                              ViString  fileName);

ViStatus _VI_FUNC rsfpl_ADEMSaveConfiguration(ViSession instrSession,
                                              ViString  fileName);

ViStatus _VI_FUNC rsfpl_ADEMRestoreConfigurations(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerAMLevel(ViSession instrSession,
                                                    ViReal64  AMLevel);

ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerRFLevel(ViSession instrSession,
                                                    ViReal64  RFLevel);

ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerFMLevel(ViSession instrSession,
                                                    ViReal64  FMLevel);

ViStatus _VI_FUNC rsfpl_ConfigureADEMTriggerPMLevel(ViSession instrSession,
                                                    ViReal64  PMLevel);

ViStatus _VI_FUNC rsfpl_ConfigureADEMBandwidth(ViSession instrSession,
                                               ViReal64  demodulationBandwidth,
                                               ViInt32   demodulationFilter,
                                               ViReal64  measurementTime);

ViStatus _VI_FUNC rsfpl_QueryADEMSampleRate(ViSession instrSession,
                                            ViReal64* sampleRate);

ViStatus _VI_FUNC rsfpl_QueryADEMRecordLength(ViSession instrSession,
                                              ViInt32*  recordLength);

ViStatus _VI_FUNC rsfpl_ConfigureADEMDemodulator(ViSession instrSession,
                                                 ViReal64  sampleRate,
                                                 ViInt32   recordLength,
                                                 ViInt32   triggerSource,
                                                 ViInt32   triggerSlope,
                                                 ViInt32   offsetSamples,
                                                 ViInt32   numberOfMeasurements);

ViStatus _VI_FUNC rsfpl_ConfigureADEMUnitAngle(ViSession instrSession,
                                               ViInt32   unit);

ViStatus _VI_FUNC rsfpl_ConfigureADEMUnitTHD(ViSession instrSession,
                                             ViInt32   unit);

ViStatus _VI_FUNC rsfpl_ConfigureADEMSquelch(ViSession instrSession,
                                             ViBoolean squelchEnabled,
                                             ViReal64  squelchLevel);

ViStatus _VI_FUNC rsfpl_ConfigureADEMZeroPhaseReferencePosition(
                            ViSession instrSession,
                            ViReal64  zeroPhaseReferencePosition);

ViStatus _VI_FUNC rsfpl_ConfigureADEMZeroPhaseReferencePositionMode(
                            ViSession instrSession,
                            ViInt32   mode);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFCoupling(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   AFCoupling);

ViStatus _VI_FUNC rsfpl_ConfigureADEMTimeDomainZoom(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViBoolean zoomEnabled,
                                                    ViReal64  start,
                                                    ViInt32   lengthMode,
                                                    ViReal64  length);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFCenter(ViSession instrSession,
                                              ViReal64  AFCenter);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFSpan(ViSession instrSession,
                                            ViReal64  AFSpan);

ViStatus _VI_FUNC rsfpl_SetADEMAFSpanFull(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFStart(ViSession instrSession,
                                             ViReal64  AFStart);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFStop(ViSession instrSession,
                                            ViReal64  AFStop);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFAutoScaleEnabled(ViSession instrSession,
                                                        ViBoolean AFAutoScaleEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureADEMRFSpan(ViSession instrSession,
                                            ViReal64  RFSpan);

ViStatus _VI_FUNC rsfpl_ConfigureADEMRFSpanZoom(ViSession instrSession,
                                                ViReal64  RFSpanZoom);

ViStatus _VI_FUNC rsfpl_ADEMAFFilterAllOff(ViSession instrSession,
                                           ViInt32   window);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFHighPassFilter(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViBoolean filterEnabled,
                                                      ViInt32   filterType);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFHighPassFilterCutoffFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  cutoffFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterEnabled(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViBoolean filterEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterCutoffFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  cutoffFrequency);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterTypeRelative(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   filterTypeRelative);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFLowPassFilterTypeAbsolute(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   filterTypeAbsolute);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedFilterCCITTEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean CCITTEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedFilterUnweightedCCIREnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean unweightedCCIREnabled);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedFilterWeightedCCIREnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean weightedCCIREnabled);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFWeightedAEnabled(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean aWeightedEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureADEMAFDeemphasisFilter(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean filterEnabled,
                                                        ViInt32   filterType);

ViStatus _VI_FUNC rsfpl_ConfigureADEMRFInputYIGFilter(ViSession instrSession,
                                                      ViBoolean filterEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureADEMRelativeDemodulationResultsUnit(
                            ViSession instrSession,
                            ViInt32   unit);

ViStatus _VI_FUNC rsfpl_SetADEMDemodulationReference(ViSession instrSession,
                                                     ViInt32   demodulationType);

ViStatus _VI_FUNC rsfpl_ConfigureADEMRelativeDemodulation(ViSession instrSession,
                                                          ViInt32   demodulationType,
                                                          ViInt32   detectorType,
                                                          ViBoolean relativeDemodulationEnabled,
                                                          ViInt32   resultMode,
                                                          ViReal64  referenceValue);

ViStatus _VI_FUNC rsfpl_ADEMLayoutAddWindow(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32   direction,
                                            ViInt32   windowType,
                                            ViUInt32  arraySize,
                                            ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsfpl_ADEMLayoutReplaceWindow(ViSession instrSession,
                                                ViString  windowName,
                                                ViInt32   windowType);

ViStatus _VI_FUNC rsfpl_ConfigureADEMTrace(ViSession instrSession,
                                           ViInt32   evaluationMethod,
                                           ViInt32   traceMode[]);

ViStatus _VI_FUNC rsfpl_ReadADEMTrace(ViSession instrSession,
                                      ViInt32   evaluationMethod,
                                      ViInt32   traceMode,
                                      ViUInt32  arrayLength,
                                      ViUInt32* actualPoints,
                                      ViReal64  result[]);

ViStatus _VI_FUNC rsfpl_ConfigureADEMRangeResultType(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   resultType);

ViStatus _VI_FUNC rsfpl_ADEMSelectTraceForResultSummary(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   trace);

ViStatus _VI_FUNC rsfpl_QueryADEMModulationResult(ViSession instrSession,
                                                  ViInt32   trace,
                                                  ViInt32   modulation,
                                                  ViInt32   measType,
                                                  ViReal64* modulationResult);

ViStatus _VI_FUNC rsfpl_QueryADEMModulationResultRelative(ViSession instrSession,
                                                          ViInt32   trace,
                                                          ViInt32   modulation,
                                                          ViInt32   measType,
                                                          ViReal64* modulationResult);

ViStatus _VI_FUNC rsfpl_QueryADEMResultModulationFrequency(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViReal64* modulationFrequency);

ViStatus _VI_FUNC rsfpl_QueryADEMResultCarrierPower(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViReal64* carrierPower);

ViStatus _VI_FUNC rsfpl_QueryADEMResultTHD(ViSession instrSession,
                                           ViInt32   window,
                                           ViInt32   trace,
                                           ViReal64* THD);

ViStatus _VI_FUNC rsfpl_QueryADEMResultSINAD(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   trace,
                                             ViReal64* SINAD);

ViStatus _VI_FUNC rsfpl_QueryADEMResultCarrierOffset(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViReal64* carrierOffset);

ViStatus _VI_FUNC rsfpl_QueryADEMTraceResultModulationFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViReal64* modulationFrequency);

ViStatus _VI_FUNC rsfpl_QueryADEMTraceResultCarrierPower(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   trace,
                                                         ViReal64* carrierPower);

ViStatus _VI_FUNC rsfpl_QueryADEMTraceResultDistortion(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   trace,
                                                       ViReal64* distortion);

ViStatus _VI_FUNC rsfpl_ConfigureEMIMeasurement(ViSession instrSession,
                                                ViBoolean enabled,
                                                ViReal64  dwellTime);

ViStatus _VI_FUNC rsfpl_ConfigureEMILISNVNetwork(ViSession instrSession,
                                                 ViInt32   LISNType,
                                                 ViInt32   phase,
                                                 ViBoolean highPassFilter);

ViStatus _VI_FUNC rsfpl_EMIMeasurementMarkerPeakSearch(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker);

ViStatus _VI_FUNC rsfpl_ConfigureEMIMeasurementMarkerDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   detector);

ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementMarkerResult(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViReal64* result);

ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementMarkerLimitCondition(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   limitLine,
                            ViInt32*  limitCondition);

ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementMarkerLimitVerticalDistance(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViInt32   limitLine,
                            ViReal64* verticalDistance);

ViStatus _VI_FUNC rsfpl_EMIMeasurementDeltaMarkerPeakSearch(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   deltaMarker);

ViStatus _VI_FUNC rsfpl_ConfigureEMIMeasurementDeltaMarkerDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   detector);

ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementDeltaMarkerResult(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   deltaMarker,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementDeltaMarkerLimitCondition(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   limitLine,
                            ViInt32*  limitCondition);

ViStatus _VI_FUNC rsfpl_QueryEMIMeasurementDeltaMarkerLimitVerticalDistance(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   limitLine,
                            ViReal64* verticalDistance);

ViStatus _VI_FUNC rsfpl_SetVSAMode(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SetVSAFactoryDefaults(ViSession instrSession,
                                              ViInt32   defaults);

ViStatus _VI_FUNC rsfpl_PresetVSAScreens(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_VSAReferenceLevelAdjust(ViSession instrSession,
                                                ViUInt32  timeout);

ViStatus _VI_FUNC rsfpl_ConfigureVSARFInput(ViSession instrSession,
                                            ViInt32   coupling,
                                            ViBoolean YIGFilter,
                                            ViInt32   padType);

ViStatus _VI_FUNC rsfpl_ConfigureVSAOutputOpimizationMode(ViSession instrSession,
                                                          ViInt32   optimizationMode);

ViStatus _VI_FUNC rsfpl_ConfigureVSAImportExportIQData(ViSession instrSession,
                                                       ViString  fileName,
                                                       ViInt32   IQFormat,
                                                       ViInt32   dataFormat);

ViStatus _VI_FUNC rsfpl_ConfigureVSAStandardPreset(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureVSAStandardComment(ViSession instrSession,
                                                    ViString  comment);

ViStatus _VI_FUNC rsfpl_ConfigureVSAStandardOperations(ViSession instrSession,
                                                       ViInt32   operation,
                                                       ViString  standard);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationSettings(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViReal64  symbolRate);

ViStatus _VI_FUNC rsfpl_LoadVSAModulationUserDefined(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsfpl_QueryVSAModulationMappingCatalog(ViSession instrSession,
                                                         ViInt32   arraySize,
                                                         ViChar    catalog[]);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationMapping(ViSession instrSession,
                                                      ViString  mapping);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationPSK(ViSession instrSession,
                                                  ViInt32   order,
                                                  ViInt32   nState);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationQAM(ViSession instrSession,
                                                  ViInt32   order,
                                                  ViInt32   nState);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationFSK(ViSession instrSession,
                                                  ViInt32   deviation,
                                                  ViReal64  FSKRefDeviation,
                                                  ViInt32   nState);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationOrders(ViSession instrSession,
                                                     ViInt32   ASKOrder,
                                                     ViInt32   APSKOrder,
                                                     ViInt32   MSKOrder,
                                                     ViInt32   QPSKOrder);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationTransmitFilter(ViSession instrSession,
                                                             ViString  transmitFilterSelect,
                                                             ViReal64  alphaBT);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationTransmitFilterUser(
                            ViSession instrSession,
                            ViString  userDefined);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationTransmitFilterOff(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureVSAModulationFilter(ViSession instrSession,
                                                     ViBoolean enabled,
                                                     ViReal64  alphaBT);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructureType(ViSession instrSession,
                                                        ViInt32   type);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructureBurst(ViSession instrSession,
                                                         ViInt32   minLength,
                                                         ViInt32   maxLength,
                                                         ViInt32   runIn,
                                                         ViInt32   runOut);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructurePatternEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructurePatternsCatalog(
                            ViSession instrSession,
                            ViInt32   patterns,
                            ViChar    catalog[]);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalStructurePatternOffset(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViInt32   offset);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFramePattern(ViSession instrSession,
                                                 ViBoolean sameAsDataSymbols,
                                                 ViInt32   type);

ViStatus _VI_FUNC rsfpl_QueryVSAFrameMappingCatalog(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    mappingList[]);

ViStatus _VI_FUNC rsfpl_VSALoadFrameConfig(ViSession instrSession,
                                           ViString  config);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameMappingSelect(ViSession instrSession,
                                                       ViString  frameMapping);

ViStatus _VI_FUNC rsfpl_VSALoadFrameStructure(ViSession instrSession,
                                              ViString  fileName);

ViStatus _VI_FUNC rsfpl_VSASaveFrameStructure(ViSession instrSession,
                                              ViString  fileName);

ViStatus _VI_FUNC rsfpl_VSAFrameStructureEdit(ViSession instrSession,
                                              ViString  structure);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameMode(ViSession instrSession,
                                              ViInt32   mode);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameUserFile(ViSession instrSession,
                                                  ViString  fileName);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameAPSKNState(ViSession instrSession,
                                                    ViInt32   nState);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameASKNState(ViSession instrSession,
                                                   ViInt32   nState);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFramePSK(ViSession instrSession,
                                             ViInt32   format,
                                             ViInt32   nState);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameQAM(ViSession instrSession,
                                             ViInt32   format,
                                             ViInt32   nState);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameQPSK(ViSession instrSession,
                                              ViInt32   format);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameBoosting(ViSession instrSession,
                                                  ViInt32   type,
                                                  ViReal64  boosting);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameModulation(ViSession instrSession,
                                                    ViInt32   type,
                                                    ViInt32   modulation);

ViStatus _VI_FUNC rsfpl_ConfigureVSAFrameDescription(ViSession instrSession,
                                                     ViString  description);

ViStatus _VI_FUNC rsfpl_QueryVSAFrameStartSample(ViSession instrSession,
                                                 ViReal64* startSamples);

ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownData(ViSession instrSession,
                                              ViBoolean enabled,
                                              ViInt32   source);

ViStatus _VI_FUNC rsfpl_VSAKnownDataLoadDataFile(ViSession instrSession,
                                                 ViString  fileName);

ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSType(ViSession instrSession,
                                                      ViInt32   type);

ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSPolynomial(ViSession instrSession,
                                                            ViBoolean polynomialAuto,
                                                            ViString  polynomialOrder);

ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSNegateFeedbackPath(
                            ViSession instrSession,
                            ViBoolean feedbackAuto,
                            ViBoolean feedback);

ViStatus _VI_FUNC rsfpl_ConfigureVSAKnownDataPRBSPattern(ViSession instrSession,
                                                         ViBoolean patternAuto,
                                                         ViBoolean patternEnabled);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalCapture(ViSession instrSession,
                                                  ViBoolean signalCaptureAuto,
                                                  ViInt32   sampleRate);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalCaptureRecordLength(
                            ViSession instrSession,
                            ViReal64  length);

ViStatus _VI_FUNC rsfpl_ConfigureVSASignalCaptureRecordLengthSymbols(
                            ViSession instrSession,
                            ViInt32   lengthSymbols);

ViStatus _VI_FUNC rsfpl_QuerySignalCaptureIQBandwidth(ViSession instrSession,
                                                      ViReal64* usableIQBandwidth);

ViStatus _VI_FUNC rsfpl_ConfigureVSABurstSearchAuto(ViSession instrSession,
                                                    ViInt32   burstSearch);

ViStatus _VI_FUNC rsfpl_ConfigureVSABurstSearch(ViSession instrSession,
                                                ViBoolean enabled,
                                                ViInt32   mode);

ViStatus _VI_FUNC rsfpl_ConfigureVSABurstSearchAutoConfiguration(
                            ViSession instrSession,
                            ViBoolean autoConfiguration,
                            ViInt32   tolerance,
                            ViInt32   minGapLength);

ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSearchAuto(ViSession instrSession,
                                                      ViInt32   patternSearchAuto);

ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSearch(ViSession instrSession,
                                                  ViBoolean enabled,
                                                  ViInt32   searchMode,
                                                  ViString  pattern);

ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSearchIQCorrelation(
                            ViSession instrSession,
                            ViBoolean autoThreshold,
                            ViReal64  level);

ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternOperations(ViSession instrSession,
                                                      ViInt32   operation,
                                                      ViString  patternName);

ViStatus _VI_FUNC rsfpl_ConfigureVSAPatternSettings(ViSession instrSession,
                                                    ViString  patternName,
                                                    ViString  description,
                                                    ViString  comment,
                                                    ViInt32   degreeOfModulation,
                                                    ViString  patternData);

ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeAccordingToFrameStructureFile(
                            ViSession instrSession,
                            ViBoolean toFrameStructureFile);

ViStatus _VI_FUNC rsfpl_ConfigureVSAResultLength(ViSession instrSession,
                                                 ViInt32   resultLength);

ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeReference(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   trace,
                                                         ViInt32   reference);

ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayAlignment(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViInt32   alignment,
                                                     ViInt32   offset);

ViStatus _VI_FUNC rsfpl_ConfigureVSASymbolNumberAtReference(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   trace,
                                                            ViInt32   symbolNumber);

ViStatus _VI_FUNC rsfpl_ConfigureVSAEvaluationRange(ViSession instrSession,
                                                    ViBoolean enabled,
                                                    ViInt32   start,
                                                    ViInt32   stop);

ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeNumber(ViSession instrSession,
                                                      ViInt32   resultRange);

ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangePPS(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   mode,
                                                   ViInt32   PPS);

ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRangeHighlightSymbols(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViBoolean highlightSymbols);

ViStatus _VI_FUNC rsfpl_ConfigureVSAResultRange(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   resultType,
                                                ViBoolean transformation,
                                                ViInt32   oversampling,
                                                ViInt32   individualResults);

ViStatus _VI_FUNC rsfpl_ConfigureVSACompensate(ViSession instrSession,
                                               ViBoolean IQOffset,
                                               ViBoolean IQImbalance,
                                               ViBoolean amplitudeDroop,
                                               ViBoolean symbolRateError,
                                               ViBoolean channel,
                                               ViBoolean IQSkew,
                                               ViBoolean FSK,
                                               ViBoolean deviationError);

ViStatus _VI_FUNC rsfpl_ConfigureVSAEqualizer(ViSession instrSession,
                                              ViBoolean enabled,
                                              ViInt32   mode,
                                              ViInt32   filterLength);

ViStatus _VI_FUNC rsfpl_ConfigureVSAEqualizerFileFormat(ViSession instrSession,
                                                        ViInt32   fileFormat);

ViStatus _VI_FUNC rsfpl_ConfigureVSAEqualizerOperations(ViSession instrSession,
                                                        ViInt32   operation,
                                                        ViString  equalizerName);

ViStatus _VI_FUNC rsfpl_ConfigureVSAEVMModulation(ViSession instrSession,
                                                  ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureVSADemodulationAdvanced(ViSession instrSession,
                                                         ViInt32   normalizeEVMTo,
                                                         ViInt32   optimization,
                                                         ViBoolean points);

ViStatus _VI_FUNC rsfpl_ConfigureVSADemodulationAdvancedEstimationPoints(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViInt32   points);

ViStatus _VI_FUNC rsfpl_ConfigureVSADemodulationAdvancedSynchronization(
                            ViSession instrSession,
                            ViBoolean coarseAuto,
                            ViInt32   coarseMode,
                            ViBoolean fineSyncAuto,
                            ViInt32   fineSyncMode,
                            ViReal64  fineSyncSERLevel);

ViStatus _VI_FUNC rsfpl_QueryVSADemodulationAdvancedFineSyncResult(
                            ViSession instrSession,
                            ViInt32*  fineSyncResult);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementFilter(ViSession instrSession,
                                                      ViBoolean measurementFilterAuto,
                                                      ViString  filterName,
                                                      ViReal64  alphaBT);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementFilterOff(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementFilterUser(ViSession instrSession,
                                                          ViString  fileName);

ViStatus _VI_FUNC rsfpl_ConfigureVSAPreamplifierCompensationEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureVSAAttenuationElectronic(ViSession instrSession,
                                                          ViBoolean enabled,
                                                          ViReal64  attenuation);

ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingQuantize(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   quantize);

ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingAngleUnit(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   angleUnit);

ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingTimeUnit(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   timeUnitX,
                                                     ViInt32   timeUnitY);

ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingRangePerDivision(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   trace,
                                                             ViInt32   rangePerDivision);

ViStatus _VI_FUNC rsfpl_ConfigureVSAScallingXAxisReferenceValue(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   referenceValue);

ViStatus _VI_FUNC rsfpl_ConfigureVSAWindowAddEvaluation(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean addEvaluation);

ViStatus _VI_FUNC rsfpl_ConfigureVSAWindowFeed(ViSession instrSession,
                                               ViInt32   window,
                                               ViString  feed);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMeasurementRefresh(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryVSAMeasurementStartSymbol(ViSession instrSession,
                                                       ViReal64* startSymbol);

ViStatus _VI_FUNC rsfpl_QueryVSAMeasurementDirty(ViSession  instrSession,
                                                 ViInt32    window,
                                                 ViBoolean* measurementDirty);

ViStatus _VI_FUNC rsfpl_QueryVSABitErrorRateResults(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   resultType,
                                                    ViReal64* result);

ViStatus _VI_FUNC rsfpl_QueryVSAResultsBurstLength(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32*  burstLength);

ViStatus _VI_FUNC rsfpl_GetVSAResultsAll(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   arraySize,
                                         ViInt32   marker,
                                         ViReal64  allValues[],
                                         ViInt32*  actualPoints);

ViStatus _VI_FUNC rsfpl_GetVSAResultsAmplitudeDroop(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViInt32   type,
                                                    ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsCarrierFrequencyError(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   marker,
                                                           ViInt32   type,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsEVM(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   marker,
                                         ViInt32   type,
                                         ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKDeviation(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViInt32   type,
                                                  ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKCarrierFrequency(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   marker,
                                                         ViInt32   type,
                                                         ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKFrequencyError(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker,
                                                       ViInt32   type,
                                                       ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKMeasurementDeviation(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViInt32   type,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsFSKReferenceDeviation(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   marker,
                                                           ViInt32   type,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsGainImbalance(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViInt32   type,
                                                   ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsIQImbalance(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViInt32   type,
                                                 ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsIQSkewError(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViInt32   type,
                                                 ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsIQOffset(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   marker,
                                              ViInt32   type,
                                              ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsMagnitudeError(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViInt32   type,
                                                    ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsPowerMeasurement(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViInt32   type,
                                                      ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsPhaseError(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   type,
                                                ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsQuadratureError(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViInt32   type,
                                                     ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsRHO(ViSession instrSession,
                                         ViInt32   window,
                                         ViInt32   marker,
                                         ViInt32   type,
                                         ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsSNRError(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   marker,
                                              ViInt32   type,
                                              ViReal64* result);

ViStatus _VI_FUNC rsfpl_GetVSAResultsSymbolRateError(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   marker,
                                                     ViInt32   type,
                                                     ViReal64* result);

ViStatus _VI_FUNC rsfpl_QueryVSATraceStartStop(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViReal64* start,
                                               ViReal64* stop);

ViStatus _VI_FUNC rsfpl_SaveVSATrace(ViSession instrSession,
                                     ViString  fileName);

ViStatus _VI_FUNC rsfpl_VSADSPMarkerToStartOfSelectedResultRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker);

ViStatus _VI_FUNC rsfpl_VSADSPDeltaMarkerToStartOfSelectedResultRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCaptureNumberOfBursts(ViSession instrSession,
                                                         ViInt32*  numberOfBursts);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCaptureNumberOfPatterns(ViSession instrSession,
                                                           ViInt32*  numberOfPatterns);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangeBurstLength(ViSession instrSession,
                                                           ViInt32*  length);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangeBurstPresent(ViSession  instrSession,
                                                            ViBoolean* burstPresent);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangeBurstStart(ViSession instrSession,
                                                          ViInt32*  burstStart);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternConfidence(
                            ViSession instrSession,
                            ViReal64* confidence);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternCorrect(
                            ViSession  instrSession,
                            ViBoolean* patternCorrect);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternPresent(
                            ViSession  instrSession,
                            ViBoolean* patternPresent);

ViStatus _VI_FUNC rsfpl_QueryVSADSPCurrentRangePatternStart(ViSession instrSession,
                                                            ViInt32*  patternStart);

ViStatus _VI_FUNC rsfpl_ConfigureVSAAnalysisTrace(ViSession instrSession,
                                                  ViInt32   trace,
                                                  ViInt32   traceEvaluation,
                                                  ViInt32   symbolSelection);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMarkerAbsolutePeak(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   marker);

ViStatus _VI_FUNC rsfpl_ConfigureVSADeltamarkerAbsolutePeak(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   deltamarker);

ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLinesEnabled(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLinesHorizontalLine(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  horizontalLine);

ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLineRelativeHorizontal(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean enabled,
                            ViReal64  horizontalLine);

ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLineAbsoluteVertical(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean enabled,
                            ViReal64  verticalLine);

ViStatus _VI_FUNC rsfpl_ConfigureVSADisplayLineRelativeVertical(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean enabled,
                            ViReal64  verticalLine);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMACCLimitDefault(ViSession instrSession,
                                                     ViInt32   window);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMACCLimitEnabled(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigureVSAMACCLimitCheck(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   measurementType,
                                                   ViInt32   limitType,
                                                   ViBoolean enabled,
                                                   ViReal64  value);

ViStatus _VI_FUNC rsfpl_QueryVSAMACCLimitCheckResults(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   measurementType,
                                                      ViInt32   limitType,
                                                      ViInt32*  checkResults);

ViStatus _VI_FUNC rsfpl_SetVSAStatusRegister(ViSession instrSession,
                                             ViInt32   questionableRegister,
                                             ViInt32   enable,
                                             ViInt32   pTransition,
                                             ViInt32   nTransition,
                                             ViInt32   channel);

ViStatus _VI_FUNC rsfpl_GetVSAStatusRegister(ViSession instrSession,
                                             ViInt32   registerPart,
                                             ViInt32   statusRegisterQuery,
                                             ViInt32   channel,
                                             ViInt32*  registerValue);

ViStatus _VI_FUNC rsfpl_PhaseNoiseMode(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_PhaseNoiseInitiateReferenceMeasurement(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_PhaseNoiseFinishHalfDecade(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseFrontEnd(ViSession instrSession,
                                                    ViReal64  nominalFrequency,
                                                    ViReal64  nominalLevel);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseFrontEndCoupling(ViSession instrSession,
                                                            ViInt32   coupling);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseVerifyFrequency(ViSession instrSession,
                                                           ViBoolean verifyFrequencyEnabled,
                                                           ViReal64  relativeTolerance,
                                                           ViReal64  absoluteTolerance);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseVerifyLevel(ViSession instrSession,
                                                       ViBoolean verifyLevelEnabled,
                                                       ViReal64  tolerance);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseOnVerifyFailed(ViSession instrSession,
                                                          ViInt32   onVerifyFailed);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTracking(ViSession instrSession,
                                                    ViBoolean frequencyTrackingEnabled,
                                                    ViBoolean levelTrackingEnabled);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseIQProcessing(ViSession instrSession,
                                                        ViBoolean AMRejectEnabled,
                                                        ViReal64  maxFreqDrift,
                                                        ViBoolean digitalPLLEnabled,
                                                        ViBoolean decimationEnabled);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSweepForwardEnabled(
                            ViSession instrSession,
                            ViBoolean sweepForward);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSelectPreset(ViSession instrSession,
                                                        ViInt32   preset);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseGlobals(ViSession instrSession,
                                                   ViReal64  globalRBW,
                                                   ViInt32   averageCount,
                                                   ViBoolean multiplierEnabled,
                                                   ViInt32   multiplier,
                                                   ViInt32   globalSweepMode);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseHalfDecade(ViSession instrSession,
                                                      ViInt32   halfDecade,
                                                      ViReal64  RBW,
                                                      ViInt32   sweepMode,
                                                      ViInt32   averageCount,
                                                      ViInt32   windowType);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceXAxisScope(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   scope,
                                                           ViReal64  start,
                                                           ViReal64  stop,
                                                           ViInt32   halfDecade);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceYAxisScaling(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   subwindow,
                                                             ViInt32   trace,
                                                             ViInt32   yAxisScaling,
                                                             ViReal64  top,
                                                             ViReal64  range,
                                                             ViReal64  bottom);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceYAxisAutoScaleEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_PhaseNoiseTraceYAxisAutoScaleOnce(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   subwindow,
                                                          ViInt32   trace);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseTraceOptions(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   trace,
                                                        ViReal64  offset,
                                                        ViBoolean spurRemovalEnabled,
                                                        ViReal64  spurThreshold,
                                                        ViInt32   smoothingMethod);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseLimitLine(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   type,
                                                     ViInt32   trace,
                                                     ViReal64  noiseFloor);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseLimitLineSelectTraces(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViInt32   traces[]);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseLimitLineSegment(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   limitSegment,
                                                            ViReal64  cornerFrequency,
                                                            ViReal64  leftSlope);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseLimitLineCheckResult(ViSession  instrSession,
                                                            ViInt32    window,
                                                            ViBoolean* limitCheckResult);

ViStatus _VI_FUNC rsfpl_PhaseNoiseLimitLineCopytoUserLimitLine(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   limitLine);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseResidualNoiseRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean rangeEnabled,
                            ViReal64  rangeStart,
                            ViReal64  rangeEnd);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseResidualNoiseUserRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   userRange,
                            ViInt32   userRangeTrace,
                            ViReal64  userRangeStart,
                            ViReal64  userRangeEnd);

ViStatus _VI_FUNC rsfpl_PhaseNoiseResidualNoiseDisableUserRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   userRange);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseIntegratedPhaseNoise(
                            ViSession instrSession,
                            ViReal64* integratedPhaseNoise);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseResidualFM(
                            ViSession instrSession,
                            ViReal64* residualFM);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseResidualRMSJitter(
                            ViSession instrSession,
                            ViReal64* residualRMSJitter);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseResidualPM(
                            ViSession instrSession,
                            ViReal64* residualPM);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeIntegratedPhaseNoise(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* integratedPhaseNoise);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeResidualFM(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* residualFM);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeResidualRMSJitter(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* residualRMSJitter);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseResidualNoiseUserRangeResidualPM(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64* residualPM);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpotNoiseOnAllDecadeEdgesEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean onDecadeEdges);

ViStatus _VI_FUNC rsfpl_PhaseNoiseSpotNoiseDisableAllMarkers(ViSession instrSession,
                                                             ViInt32   window);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpotNoiseDecadeEdgeMarkerEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   SNMarker,
                            ViBoolean edgeMarkerEnabled);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpotNoiseCustomMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   SNMarker,
                            ViBoolean markerEnabled,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpotNoiseDecadeEdgeResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   buffersSize,
                            ViReal64  frequencies[],
                            ViReal64  values[]);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpotNoiseMarkerValue(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   SNMarker,
                                                            ViReal64* markerValue);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpurRemovalEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsfpl_ConfigurePhaseNoiseSpurThreshold(ViSession instrSession,
                                                         ViReal64  threshold);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpurListResults(ViSession instrSession,
                                                       ViInt32   trace,
                                                       ViInt32   buffersSize,
                                                       ViReal64  frequencies[],
                                                       ViReal64  levels[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpurListDiscreteJitter(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViReal64* discreteJitter);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSpurListRandomJitter(ViSession instrSession,
                                                            ViInt32   trace,
                                                            ViReal64* randomJitter);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseTraceResults(ViSession instrSession,
                                                    ViInt32   trace,
                                                    ViReal64* frequency,
                                                    ViReal64* level);

ViStatus _VI_FUNC rsfpl_QueryPhaseNoiseSweepResultList(ViSession instrSession,
                                                       ViInt32   buffersSize,
                                                       ViReal64  start[],
                                                       ViReal64  stop[],
                                                       ViReal64  samplingRate[],
                                                       ViReal64  averages[],
                                                       ViReal64  frequencyDrift[],
                                                       ViReal64  levelDrift[],
                                                       ViReal64  maxDrift[]);

ViStatus _VI_FUNC rsfpl_reset(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking);

ViStatus _VI_FUNC rsfpl_self_test(ViSession instrSession,
                                  ViInt16*  selfTestResult,
                                  ViChar    selfTestMessage[]);

ViStatus _VI_FUNC rsfpl_revision_query(ViSession instrSession,
                                       ViChar    instrumentDriverRevision[],
                                       ViChar    firmwareRevision[]);

ViStatus _VI_FUNC rsfpl_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[]);

ViStatus _VI_FUNC rsfpl_ErrorList(ViSession instrSession,
                                  ViInt32   bufferSize,
                                  ViChar    errors[],
                                  ViInt32*  returnCount);

ViStatus _VI_FUNC rsfpl_ErrorListSpecificType(ViSession instrSession,
                                              ViInt32   messageType,
                                              ViString  channelName,
                                              ViInt32   bufferSize,
                                              ViChar    messages[],
                                              ViInt32*  returnCount);

ViStatus _VI_FUNC rsfpl_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[]);

ViStatus _VI_FUNC rsfpl_ClearInstrumentErrors(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_ClearRemoteErrors(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryRemoteErrors(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    errorTable[],
                                          ViInt32*  returnCount);

ViStatus _VI_FUNC rsfpl_QuerySystemBIOSInfo(ViSession instrSession,
                                            ViUInt32  arraySize,
                                            ViChar    systemBIOSInfo[],
                                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_QuerySystemVersionInfo(ViSession instrSession,
                                               ViUInt32  arraySize,
                                               ViChar    systemVersionInfo[],
                                               ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_QuerySystemHWInfo(ViSession instrSession,
                                          ViUInt32  arraySize,
                                          ViChar    systemHWInfo[],
                                          ViUInt32* returnedValues);

ViStatus _VI_FUNC rsfpl_QuerySystemBatteryLevel(ViSession instrSession,
                                                ViReal64* systemBatteryLevel);

ViStatus _VI_FUNC rsfpl_SystemReboot(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SystemShutdown(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SetTimeout(ViSession instrSession,
                                   ViUInt32  OPCTimeout);

ViStatus _VI_FUNC rsfpl_GetTimeout(ViSession instrSession,
                                   ViUInt32* timeout);

ViStatus _VI_FUNC rsfpl_ClearStatus(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_IDQueryResponse(ViSession instrSession,
                                        ViUInt32  bufferSize,
                                        ViChar    IDQueryResponse[]);

ViStatus _VI_FUNC rsfpl_ProcessAllPreviousCommands(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_QueryOPC(ViSession instrSession,
                                 ViInt32*  OPC);

ViStatus _VI_FUNC rsfpl_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery);

ViStatus _VI_FUNC rsfpl_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking);

ViStatus _VI_FUNC rsfpl_GetError(ViSession instrSession,
                                 ViStatus* code,
                                 ViUInt32  bufferSize,
                                 ViChar    description[]);

ViStatus _VI_FUNC rsfpl_ClearError(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SetIOTimeout(ViSession instrSession,
                                     ViUInt32  VISATimeout);

ViStatus _VI_FUNC rsfpl_GetIOTimeout(ViSession instrSession,
                                     ViUInt32* VISATimeout);

ViStatus _VI_FUNC rsfpl_ReadToFileFromInstrument(ViSession instrSession,
                                                 ViString  source,
                                                 ViString  destination);

ViStatus _VI_FUNC rsfpl_WriteFromFileToInstrument(ViSession instrSession,
                                                  ViString  source,
                                                  ViString  destination);

ViStatus _VI_FUNC rsfpl_WriteCommand(ViSession instrSession,
                                     ViString  writeBuffer);

ViStatus _VI_FUNC rsfpl_QueryViBoolean(ViSession  instrSession,
                                       ViString   command,
                                       ViBoolean* value);

ViStatus _VI_FUNC rsfpl_QueryViInt32(ViSession instrSession,
                                     ViString  command,
                                     ViInt32*  value);

ViStatus _VI_FUNC rsfpl_QueryViReal64(ViSession instrSession,
                                      ViString  command,
                                      ViReal64* value);

ViStatus _VI_FUNC rsfpl_QueryViString(ViSession instrSession,
                                      ViString  command,
                                      ViUInt32  bufferSize,
                                      ViChar    value[],
                                      ViPUInt32 returnCount);

ViStatus _VI_FUNC rsfpl_WriteCommandWithOPCSync(ViSession instrSession,
                                                ViString  writeBuffer,
                                                ViUInt32  timeout);

ViStatus _VI_FUNC rsfpl_QueryWithOPCSync(ViSession instrSession,
                                         ViString  writeBuffer,
                                         ViUInt32  timeout,
                                         ViUInt32  bufferSize,
                                         ViChar    readBuffer[],
                                         ViUInt32* numBytesRead);

ViStatus _VI_FUNC rsfpl_SetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32   attributeValue);

ViStatus _VI_FUNC rsfpl_SetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64   attributeValue);

ViStatus _VI_FUNC rsfpl_SetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64  attributeValue);

ViStatus _VI_FUNC rsfpl_SetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViString  attributeValue);

ViStatus _VI_FUNC rsfpl_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViBoolean attributeValue);

ViStatus _VI_FUNC rsfpl_GetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32*  attributeValue);

ViStatus _VI_FUNC rsfpl_GetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64*  attributeValue);

ViStatus _VI_FUNC rsfpl_GetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64* attributeValue);

ViStatus _VI_FUNC rsfpl_GetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt32   bufferSize,
                                             ViChar    attributeValue[]);

ViStatus _VI_FUNC rsfpl_GetAttributeViBoolean(ViSession  instrSession,
                                              ViString   repeatedCapabilityName,
                                              ViAttr     attributeId,
                                              ViBoolean* attributeValue);

ViStatus _VI_FUNC rsfpl_GetAttributeRepCapName(ViSession instrSession,
                                               ViAttr    attributeId,
                                               ViString  repCapNameId,
                                               ViInt32   index,
                                               ViInt32   bufferSize,
                                               ViChar    repCapName[]);

ViStatus _VI_FUNC rsfpl_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr    attributeId,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityIds[]);

ViStatus _VI_FUNC rsfpl_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[]);

ViStatus _VI_FUNC rsfpl_PresetStatusRegister(ViSession instrSession);

ViStatus _VI_FUNC rsfpl_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   questionableRegister,
                                          ViUInt16  enable,
                                          ViUInt16  PTransition,
                                          ViUInt16  NTransition);

ViStatus _VI_FUNC rsfpl_GetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   questionableRegister,
                                          ViInt32   registerPart,
                                          ViUInt16* registerValue);

ViStatus _VI_FUNC rsfpl_close(ViSession instrSession);

    /****************************************************************************
     *------------------------ Error And Completion Codes ----------------------*
     ****************************************************************************/

#define RSFPL_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSFPL_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSFPL_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSFPL_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSFPL_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSFPL_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSFPL_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSFPL_ERRMSG_INCORRECT_STATE     "Instrument in incorrect state"
#define RSFPL_ERRMSG_INCORRECT_RESPONSE      "Response incorrect; report problem to R&S support!"
#define RSFPL_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSFPL_ERROR_CODES_AND_MSGS \
    {RSFPL_WARN_MEASURE_UNCALIBRATED,  RSFPL_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSFPL_WARN_OVER_RANGE,            RSFPL_WARNMSG_OVER_RANGE},\
    {RSFPL_ERROR_INCORRECT_STATE ,   RSFPL_ERRMSG_INCORRECT_STATE },\
    {RSFPL_ERROR_INCORRECT_RESPONSE,     RSFPL_ERRMSG_INCORRECT_RESPONSE},\
    {RSFPL_ERROR_MAX_TIME_EXCEEDED,    RSFPL_ERRMSG_MAX_TIME_EXCEEDED}

     /****************************************************************************
      *---------------------------- End Include File ----------------------------*
      ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSFPL_HEADER */

